var single = new Vue({
    el: "#ulisting-builder",
    data: {
        layouts: [],
        layout_selected: {
            id: 0,
            layout_name: "",
        },
        builder_data: [],
        elements: [],
        sections: [],
        config: [],
        donor: [],

        image: {
            file: null
        },
        files: null,
        file: null,
        noFile: 'No file chosen',

        load: false,
        form_load: false,
        layouts_load: false,
        listing_type_id: 0,
        create_panel: false,
        prev_layout: null,
    },
    created() {
        var vm = this;
        vm.show_panel = true;

        if (typeof stm_listing_single_layout_builder_data == "undefined")
            return;

        if (typeof stm_listing_single_layout_builder_data.listing_type_id != "undefined")
            vm.listing_type_id = stm_listing_single_layout_builder_data.listing_type_id

        setTimeout(function () {
            vm.getLayouts();
            vm.getSections();
        }, 100)

    },
    methods: {
        createNewLayout: function () {
            this.sections = [];
            this.layout_selected = {
                id: 0,
                layout_name: "",
            };
            this.create_panel = true;

        },

        applyImporter: function() {
            let vm = this;
            vm.file = document.querySelector('#ulisting-file');
            vm.file.click();
        },

        loadImage: function () {
            let vm = this;

            if(vm.file.value)
                vm.noFile = vm.file.value.match(/[\/\\]([\w\d\s\.\-\(\)]+)$/)[1];
            else
                vm.noFile = 'No file chosen';

            let fileToUpload = vm.$refs['image-file'].files[0];

            if (fileToUpload) {
                vm.files = fileToUpload;
            }
        },

        importNewLayout: function(){
            let vm = this;
            if(vm.files){
                let formData = new FormData();
                let _id = vm.generateId();
                formData.append('type', 'single');
                formData.append('file', vm.files);
                formData.append('listing_type_id', vm.listing_type_id);
                formData.append('id', 'ulisting_single_page_layout_' + _id);
                formData.append('nonce', ulistingAjaxNonce);

                vm.$set(vm.image, 'message', '');
                vm.$http.post("ulisting-builder/listing-single-layout/new-layout", formData).then(function (response) {
                    if(response.body.success) {
                        vm.files = null;
                        vm.noFile = 'No file chosen';
                        vm.$set(vm.image, 'file', '');
                        vm.getLayouts();
                    }
                });
            }
        },

        getSections: function () {
            var vm = this;
            vm.load = true;
            this.$http.post("ulisting-builder/listing-single-page/get_data", {
                'listing_type_id': vm.listing_type_id,
                'nonce': ulistingAjaxNonce
            }).then(function (response) {
                if (response.body.success) {
                    vm.sections = response.body.data.sections;
                    vm.config = response.body.data.config;
                    vm.elements = response.body.data.elements;
                    vm.donor = response.body.data.donor;
                    if (response.body.data.layout) {
                        vm.layout_selected = response.body.data.layout;
                    }
                }
                vm.load = false;
            }).catch(error => {
                setTimeout(function () {
                    vm.getSections();
                }, 1000)
            });
        },
        getLayouts: function () {
            var vm = this;
            vm.layouts_load = true;
            this.$http.get("ulisting-builder/listing-single-page/layout-list?listing_type_id=" + vm.listing_type_id).then(function (response) {
                if (response.body.success) {
                    vm.layouts = response.body.layouts
                }
                vm.layouts_load = false;
            }).catch(error => {
                setTimeout(function () {
                    vm.getLayouts();
                }, 1000)
            });
        },
        getLayout: function (layout) {
            var vm = this;
            vm.load = true;
            vm.layout_selected = {
                id: 0,
                layout_name: "",
            }
            vm.create_panel = false;
            this.$http.post("ulisting-builder/listing-single-page/get-layout", {
                layout_id: layout.id,
                listing_type_id: vm.listing_type_id,
                nonce: ulistingAjaxNonce
            }).then(function (response) {
                if (response.body.success) {
                    vm.sections = response.body.data.section;
                }
                vm.layout_selected = layout;
                vm.load = false;
            });
        },
        create: function () {
            var vm = this;
            vm.form_load = true;

            vm.layout = {
                id: "ulisting_single_page_layout_" + vm.generateId(),
                name: vm.layout_selected.name,
                sections: vm.sections,
                listing_type_id: vm.listing_type_id,
                nonce: ulistingAjaxNonce
            };
            this.$http.post("ulisting-builder/listing-single-page/save_layout", vm.layout).then(function (response) {
                vm.create_panel = false;
                vm.form_load = false;
                if (response.body.success) {
                    vm.layout_selected.name = response.body.data.name;
                    vm.layout_selected.id = response.body.data.id;
                    if (typeof vm.layouts == "undefined")
                        vm.layouts = [];
                    vm.layouts.push({id: vm.layout_selected.id, name: vm.layout_selected.name});
                    vm.getLayout(vm.layouts[vm.layouts.length - 1])
                }
            });
        },
        update: function () {
            var vm = this;
            vm.form_load = true;
            vm.layout = {
                id: vm.layout_selected.id,
                name: vm.layout_selected.name,
                sections: vm.sections,
                listing_type_id: vm.listing_type_id,
                nonce: ulistingAjaxNonce
            };

            this.$http.post("ulisting-builder/listing-single-page/save_layout", vm.layout).then(function (response) {
                vm.form_load = false;
                if (response.body.success) {
                    vm.layouts.forEach(function (item) {
                        if (item.id == response.body.data.id)
                            item.name = response.body.data.name
                    })
                }
            });
        },
        detele: function () {
            if (!confirm("Are you sure to remove the Layout ?"))
                return;
            var vm = this;
            vm.form_load = true;
            this.$http.post("ulisting-builder/listing-single-page/delete-layout", {
                layout_id: vm.layout_selected.id,
                listing_type_id: vm.listing_type_id,
                nonce: ulistingAjaxNonce
            }).then(function (response) {
                vm.form_load = false;
                if (response.body.success) {
                    for (key in vm.layouts) {
                        if (vm.layouts[key].id == vm.layout_selected.id) {
                            vm.layouts.splice(key, 1);
                        }
                    }
                    if (vm.layouts.length)
                        vm.getLayout(vm.layouts[0])

                    else {
                        vm.sections = [];
                        vm.createNewLayout();
                    }
                }
                return false
            });
        },
        generateId: function () {
            let vm = this;
            if (!vm.layouts) return 0;

            let count = 0, result = null, keys = [], access = true;

            vm.layouts.forEach(function (value) {
                let index = parseInt(value.id.slice(value.id.lastIndexOf('_') + 1, value.id.length));
                keys.push(index);
            });

            keys = keys.sort(function (a, b) {
                return a - b
            });

            keys.forEach(function (value) {
                if (keys.indexOf(count) === -1 && access) {
                    access = false;
                    result = count;
                }
                count++;
            });

            if (result || result === 0) return result;
            return vm.layouts.length;
        },
    },

    computed: {
        generateUrl(){
            return "/wp-admin/admin-ajax.php?type=single&action=stm_export_current_layout&download=" + this.layout_selected.id + '&listing_type=' + this.listing_type_id;
        }
    },
});