let $ = jQuery;

Vue.component('stm-map-settings', {
    props: {
        map_type: {
            default: null,
        },
        api_keys: {
            default: [],
        },
        map_lists: {
            default: [],
        },
        open_by_hover: {
            default: 'yes'
        }
    },

    data: function () {
        return {
            type: null,
            keys: null,
            maps: null,
        }
    },

    mounted() {
        let vm = this, selector = '#map_settings', hover_selector = '#open_by_hover';

        if (typeof vm.map_type !== 'undefined')
            vm.type = vm.map_type;

        if (typeof vm.map_lists !== 'undefined')
            vm.maps = vm.map_lists;

        if (typeof vm.api_keys !== 'undefined' && typeof vm.api_keys === 'object')
            vm.keys = vm.api_keys;

        vm.checkPropTypes();

        setTimeout(function () {
            $(selector).on('change', function (event) {
                vm.changed(event.target.value);
            });

            $(hover_selector).on('change', function (event) {
                vm.hover_selector(event.target.value);
            });
        }, 1000);
    },

    methods: {
        changed: function (value) {
            this.type = value;
        },

        hover_selector: function(value){
            this.open_by_hover = value;
        },

        checkPropTypes: function () {
            let vm = this;
            let maps = Object.keys(vm.maps);

            if(typeof vm.api_keys === 'string'){
                vm.keys = {
                    osm: '',
                    mapbox: '',
                    google: ''
                };

                vm.keys[vm.type] = vm.api_keys;
            }

            maps.forEach(function (value) {
                if (!(vm.keys.hasOwnProperty(value))) {
                    vm.keys[value] = '';
                }
            });
        }
    },

    template: `
         <div>
            <div class="form-group">
                <div class="stm-row">
                    <div class="stm-col-3">
                        <label>Choose your map</label>
                    </div>
                    <div class="stm-col-5">
                        <select id="map_settings" name="StmListingsSettings[map][type]" class="_select2" v-model="type">
                            <option v-for="(value, index) in maps" v-bind:value="index">{{value}}</option>
                        </select>
                    </div>
                </div>
            </div>
            <div class="form-group" v-show="type && type !== 'osm' ">
                <div class="stm-row">
                    <div class="stm-col-3">
                        <label>{{ type === 'mapbox' ? 'Enter access token' : 'Enter api key' }}</label>
                    </div>
                    <div class="stm-col-5">
                        <input v-if="type === index" v-for="(val, index) in maps" class="form-control" v-model="keys[type]" type="text">
                    </div>
                 </div>
            </div>
            <div class="form-group">
                <div class="stm-row">
                    <div class="stm-col-3">
                        <label>Show info-window on map by hover</label>
                    </div>
                    <div class="stm-col-5">
                        <select name="StmListingsSettings[map][open_by_hover]" id="open_by_hover" v-model="open_by_hover" class="_select2">
                            <option value="yes">Yes</option>
                            <option value="no">No</option>
                        </select>
                    </div>
                </div>
            </div>
                        
            <input type="hidden" v-for="(val, index) in keys" v-bind:name="'StmListingsSettings[map][api_key]['+ index +']'" :value="val">
        </div>
    `,
});