new Vue({
    el: "#ulisting-inventory-builder",
    data: {
        show_panel: false,
        active_layout_id: null,
        layouts: [],
        builder_data: [],
        elements: [],
        sections: [],
        config: [],
        donor: [],
        load: false,
        form_load: false,
        layouts_load: false,
        listing_type_id: 0,
        create_panel: false,
        layout_selected: {
            id: 0,
            layout_name: "",
        },


        image: {
            file: null
        },
        files: null,
        file: null,
        noFile: 'No file chosen',

    },
    created() {
        if (typeof ulisting_inventory_builder_data != "undefined") {

            if (typeof ulisting_inventory_builder_data.active_layout_id != "undefined") {
                this.active_layout_id = ulisting_inventory_builder_data.active_layout_id;
            }
        }

        var vm = this;
        vm.show_panel = true;

        setTimeout(function () {
            vm.getData();
            vm.getLayouts();
        }, 100)
    },

    computed: {
        generateUrl(){
            return "/wp-admin/admin-ajax.php?type=inventory&action=stm_export_current_layout&download=" + this.layout_selected.id + '&listing_type=' + this.listing_type_id;
        }
    },

    methods: {
        getData: function () {
            var vm = this;
            vm.load = true;
            this.$http.get("ulisting-builder/listing-type-layout/get_data").then(function (response) {
                if (response.body.success) {
                    vm.sections = response.body.data.sections;
                    vm.config = response.body.data.config;
                    vm.elements = response.body.data.elements;
                    vm.donor = response.body.data.donor;
                }
                vm.load = false;
            }).catch(error => {
                setTimeout(function () {
                    vm.getData();
                }, 1000)
            });
        },


        applyImporter: function() {
            let vm = this;
            vm.file = document.querySelector('#ulisting-file');
            vm.file.click();
        },

        loadImage: function () {
            let vm = this;

            if(vm.file.value)
                vm.noFile = vm.file.value.match(/[\/\\]([\w\d\s\.\-\(\)]+)$/)[1];
            else
                vm.noFile = 'No file chosen';

            let fileToUpload = vm.$refs['image-file'].files[0];

            if (fileToUpload) {
                vm.files = fileToUpload;
            }
        },

        importNewLayout: function(){
            let vm = this;
            if(vm.files){
                let formData = new FormData();
                let _id = vm.generateId();
                formData.append('file', vm.files);
                formData.append('type', 'inventory');
                formData.append('id', 'ulisting_type_page_layout_' + _id);
                formData.append('nonce', ulistingAjaxNonce);

                vm.$set(vm.image, 'message', '');
                vm.$http.post("ulisting-builder/listing-single-layout/new-layout", formData).then(function (response) {
                    if(response.body.success) {
                        vm.files = null;
                        vm.noFile = 'No file chosen';
                        vm.$set(vm.image, 'file', '');
                        vm.getLayouts();
                    }
                });
            }
        },


        create: function () {
            var vm = this;
            vm.form_load = true;
            vm.layout = {
                id: "ulisting_type_page_layout_" + vm.generateId(),
                name: vm.layout_selected.name,
                sections: vm.sections,
                nonce: ulistingAjaxNonce
            };
            this.$http.post("ulisting-builder/listing-type-layout/save_layout", vm.layout).then(function (response) {
                vm.create_panel = false;
                vm.form_load = false;
                if (response.body.success) {
                    vm.layout_selected.name = response.body.data.name;
                    vm.layout_selected.id = response.body.data.id;
                    if (typeof vm.layouts == "undefined")
                        vm.layouts = [];
                    vm.layouts.push({id: vm.layout_selected.id, name: vm.layout_selected.name})
                    // vm.getLayout(vm.layouts[vm.layouts.length - 1])
                }
            });
        },
        getLayouts: function () {
            var vm = this;
            vm.layouts_load = true;
            var layout = null;
            this.$http.get("ulisting-builder/listing-type-layout/layout-list").then(function (response) {
                if (response.body.success) {
                    vm.layouts = response.body.layouts
                    if (vm.layouts.length) {

                        vm.layouts.forEach(function (item) {
                            if (vm.active_layout_id == item.id) {
                                layout = item;
                            }
                        })

                        if (!layout)
                            layout = vm.layouts[0];

                        vm.getLayout(layout)
                    }

                }
                vm.layouts_load = false;
            }).catch(error => {
                setTimeout(function () {
                    vm.getLayouts();
                }, 1000)
            });
        },
        getLayout: function (layout) {
            var vm = this;
            vm.load = true;
            vm.layout_selected = {
                id: 0,
                layout_name: "",
            };
            vm.create_panel = false;
            this.$http.post("ulisting-builder/listing-type-layout/get-layout", {
                listing_type_layout_id: layout.id,
                nonce: ulistingAjaxNonce
            }).then(function (response) {
                if (response.body.success) {
                    vm.sections = response.body.data.section;
                }
                vm.layout_selected = layout;
                vm.load = false;
            }).catch(function (e) {
                vm.getLayout();
            });
        },
        createNewLayout: function () {
            this.sections = [];
            this.layout_selected = {
                id: 0,
                layout_name: "",
            },
                this.create_panel = true;

        },
        update: function () {
            var vm = this;
            vm.form_load = true;
            vm.layout = {
                id: vm.layout_selected.id,
                name: vm.layout_selected.name,
                sections: vm.sections,
                nonce: ulistingAjaxNonce
            };

            this.$http.post("ulisting-builder/listing-type-layout/save_layout", vm.layout).then(function (response) {
                vm.form_load = false;
                if (response.body.success) {
                    vm.layouts.forEach(function (item) {
                        if (item.id == response.body.data.id)
                            item.name = response.body.data.name
                    })
                }
            });
        },
        detele: function () {

            if (!confirm("Are you sure to remove the Layout ?"))
                return;

            var vm = this;
            vm.form_load = true;
            this.$http.post("ulisting-builder/listing-type-layout/delete-layout", {
                listing_type_layout_id: vm.layout_selected.id,
                nonce: ulistingAjaxNonce
            }).then(function (response) {
                vm.form_load = false;
                if (response.body.success) {
                    for (key in vm.layouts) {
                        if (vm.layouts[key].id == vm.layout_selected.id) {
                            vm.layouts.splice(key, 1);
                        }
                    }
                    if (vm.layouts.length)
                        vm.getLayout(vm.layouts[0])
                    else {
                        vm.sections = [];
                        vm.createNewLayout();
                    }
                }
                return false
            });
        },
        generateId: function () {
            let vm = this;
            if (!vm.layouts) return 0;

            let count = 0, result = null, keys = [], access = true;

            vm.layouts.forEach(function (value) {
                let index = parseInt(value.id.slice(value.id.lastIndexOf('_') + 1, value.id.length));
                keys.push(index);
            });

            keys = keys.sort(function (a, b) {
                return a - b
            });

            keys.forEach(function (value) {
                if (keys.indexOf(count) === -1 && access) {
                    access = false;
                    result = count;
                }
                count++;
            });

            if (result || result === 0) return result;
            return vm.layouts.length;
        },
    }
});

new Vue({
    el: "#ulisting-listing-type-layout-page",
    data: {
        activeLayout: 0,
        layouts: []
    },
    created() {

        if (typeof stm_listing_setting_inventory_layout == "undefined")
            return;

        if (typeof stm_listing_setting_inventory_layout.layouts != "undefined")
            this.layouts = stm_listing_setting_inventory_layout.layouts;

        if (typeof stm_listing_setting_inventory_layout.activeLayout != "undefined")
            this.activeLayout = stm_listing_setting_inventory_layout.activeLayout;

    },
    methods: {
        setActive: function (layout) {
            this.activeLayout = layout.id
        }
    }
})