// Text field
Vue.component('text-field', {
	data: function () {
		return {
		}
	},
	mounted: function () {
		if(typeof this.data !== 'undefined' && (this.data.hasOwnProperty('short_code')) ){
			String.prototype.replaceAll = function (search, replace) {
				return this.split(search).join(replace);
			};

			this.data.short_code = this.data.short_code.replaceAll('u0022', '"');
		}
	},
	methods:{

	},
	props: {
		data: { default: "" },
		field:{default: ""}
	},
	watch:{
		data: function(val) {
			this.$emit('input', val)
		}
	},
	template:`
		<div class="form-group">
			<label>{{field.label}}</label>
			<input type="text" class="form-control" v-model="data[field.name]">
	    </div>
	`,
})

// Number field
Vue.component('number-field', {
	data: function () {
		return {
		}
	},
	mounted: function () {

	},
	methods:{

	},
	props: {
		data: { default: "" },
		field:{default: ""}
	},
	watch:{
		data: function(val) {
			this.$emit('input', val)
		}
	},
	template:`
		<div class="form-group">
			<label>{{field.label}}</label>
			<input type="number" class="form-control" v-model="data[field.name]">
	    </div>
	`,
})

// Textare
Vue.component('textarea-field', {
	data: function () {
		return {
		}
	},
	mounted: function () {

		if(typeof this.data !== 'undefined' && (this.data.hasOwnProperty('html')) ){
			String.prototype.replaceAll = function (search, replace) {
				return this.split(search).join(replace);
			};

			this.data.html = this.data.html.replaceAll('u0022', '"');
		}
	},
	methods:{

	},
	props: {
		id:{
			default:0
		},
		data: { default: "" },
		field:{default: ""}
	},
	watch:{
		data: function(val) {
			this.$emit('input', val)
		}
	},
	template:`
		<div class="form-group">
			<label>{{field.label}}</label>
			<textarea class="form-control" v-model="data[field.name]"></textarea>
		</div>
	`,
})

// Checkbox
Vue.component('checkbox-field', {
	data: function () {
		return {
		}
	},
	mounted: function () {

	},
	methods:{

	},
	props: {
		id:{
			default:0
		},
		data: { default: "" },
		field:{default: ""}
	},
	watch:{
		data: function(val) {
			this.$emit('input', val)
		}
	},
	template:`
		<div class="form-group">
			<label class="ulisting-checkbox">
				<input type="checkbox" class="form-control" :true-value="1" :false-value="0" v-model="data[field.name]" />
				<span></span>
				{{field.label}}
			</label>
		</div>
	`,
})

// Select
Vue.component('select-field', {
	data: function () {
		return {
		}
	},
	mounted: function () {
	},
	methods:{

	},
	props: {
		id:{
			default:0
		},
		data: { default: "" },
		field:{default: ""}
	},
	watch:{
		data: function(val) {
			this.$emit('input', val)
		}
	},
	template:`
		<div class="form-group">
			<label>{{field.label}}</label>
			<select class="form-control"  v-model="data[field.name]">
				<option v-for="(val, key) in field.items" v-bind:value="key">{{val}}</option>
			</select>
			
	    </div>
	`,
})

// Color
Vue.component('color-field', {
	components: {
		'sketch-picker': VueColor.Chrome
	},
	props: ['data','field','element'],
	data() {
		return {
			colors: {
				hex: '#000000',
			},
			colorValue: '',
			displayPicker: false,
		}
	},
	mounted() {
		if(this.data[this.field.name] || !this.field.hasOwnProperty('default'))
			this.colorValue = this.data[this.field.name];
		else if(this.field.hasOwnProperty('default'))
			this.colorValue = this.field.default;

	},
	methods: {
		setColor(color) {
			this.updateColors(color);
			this.colorValue = color;
		},
		updateColors(color) {
			if(color.slice(0, 1) == '#') {
				this.colors = {
					hex: color
				};
			}
			else if(color.slice(0, 4) == 'rgba') {
				var rgba = color.replace(/^rgba?\(|\s+|\)$/g,'').split(','),
					hex = '#' + ((1 << 24) + (parseInt(rgba[0]) << 16) + (parseInt(rgba[1]) << 8) + parseInt(rgba[2])).toString(16).slice(1);
				this.colors = {
					hex: hex,
					a: rgba[3],
				}
			}
		},
		showPicker() {
			document.addEventListener('click', this.documentClick);
			this.displayPicker = true;
		},
		hidePicker() {
			document.removeEventListener('click', this.documentClick);
			this.displayPicker = false;
		},
		togglePicker() {
			this.displayPicker ? this.hidePicker() : this.showPicker();
		},
		updateFromInput() {
			this.updateColors(this.colorValue);
		},
		updateFromPicker(color) {
			this.colors = color;
			if(color.rgba.a == 1) {
				this.colorValue = color.hex;
			}
			else {
				this.colorValue = 'rgba(' + color.rgba.r + ', ' + color.rgba.g + ', ' + color.rgba.b + ', ' + color.rgba.a + ')';
			}
		},
		documentClick(e) {
			var el = this.$refs.colorpicker,
				target = e.target;
			if(el !== target && !el.contains(target)) {
				this.hidePicker()
			}
		},
		clear: function() {
			this.colorValue = "";
			this.data[this.field.name] = "";
			this.data.__ob__.dep.notify()
		}
	},
	watch: {
		colorValue(val) {
			if(val) {
				this.updateColors(val);
				this.data[this.field.name] = val;
				this.data.__ob__.dep.notify()
			}
		}
	},
	template:`
		<div class="ulisting-color-picer m-b-15">
			<label>{{field.label}}</label>
			<div class="input-group color-picker" ref="colorpicker">
				<!--<input type="text" class="form-control" v-model="data[field.name]" @focus="showPicker()" @input="updateFromInput" />-->
				<span class="form-control color-picker-container" :style="'background-color: ' + colorValue" @click="togglePicker()"> </span>
				<sketch-picker :value="colors" @input="updateFromPicker" v-if="displayPicker" :key="field.name+'_'+element.id"></sketch-picker>
				
				<div class="input-group-append">
					<button type="button" class="btn btn-light" @click="clear()">
							<i class="fa fa-times-circle" aria-hidden="true"></i>
					</button>
				 </div>
			</div>
		</div>
	`,
});

// Blog
Vue.component('blog-field', {
	data: function () {
		return {
		}
	},
	mounted: function () {

	},
	methods:{

	},
	props: {
		id:{
			default:0
		},
		data: { default: "" },
		field:{default: ""}
	},
	watch:{
		data: function(val) {
			this.$emit('input', val)
		}
	},
	template:`
		<div class="form-group">
			<label>{{field.label}}</label>
			<div class="ulisting-blog-field-list p-l-10 p-r-10">
				<div class="stm-row">
					<div class="stm-col-6 p-3" v-for="(item, key) in field.items">
						<label>
							<input v-show="false" type="radio" v-model="data[field.name]" v-bind:value="key">
							<div class="ulisting-blog-field-item" 
									 v-bind:class="{'active': data[field.name] == key}" >
								  <img class="stm-max-width-100" v-bind:src="item.icon">
								  <span>{{item.name}}</span>
							</div>
						</label>
						
					</div>
				</div>		
			</div>
		</div>
	`,
})

// Margin
Vue.component('margin-field', {
	data: function () {
		return {
			active:"extra_large",
			icons:{
				extra_large:"fa fa-desktop",
				large:"fa fa-tablet transform-rotate-90",
				medium:"fa fa-tablet",
				small:"fa fa-mobile transform-rotate-90",
				extra_small:"fa fa-mobile",
			},
			margin:{
				extra_large:{
					top: "",
					right: "",
					bottom: "",
					left: "",
				},
				large:{
					top: "",
					right: "",
					bottom: "",
					left: "",
				},
				medium:{
					top: "",
					right: "",
					bottom: "",
					left: "",
				},
				small:{
					top: "",
					right: "",
					bottom: "",
					left: "",
				},
				extra_small:{
					top: "",
					right: "",
					bottom: "",
					left: "",
				},

			}
		}
	},
	mounted: function () {
		if(typeof this.data[this.field.name] != "undefined")
			this.margin = this.data[this.field.name];
	},
	methods:{

		set_active: function(active){
			this.active = active;
		},
		change: function(){
			this.data[this.field.name] = this.margin
			this.data.__ob__.dep.notify()
		}
	},
	props: {
		id:{
			default:0
		},
		data: { default: "" },
		field: { default: ""}
	},
	watch:{
		margin:{
			handler(val){
				this.change()
			},
			deep: true
		}
	},
	template:`
		<div class="form-group p-t-15">
			
			<div class="stm-row">
				<div class="stm-col-12">
					<label>{{field.label}}</label>
				</div>
				<div class="stm-col-12 ulisting-builder-device-width">
					<button v-for="(val, key) in margin" @click="set_active(key)" class="btn" v-bind:class="{'active': active==key}" type="button"><i  v-bind:class="icons[key]"></i></button>	
				</div>
			</div>
						
			<div v-for="(val, key) in margin" v-if="key == active">
				<div class="stm-row stm-no-gutters form-control-group">
					<div class="stm-col-3 text-center">
						<input type="number" class="form-control" v-model="val.top">
						<span class="label label-default">TOP</span>
					</div>
					<div class="stm-col-3 text-center">
						<input type="number" class="form-control" v-model="val.right">
						<span class="label label-default">RIGHT</span>
					</div>
					<div class="stm-col-3 text-center">
						<input type="number" class="form-control" v-model="val.bottom">
						<span class="label label-default">BOTTOM</span>
					</div>
					<div class="stm-col-3 text-center">
						<input type="number" class="form-control" v-model="val.left">
						<span class="label label-default">LEFT</span>
					</div>
				</div>
			</div>			
		</div> <br>
	`,
})

// Padding
Vue.component('padding-field', {
	data: function () {
		return {
			active:"extra_large",
			icons:{
				extra_large:"fa fa-desktop",
				large:"fa fa-tablet transform-rotate-90",
				medium:"fa fa-tablet",
				small:"fa fa-mobile transform-rotate-90",
				extra_small:"fa fa-mobile",
			},
			padding:{
				extra_large:{
					top: "",
					right: "",
					bottom: "",
					left: "",
				},
				large:{
					top: "",
					right: "",
					bottom: "",
					left: "",
				},
				medium:{
					top: "",
					right: "",
					bottom: "",
					left: "",
				},
				small:{
					top: "",
					right: "",
					bottom: "",
					left: "",
				},
				extra_small:{
					top: "",
					right: "",
					bottom: "",
					left: "",
				},

			}
		}
	},
	mounted: function () {
		if(typeof this.data[this.field.name] != "undefined")
			this.padding = this.data[this.field.name];
	},
	methods:{
		set_active: function(active){
			this.active = active;
		},
		change: function(){
			this.data[this.field.name] = this.padding;
			this.data.__ob__.dep.notify()
		}
	},
	props: {
		id:{
			default:0
		},
		data: { default: "" },
		field: { default: ""}
	},
	watch:{
		padding:{
			handler(val){
				this.change()
			},
			deep: true
		}
	},
	template:`
		<div class="form-group p-t-15">
			<div class="stm-row">
				<div class="stm-col-12">
					<label>{{field.label}}</label>
				</div>
				<div class="stm-col-12 ulisting-builder-device-width">
					<button v-for="(val, key) in padding" @click="set_active(key)" class="btn" v-bind:class="{'active': active==key}" type="button"><i v-bind:class="icons[key]"></i></button>	
				</div>
			</div>
						
			<div v-for="(val, key) in padding" v-if="key == active">
				<div class="stm-row stm-no-gutters form-control-group">
					<div class="stm-col-3 text-center">
						<input type="number" class="form-control" v-model="val.top">
						<span class="label label-default">TOP</span>
					</div>
					<div class="stm-col-3 text-center">
						<input type="number" class="form-control" v-model="val.right">
						<span class="label label-default">RIGHT</span>
					</div>
					<div class="stm-col-3 text-center">
						<input type="number" class="form-control" v-model="val.bottom">
						<span class="label label-default">BOTTOM</span>
					</div>
					<div class="stm-col-3 text-center">
						<input type="number" class="form-control" v-model="val.left">
						<span class="label label-default">LEFT</span>
					</div>
				</div>
			</div>			
		</div><br>
	`,
})

// Column size
Vue.component('column-size-field', {
	data: function () {
		return {
			active:"extra_large",
			icons:{
				extra_large:"fa fa-desktop",
				large:"fa fa-tablet transform-rotate-90",
				medium:"fa fa-tablet",
				small:"fa fa-mobile transform-rotate-90",
				extra_small:"fa fa-mobile",
			},
			size:{
				extra_large:0,
				large:0,
				medium:0,
				small:0,
				extra_small:0,
			}
		}
	},
	mounted: function () {
		if(typeof this.data[this.field.name] != "undefined")
			this.size = this.data[this.field.name];
	},
	methods:{
		set_active: function(active){
			this.active = active;
		},
		change: function(){
			this.data[this.field.name] =  JSON.parse(JSON.stringify(this.size));
			this.data.__ob__.dep.notify()
		}
	},
	props: {
		id:{
			default:0
		},
		element: { default: []},
		data: { default: []},
		field: { default: [] }
	},
	watch:{
		size:{
			handler(val){
				this.change()
			},
			deep: true
		}
	},
	activate: function() {

	},
	template:`
		<div class="form-group p-t-15">
			<div class="stm-row">
				<div class="stm-col-12">
					<label>{{field.label}}</label>
				</div>
				<div class="stm-col-12 ulisting-builder-device-width">
					<button v-for="(val, key) in size" @click="set_active(key)" class="btn" v-bind:class="{'active': active==key}" type="button"><i v-bind:class="icons[key]"></i></button>	
				</div>
			</div>
						
			<div v-for="(val, key) in size" v-if="key == active">
				<ulisting-select2 :options='build_array_for_select2(field.items)'
						  :key="size[key]+'_'+key"
						 v-model="size[key]"
						 :placeholder="'Select size'"
						 class="ulisting-select">
				</ulisting-select2>
			</div>		
		</div><br>
	`,
})

// Column
Vue.component('column-field', {
	methods:{
		isActive:function(id) {
			return this.data[this.field.name] == id;
		},
		setActive:function(id) {
			this.data[this.field.name] = id;
			this.columnChanged()
		},
		columnChanged() {
			var vm = this;

			if(typeof vm.element.columns == "undefined")
				return

			var column = vm.data[vm.field.name]
			if(vm.element.columns.length < column) {
				var i = vm.element.columns.length;
				while (i < column) {
					EventBus.$emit("ulisting_builder_add_column_"+vm.id, vm.element);
					i++;
				}
			}else{
				var i = vm.element.columns.length ;
				var elements = [];
				while (i > column) {
					elements = elements.concat(vm.element.columns[i - 1].elements);
					EventBus.$emit("ulisting_builder_remove_"+vm.id, vm.element.columns, i - 1);
					i--;
				}
				vm.element.columns[vm.element.columns.length - 1].elements = vm.element.columns[vm.element.columns.length - 1].elements.concat(elements)
			}
		}
	},
	props: {
		id:{
			default:0
		},
		element: { default: [] },
		data: { default: [] },
		field: { default: [] }
	},
	watch:{
		data: function(val){
			this.$emit('input', val)
		},
	},
	template:`
		<div class="panel-custom p-t-15 p-b-15">
			<label>{{field.label}}</label>
			<div class="stm-row ulisting-column-field-panel">
				<div class="stm-col-6">
					<div class="column" v-bind:class="{active: isActive(1)}" @click="setActive(1)">
						<span></span> 
					</div>
				</div>
				<div class="stm-col-6">
					<div class="column" v-bind:class="{active: isActive(2)}" @click="setActive(2)">
						<span></span> 
						<span></span> 
					</div>
				</div>
				<div class="stm-col-6">
					<div class="column" v-bind:class="{active: isActive(3)}" @click="setActive(3)">
						<span></span> 
						<span></span> 
						<span></span> 
					</div>
				</div>
				<div class="stm-col-6">
					<div class="column" v-bind:class="{active: isActive(4)}" @click="setActive(4)">
						<span></span> 
						<span></span> 
						<span></span> 
						<span></span> 
					</div>
				</div>
				<div class="stm-col-6">
					<div class="column" v-bind:class="{active: isActive(5)}" @click="setActive(5)">
						<span></span> 
						<span></span> 
						<span></span> 
						<span></span> 
						<span></span> 
					</div>
				</div>
				<div class="stm-col-6">
					<div class="column" v-bind:class="{active: isActive(6)}" @click="setActive(6)">
						<span></span> 
						<span></span> 
						<span></span> 
						<span></span> 
						<span></span> 
						<span></span> 
					</div>
				</div>
			</div>
		</div>
	`,
})

// Column
Vue.component('size-column-field', {
	methods:{
		isActive:function(id) {
			return this.data[this.field.name] == id;
		},
		setActive:function(id) {
			this.data[this.field.name] = id;
			this.columnChanged()
		},
		columnChanged() {
			var vm = this;

			if(typeof vm.element.columns == "undefined")
				return;

			var column = vm.data[vm.field.name]
			if(vm.element.columns.length < column) {
				var i = vm.element.columns.length;
				while (i < column) {
					EventBus.$emit("ulisting_builder_add_column_"+vm.id, vm.element);
					i++;
				}
			}else{
				var i = vm.element.columns.length ;
				var elements = [];
				while (i > column) {
					elements = elements.concat(vm.element.columns[i - 1].elements);
					EventBus.$emit("ulisting_builder_remove_"+vm.id, vm.element.columns, i - 1);
					i--;
				}
				vm.element.columns[vm.element.columns.length - 1].elements = vm.element.columns[vm.element.columns.length - 1].elements.concat(elements)
			}
		}
	},
	props: {
		id:{
			default:0
		},
		element: { default: [] },
		data: { default: [] },
		field: { default: [] }
	},
	watch:{
		data: function(val){
			this.$emit('input', val)
		},
	},
	template:`
		<div class="panel-custom p-t-15 p-b-15">
			<label>{{field.label}}</label>
			<div class="stm-row ulisting-column-field-panel">
				<div class="stm-col-6">
					<div class="column" v-bind:class="{active: isActive(1)}" @click="setActive(1)">
						<span></span> 
					</div>
				</div>
				<div class="stm-col-6">
					<div class="column" v-bind:class="{active: isActive(2)}" @click="setActive(2)">
						<span></span> 
						<span></span> 
					</div>
				</div>
				<div class="stm-col-6">
					<div class="column" v-bind:class="{active: isActive(3)}" @click="setActive(3)">
						<span></span> 
						<span></span> 
						<span></span> 
					</div>
				</div>
				<div class="stm-col-6">
					<div class="column" v-bind:class="{active: isActive(4)}" @click="setActive(4)">
						<span></span> 
						<span></span> 
						<span></span> 
						<span></span> 
					</div>
				</div>
				<div class="stm-col-6">
					<div class="column" v-bind:class="{active: isActive(6)}" @click="setActive(6)">
						<span></span> 
						<span></span> 
						<span></span> 
						<span></span> 
						<span></span> 
						<span></span> 
					</div>
				</div>
			</div>
		</div>
	`,
})

//  responsive  input
Vue.component('responsive-input-field', {
	data: function () {
		return {
			active:"extra_large",
			icons:{
				extra_large:"fa fa-desktop",
				large:"fa fa-tablet transform-rotate-90",
				medium:"fa fa-tablet",
				small:"fa fa-mobile transform-rotate-90",
				extra_small:"fa fa-mobile",
			},
			size:{
				extra_large:"",
				large:"",
				medium:"",
				small:"",
				extra_small:"",
			}
		}
	},
	mounted: function () {
		if(typeof this.data[this.field.name] != "undefined")
			this.size = this.data[this.field.name];
	},
	methods:{
		set_active: function(active){
			this.active = active;
		},
		change: function(){
			this.data[this.field.name] =  JSON.parse(JSON.stringify(this.size));
			this.data['values'] = this.values;
			this.data.__ob__.dep.notify()
		}
	},
	props: {
		id:{
			default:0
		},
		element: { default: []},
		data: { default: []},
		field: { default: [] },
		input_type: { default: "text" },
	},
	watch:{
		size:{
			handler(val){
				this.change()
			},
			deep: true
		}
	},
	activate: function() {

	},
	template:`
		<div class="form-group p-t-15">
			<div class="stm-row">
				<div class="stm-col-12">
					<label>{{field.label}}</label>
				</div>
				<div class="stm-col-12 ulisting-builder-device-width">
					<button v-for="(val, key) in size" @click="set_active(key)" class="btn" v-bind:class="{'active': active==key}" type="button"><i v-bind:class="icons[key]"></i></button>	
				</div>
			</div>
						
			<div v-for="(val, key) in size" v-if="key == active">
				<input class="form-control" type="text"  v-model="size[key]">
			</div>		
		</div><br>
	`,
})

//  responsive  input
Vue.component('responsive-number-input-field', {
	data: function () {
		return {
			active:"extra_large",
			icons:{
				extra_large:"fa fa-desktop",
				large:"fa fa-tablet transform-rotate-90",
				medium:"fa fa-tablet",
				small:"fa fa-mobile transform-rotate-90",
				extra_small:"fa fa-mobile",
			},
			size:{
				extra_large:"",
				large:"",
				medium:"",
				small:"",
				extra_small:"",
			}
		}
	},
	mounted: function () {
		if(typeof this.data[this.field.name] != "undefined")
			this.size = this.data[this.field.name];
	},
	methods:{
		set_active: function(active){
			this.active = active;
		},
		change: function(){
			this.data[this.field.name] =  JSON.parse(JSON.stringify(this.size));
			this.data.__ob__.dep.notify()
		}
	},
	props: {
		id:{
			default:0
		},
		element: { default: []},
		data: { default: []},
		field: { default: [] },
		input_type: { default: "text" },
	},
	watch:{
		size:{
			handler(val){
				this.change()
			},
			deep: true
		}
	},
	activate: function() {

	},
	template:`
		<div class="form-group p-t-15">
			<div class="stm-row">
				<div class="stm-col-12">
					<label>{{field.label}}</label>
				</div>
				<div class="stm-col-12 ulisting-builder-device-width">
					<button v-for="(val, key) in size" @click="set_active(key)" class="btn" v-bind:class="{'active': active==key}" type="button"><i v-bind:class="icons[key]"></i></button>	
				</div>
			</div>
						
			<div v-for="(val, key) in size" v-if="key == active">
				<input class="form-control" type="number"  v-model="size[key]">
			</div>		
		</div><br>
	`,
})

// Column size
Vue.component('responsive-select-field', {
	data: function () {
		return {
			active:"extra_large",
			icons:{
				extra_large:"fa fa-desktop",
				large:"fa fa-tablet transform-rotate-90",
				medium:"fa fa-tablet",
				small:"fa fa-mobile transform-rotate-90",
				extra_small:"fa fa-mobile",
			},
			size:{
				extra_large:"none",
				large:"none",
				medium:"none",
				small:"none",
				extra_small:"none",
			}
		}
	},
	mounted: function () {
		if(typeof this.data[this.field.name] != "undefined")
			this.size = this.data[this.field.name];
	},
	methods:{
		set_active: function(active){
			this.active = active;
		},
		change: function(){
			this.data[this.field.name] =  JSON.parse(JSON.stringify(this.size));
			this.data.__ob__.dep.notify()
		}
	},
	props: {
		id: {
			default: 0
		},
		element: {default: []},
		data: {default: []},
		field: {default: []},
	},
	watch:{
		size:{
			handler(val){
				this.change()
			},
			deep: true
		}
	},
	activate: function() {

	},
	template:`
		<div class="form-group p-t-15">
			<div class="stm-row">
				<div class="stm-col-12">
					<label>{{field.label}}</label>
				</div>
				<div class="stm-col-12 ulisting-builder-device-width">
					<button v-for="(val, key) in size" @click="set_active(key)" class="btn " v-bind:class="{'active': active==key}" type="button"><i v-bind:class="icons[key]"></i></button>	
				</div>
			</div>
						
			<div v-for="(val, key) in size" v-if="key == active">
				<ulisting-select2 :options='build_array_for_select2(field.items)'
						 :key="size[key]+'_'+key"
						 v-model="size[key]"
						 :placeholder="'Select size'"
						 class="ulisting-select">
				</ulisting-select2>
			</div>
		</div><br>
	`,
});


Vue.component('responsive-position-field', {
	data: function () {
		return {
			active:"extra_large",
			type: '',
			icons:{
				extra_large:"fa fa-desktop",
				large:"fa fa-tablet transform-rotate-90",
				medium:"fa fa-tablet",
				small:"fa fa-mobile transform-rotate-90",
				extra_small:"fa fa-mobile",
			},
			size:{
				extra_large:"none",
				large:"none",
				medium:"none",
				small:"none",
				extra_small:"none",
			},
			values: {
				extra_large:{
					top: "",
					right: "",
					bottom: "",
					left: "",
				},
				large:{
					top: "",
					right: "",
					bottom: "",
					left: "",
				},
				medium:{
					top: "",
					right: "",
					bottom: "",
					left: "",
				},
				small:{
					top: "",
					right: "",
					bottom: "",
					left: "",
				},
				extra_small:{
					top: "",
					right: "",
					bottom: "",
					left: "",
				},
			}
		}
	},
	mounted: function () {

		if(typeof this.data[this.field.name] != "undefined")
			this.size = this.data[this.field.name];
		if(typeof this.data.values != "undefined")
			this.values = this.data.values;
	},
	methods:{
		set_active: function(active){
			this.active = active;
		},
		change: function(){
			this.data['values'] = this.values;
			this.data[this.field.name] = JSON.parse(JSON.stringify(this.size));
			this.data.__ob__.dep.notify()
		},
	},
	props: {
		id:{
			default:0
		},
		element: { default: []},
		data: { default: []},
		field: { default: [] },
	},
	watch:{

		values:{
			handler(val){
				this.change();
			},
			deep: true
		},

		size:{
			handler(val){
				this.change()
			},
			deep: true
		},
	},
	activate: function() {

	},
	template:`
		<div class="form-group p-t-15">
			<div class="stm-row">
				<div class="stm-col-12">
					<label>{{field.label}}</label>
				</div>
				<div class="stm-col-12 ulisting-builder-device-width">
					<button v-for="(val, key) in size" @click="set_active(key)" class="btn " v-bind:class="{'active': active==key}" type="button"><i v-bind:class="icons[key]"></i></button>	
				</div>
			</div>

			<div v-for="(val, key) in size" v-if="key == active">
				<div class="form-group" style="margin: 0;">
					<select class="form-control" v-model="size[key]">
						<option v-for="option in build_array_for_select2(field.items)" v-bind:value="option['id']">
							{{option.text}}
						</option>
					</select>					
				</div>					
			</div>
			
			<div v-for="(val, key) in values" v-if="key == active">
				<div class="stm-row stm-no-gutters form-control-group">
					<div class="stm-col-3 text-center">
						<input type="number" class="form-control" v-model="val.top">
						<span class="label label-default">TOP</span>
					</div>
					<div class="stm-col-3 text-center">
						<input type="number" class="form-control" v-model="val.right">
						<span class="label label-default">RIGHT</span>
					</div>
					<div class="stm-col-3 text-center">
						<input type="number" class="form-control" v-model="val.bottom">
						<span class="label label-default">BOTTOM</span>
					</div>
					<div class="stm-col-3 text-center">
						<input type="number" class="form-control" v-model="val.left">
						<span class="label label-default">LEFT</span>
					</div>
				</div>
			</div>			
				
		</div><br>
	`,
});

// Column size
Vue.component('items-field', {
	data: function () {
		return {
			open_item: null
		}
	},
	mounted: function () {

	},
	methods:{
		add_item: function(){
			var vm = this;
			vm.data[vm.field.name].push({
				title: "Tab Title",
				is_active: false,
				elements: []
			})
		},
		remove_item: function(index){
			var vm = this;
			if(vm.data[vm.field.name].length > 1)
				vm.data[vm.field.name].splice(index, 1);
		},
		open: function(index){
			if(this.open_item != index){
				this.open_item = index;
				return
			}
			this.open_item = null;
		}
	},
	props: {
		id:{
			default:0
		},
		element: { default: []},
		data: { default: []},
		field: { default: [] }
	},
	watch:{
		size:{
			handler(val){
				this.change()
			},
			deep: true
		}
	},
	activate: function() {

	},
	template:`
		<div class="form-group p-t-15">
			<div class="stm-row">
				<div class="stm-col-12">
					<label>{{field.label}}</label>
				</div>
			</div>
				
			<div class="stm-row">
				<draggable v-model="data[field.name]" class="ulisting-draggable-panel stm-col" :options="{group:'items', handle:'.handle'}">
					<div class="ulisting-draggable-panel-items" v-for="(element, index) in data[field.name]" :key="index">
						<div class="ulisting-draggable-panel-items-top min">
							<div class="title">
								<span class="handle min">
									<i class="fa fa-arrows"></i>
								</span>
								<p @click="open(index)" class="cursor-pointer"> {{element.title}} </p>
							</div>
							<div class="action">
								<span class="btn" @click="remove_item(index)">
									<i class="fa fa-trash"></i>
								</span>
							</div>
						</div>
						<div class="ulisting-draggable-panel-items-inside" v-if="open_item == index">
							<div class="panel-custom p-t-15 p-b-15 p-l-15 p-r-15">
								<div class="stm-row">
									<div class="stm-col">
										<input class="form-control" type="text" v-model="element.title">
									</div>
								</div>
							</div>
							<div style="clear:both;"></div>
						</div>
					</div>
				</draggable>

			</div>	
			
			<div class="text-center p-t-15">
				<button @click="add_item" type="button" class="btn btn-success">Add item</button>
			</div>
			
			
		</div>
		<br>
	`,
});









