Vue.component('ulisting-builder', {
	data: function() {
		return {
			controls:null,
			basic_element_panel:true,
			general_element_panel:true,
			sidebar_id : 0,
			sidebar: null
		}
	},
	props: {
		id:{
			default: null
		},
		sections: {
			default: []
		},
		config: {
			default: []
		},
		elements: {
			default: []
		},
		donor: {
			default: []
		},
		load: {
			default: false
		},
		show_section: {
			default: 0
		}
	},
	created: function() {
		var vm = this;

		if(!vm.id)
			vm.id = vm.generateRandomId();

		EventBus.$off('ulisting_builder_remove_'+vm.id);
		EventBus.$off('ulisting_builder_remove_item_'+vm.id);
		EventBus.$off('ulisting_builder_add_column_'+vm.id);
		EventBus.$off('ulisting_builder_set_control_data_'+vm.id);

		EventBus.$on('ulisting_builder_set_control_data_'+vm.id, function(items, index) {
			vm.setСontrolData(items, index)
		});

		EventBus.$on('ulisting_builder_remove_'+vm.id, function(items, index) {
			vm.remove(items, index);
		});

		EventBus.$on('ulisting_builder_remove_item_'+vm.id, function(id) {
			if(vm.controls && vm.controls.element.id == id)
				vm.controls = null;
		});

		EventBus.$on('ulisting_builder_add_column_'+vm.id, function(row) {
			vm.addColumn(row);
		});
		this.scroll();
		this.resize();
	},
	methods:{
		scroll:function(){
			var vm = this;
			window.addEventListener('scroll', function(e) {
				vm.stickySidebar();
			});
		},
		resize:function(){
			var vm = this;
			window.addEventListener('resize', function(e) {
				vm.stickySidebar();
			});
		},
		stickySidebar:function() {
			var vm = this;

			 var containerSelector = document.getElementById('ulisting_builder_sidebar_'+vm.id);
			 var sidebar = document.getElementById('ulisting_builder_sidebar_inner_'+vm.id);
		  	 var rect = containerSelector.getBoundingClientRect()
		     if(rect.top < 50 && rect.width != 0){
				 sidebar.style.position = "fixed";
				 sidebar.style.width = (rect.width)+"px";
				 sidebar.style.top = "45px";
				 sidebar.style.left = (rect.left)+"px";
				 sidebar.style.right = (rect.right + rect.left)+"px";
			 }else{
				 sidebar.style.position = "initial";
			 }
		},
		addSection: function() {
			var new_section = this.object_clone(this.donor['section']);
			new_section.id = this.generateRandomId();
			this.addRow(new_section);
			this.sections.push(new_section);

		},
		addRow: function(section) {
			var new_row = this.object_clone(this.donor['row']);
			new_row.id = this.generateRandomId();
			this.addColumn(new_row);
			section.rows.push(new_row);
		},
		addColumn: function(row) {
			var new_col = this.object_clone(this.donor['col']);
			new_col.id = this.generateRandomId();
			row.columns.push(new_col);
		},
		remove: function(items, index) {
			EventBus.$emit("ulisting_builder_remove_item", items[index].id);
			items.splice(index, 1);
			this.controls = false;
		},
		duplicate: function() {

		},
		object_clone: function(data) {
			return JSON.parse(JSON.stringify(data))
		},
		setСontrolData(items, index) {
			var vm = this;
			vm.controls = null;
			vm.controls = {
				title : items[index].title,
				element : items[index],
				items : items,
				index : index,
				field_group : vm.config[items[index].field_group].field_group,
			}
		},
		generateRandomId : function() {
			return parseFloat(Math.round( Math.random() * 100) / 100 ).toFixed(4) * 1000+'_'+Date.now();
		},
		cloneOverride : function(original) {
			if(typeof original != "undefined") {
				var new_original    =  JSON.parse(JSON.stringify(original));
				    new_original.id = this.generateRandomId();

				    if(typeof new_original.columns != "undefined"){
						new_original.columns[0].id = this.generateRandomId();
					}

				return new_original;
			}
		}
	},
	watch:{
		sections: {
			handler(value){
				
			},
			deep: true
		},
		show_section: function(val) {
			var vm = this;
			vm.stickySidebar();
		}
	},
	components:{
		"controlPanel" : ulisting_control_panel
	},
	template:`
		<div class="stm-row">
			<div class="col-sm-9">
				<div v-if="load" class="panel-custom p-b-30 text-center">
					<div class="stm-spinner"> <div></div> <div></div> <div></div> <div></div> <div></div> </div>
				</div>	
				<div v-if="show_section && !load">
					<draggable class="ulisting-builder-draggabble ulisting-builder-draggabble-section border-0" :list="sections" :options="{group:{name:'section'}, handle:'.handle'}">
						<div class="ulisting-builder-draggabble-items" v-for="(section, sectionIndex) in sections" :key="sectionIndex">
							<div class="ulisting-builder-draggabble-items-btn-panel">
								<span  class="btn btn-default btn-xs" @click="setСontrolData(sections, sectionIndex)"><i class="fa fa-pencil"></i></span>
								<span class="btn btn-info btn-xs handle"><i class="fa fa-arrows"></i></span>
								<span class="btn btn-danger btn-xs" @click.prevent="remove(sections, sectionIndex)"><i class="fa fa-trash"></i></span>
							</div>
							<!-- row -->
							<draggable class="ulisting-builder-draggabble" :list="section.rows" :options="{group:{ name: 'rows'}, handle:'.handle'}" >
								<div class="ulisting-builder-draggabble-items" v-for="(row, rowIndex) in section.rows">
									<div class="ulisting-builder-draggabble-items-btn-panel">
										<span  class="btn btn-default btn-xs" @click="setСontrolData(section.rows, rowIndex)"><i class="fa fa-pencil"></i></span>
										<span class="btn btn-info btn-xs handle"><i class="fa fa-arrows"></i></span>
										<!--<span class="btn btn-danger btn-xs" @click.prevent="remove(section.rows, rowIndex)"><i class="fa fa-trash"></i></span>-->
									</div>
		
									<!-- column -->
									<draggable class="ulisting-builder-draggabble ulisting-builder-draggabble-row" :list="row.columns" :options="{group:{ name: 'column'}, handle:'.handle'}" >
		
										<div class="ulisting-builder-draggabble-items stm-col" v-for="(col, colIndex) in row.columns" 
																				 v-bind:class="'stm-col-xl-' + col.params.size.extra_large + 
																							   ' stm-col-gl-' + col.params.size.large + 
																							   ' stm-col-md-' + col.params.size.medium + 
																							   ' stm-col-sm-' + col.params.size.small + 
																							   ' stm-col-' + col.params.size.extra_small">
											
												<div class="ulisting-builder-draggabble-items-btn-panel">
													<span  class="btn btn-default btn-xs" @click="setСontrolData(row.columns, colIndex)"><i class="fa fa-pencil"></i></span>
													<span class="btn btn-info btn-xs handle"><i class="fa fa-arrows"></i></span>
												</div>
												<!-- element -->
												<draggable class="ulisting-builder-draggabble" 
														   :list="col.elements"
														   :options="{ group:{ name: 'elements', put:['elements','basic']  }, handle:'.handle'}" >
													<component v-for="(element, elementIndex) in col.elements"
															   v-bind:is="'ulisting-builder-module-'+element.module"
															   :id="id"
															   :data-type="element.type"
															   :element="element"
															   :items="col.elements"
															   :index="elementIndex" >
													</component>
												</draggable>
										</div>
		
									</draggable>
								</div>
							</draggable>
		
							<!-- <div class="ulisting-builder-add-section min" @click.prevent="addRow(section)">+</div> -->
							
							
							
						</div>
					</draggable>
					<div class="ulisting-builder-add-section" @click.prevent="addSection()">+</div>
				</div>
			</div>
			<div class="col-sm-3">
				<div class="ulisting_builder_sidebar" v-bind:id="'ulisting_builder_sidebar_'+id">
					<div class="ulisting_builder_sidebar_inner" v-bind:id="'ulisting_builder_sidebar_inner_'+id">
						<div class='card ulisting-panel' v-if="controls">
							<div class='card-header p-0'> 
								<span class="btn left" @click="controls = null"><i class="fa  fa-angle-left"></i></span>
								<span class="title">{{controls.title}}</span> 
							</div>
							<div class='card-body'>
							   <controlPanel :control="controls" :key="controls.element.id" :id="id"></controlPanel>
							</div>
						</div>
						<div v-if="!controls">
							<div class="ulisting-element-panel m-b-15">
								<div class="ulisting-element-panel-heading" @click="basic_element_panel = !basic_element_panel">
									Basic
									<span><i class="fa " v-bind:class="{'fa-angle-up':basic_element_panel, 'fa-angle-down':!basic_element_panel}"></i> </span>
								</div>
								<div class="ulisting-element-panel-body" v-if="basic_element_panel">
									<draggable :list="elements" :options="{group:{ name: 'elements', pull:'clone', put:false}, sort: false}" :clone="cloneOverride" class="ulisting-element-list clear-both">
										<div v-for="element in elements" v-if="element.group == 'basic'" class="ulisting-element-list-item" :data-type="element.type" :data-module="element.module" >
											<div class="content">
												<span class="title">{{element.title}}</span>
											</div>
										</div>
									</draggable>
								</div>
							</div>
							
							<div class="ulisting-element-panel">
								<div class="ulisting-element-panel-heading" @click="general_element_panel = !general_element_panel">
									General
									<span><i class="fa " v-bind:class="{'fa-angle-up':general_element_panel, 'fa-angle-down':!general_element_panel}"></i> </span>
								</div>
								<div class="ulisting-element-panel-body" v-if="general_element_panel">
									<draggable :list="elements" :options="{group:{ name: 'elements', pull:'clone', put:false}, sort: false}" :clone="cloneOverride" class="ulisting-element-list clear-both">
										<div v-for="element in elements" v-if="element.group == 'general'" class="ulisting-element-list-item" :data-type="element.type">
											<div class="content">
												<span class="title">{{element.title}}</span>
											</div>
										</div>
									</draggable>
								</div>
							</div>
					</div>
					</div>
				</div>
			</div>
		</div>
	`
});