Vue.component('stm-icon-picker', {
	data() {
		return {
			icons_load: false,
			icon_list_load: true,
			showModal: false,
			search: "",
			icon_id:0,
			hasAccess: true,
			icons:[],
			icon_groups:[]
		};
	},
	created() {
		this.get_icons_list();
	},
	methods: {
		get_icons_list: function(){
			var vm = this;
			vm.icon_list_load = true;
			this.$http.get("ulisting-icons/list").then(function(response){
				vm.icon_list_load = false;
				if(response.body.success){
					vm.icon_groups = response.body.data;
					vm.icon_id = Object.keys(vm.icon_groups)[0]
				}
			}).catch(error => {
				vm.icon_list_load = false;
				setTimeout(function() {
					vm.get_icons_list();
				}, 1000)
			});
		},
		get_icon: function(){
			var vm = this;
			vm.icons_load = true;
			this.$http.get("ulisting-icons/list?id="+vm.icon_id).then(function(response){
				vm.icons_load = false;
				if(response.body.success)
					vm.icons = response.body.data.icons;
			}).catch(error => {
				vm.icons_load = false;
				setTimeout(function() {
					vm.get_icon();
				}, 2000)
			});
		},
		setIcon: function(icon){
			this.icon = icon;
			this.showModal = false;
		},

		removeIcon: function () {
			this.icon = null;
			this.showModal = false;
			this.hasAccess = false;
			let vm = this;
			setTimeout(function () {
				vm.showModal = false;
				vm.hasAccess = true;
			}, 500)
		}
	},
	computed: {
		filteredIcons() {
			if(!this.icons.length)
				return [];
			return this.icons.filter(item => {
				var name = item.name.toLowerCase();
				return name.indexOf(this.search.toLowerCase()) > -1
			})
		}
	},
	props: {
		icon: {
			default: null
		},
	},
	watch:{
		icon_id: function(val) {
			this.get_icon();
		},
		icon: function(val) {
			this.$emit('icon-event', val)
		}
	},
	template: `
		<div class="stm-main-panel">
	      <div class="stm-icon-picker">
			
			<div v-on:click="showModal = true" class="stm-icon-picker-panel">
				<div @click="removeIcon"  :class="{'ulisting-remove-icon': icon }"></div>
				<strong v-if="!icon">Select Icon</strong>
			    <i v-bind:class="icon"></i>
			</div>
		  </div>

		  <stm-modal v-if="showModal && hasAccess" v-on:close="showModal = false" size="modal-lg">
				<div slot="content">
					<div class="modal-body">
						<div class="stm-row">
							<div class="stm-col-3">
								
								<div class="form-group">
									<label>Filter by name</label>
									<input type="text" class="form-control" placeholder="search" v-model="search">
								</div>
								
								<div v-if="icon_list_load" class="panel-custom p-t-30 text-center">
									<div class="stm-spinner"> <div></div> <div></div> <div></div> <div></div> <div></div> </div>
								</div>
								
								<div v-if="!icon_list_load" class="form-group">
									<label>Filter by icon pack</label>
									<select class="form-control" v-model="icon_id">
										<option v-for="(name, id) in icon_groups" v-bind:value="id">{{name}}</option>
									</select>
								</div>
								
								<div class="form-group">
									<h1 class="text-center">
										<i v-bind:class="icon"></i>
									</h1>
								</div>
								
								<div class="form-group">
									<button type="button" class="btn btn-light w-full" v-on:click="showModal = false">Close</button>
								</div>
								
							</div>
							<div class="stm-col-9 stm-icon-picker-list-panel">
								<div class="stm-scrol-panel-y">
									<div class="stm-icon-picker-list">
										<div v-if="icons_load" class="panel-custom p-t-30 text-center">
											<div class="stm-spinner"> <div></div> <div></div> <div></div> <div></div> <div></div> </div>
										</div>
										<div v-if="!icons_load"  class="stm-row">
											<div class="stm-col-2" v-for="icon in filteredIcons">
												<div class="stm-icon-picker-item" @click="setIcon(icon.class)">
													<i v-bind:class="icon.class"></i>
													<span>{{icon.name}}</span>
												</div>
											</div>
										</div>				
									</div>
								</div>
							</div>
						</div>			
					</div>
				</div>
		  </stm-modal>
		</div>
   `,
})



