<?php

namespace uListing\Admin\Classes;

use uListing\Classes\StmListingSettings;
use uListing\Classes\UlistingSearch;

class StmAdminMenu
{
    public function __construct()
    {
        add_action('admin_menu', [$this, 'settings_menu'], 20);
        add_action('admin_menu', [$this, 'status_menu'], 20);
        add_action('admin_menu', [$this, 'extensions_menu'], 20);
	    if(apply_filters('show_uListing_demo_import', true))
		    add_action('admin_menu', [$this, 'demo_import_menu'], 20);

	    add_filter('plugin_action_links_' . plugin_basename(ULISTING_PLUGIN_FILE), [$this, 'plugin_action_links']);

	    if ( ! uListing_subscription_active() && ! uListing_user_role_active() && ! uListing_social_login_active() )
		    add_action( 'admin_menu', [ $this, 'go_pro_menu' ], 25 );

    }

    /**
     * @return StmAdminMenu
     */
    public static function init()
    {
        return new StmAdminMenu();
    }

    public function settings_menu()
    {
        add_submenu_page(
            'edit.php?post_type=listing_type',
            esc_html__("Settings", "ulisting"),
            esc_html__("Settings", "ulisting"),
            'manage_options',
            'stm_listing_settings',
            array(StmListingSettings::class, 'render_settings')
        );
    }

    public function status_menu()
    {
        add_submenu_page(
            'edit.php?post_type=listing_type',
            __("uListing status", "ulisting"),
            __("Status", "ulisting"),
            'manage_options',
            'ulisting-status',
            array($this, 'render_status')
        );
    }

	public function go_pro_menu()
	{
		add_submenu_page(
			'edit.php?post_type=listing_type',
			__( "Upgrade", "ulisting" ),
			'<span style="color: #adff2f;"><span style="font-size: 14px;" class="dashicons dashicons-star-filled stm_go_pro_menu"></span>' . __( "Upgrade", "ulisting" ) . '</span>',
			'manage_options',
			'ulisting-upgrade',
			array( $this, 'render_go_pro' )
		);
	}

    public function extensions_menu()
    {
        add_submenu_page(
            'edit.php?post_type=listing_type',
            __("uListing extensions", "ulisting"),
            __("Extensions", "ulisting"),
            'manage_options',
            'ulisting-extensions',
            array($this, 'render_extensions')
        );

        /**
         * uListing exporter submenu
         */

        do_action('uListing_exporter_add_submenu');
    }

    public function demo_import_menu()
    {
        add_submenu_page(
            'edit.php?post_type=listing_type',
            __("uListing demo import", "ulisting"),
            __("Demo import", "ulisting"),
            'manage_options',
            'demo-import',
            array($this, 'render_demo_import')
        );
    }

	public function plugin_action_links($links)
	{
		$settings_link = sprintf( '<a href="%1$s">%2$s</a>', admin_url( 'edit.php?post_type=listing_type&page=stm_listing_settings' ), __("Settings", "ulisting") );

		array_unshift( $links, $settings_link );

		if (!ulisting_subscription_active() && !ulisting_user_role_active()) {
			$links['get_pro'] = sprintf( '<a href="%1$s" target="_blank" class="ulisting-get-pro">%2$s</a>', esc_url('https://stylemixthemes.com/wordpress-classified-plugin/?utm_source=admin&utm_medium=promo&utm_campaign=2020'), __("Upgrade to Pro Bundle", "ulisting") );
		}

		return $links;
	}

    /**
     * Render page status
     */
    public static function render_status()
    {
        ulisting_render_template(ULISTING_ADMIN_PATH . '/views/status/index.php', [], true);
    }

    /**
     * Render page extensions
     */
    public static function render_extensions()
    {
        ulisting_render_template(ULISTING_ADMIN_PATH . '/views/extensions/extensions.php', [], true);
    }

    /**
     * Render page extensions
     */
    public static function render_demo_import()
    {
        ulisting_render_template(ULISTING_ADMIN_PATH . '/views/demo/import.php', [], true);
    }

    /**
     * Render page go_pro
     */
    public static function render_go_pro()
    {
		ulisting_render_template(ULISTING_ADMIN_PATH . '/views/attribute_template/go_pro.php', [], true);
    }
}