<?php

namespace uListing\Admin\Classes;

class StmAdminNotice
{
	public static $features = [
		[
			'title' => "uListing Subscription",
			'subtitle' => "Monetize your website by applying your own daily, weekly, or monthly pricing plans for clients.",
			'image' => ULISTING_URL . '/assets/img/notices/ulisting-subscription.jpg',
		],
		[
			'title' => "uListing User Roles",
			'subtitle' => "Distribute roles among the users of your website and assign each profile a specific user type.",
			'image' => ULISTING_URL . '/assets/img/notices/ulisting-user-roles.jpg',
		],
		[
			'title' => "uListing Compare",
			'subtitle' => "Enable the comparison of similar type listings and provide a better experience for your users.",
			'image' => ULISTING_URL . '/assets/img/notices/ulisting-compare.jpg',
		],
		[
			'title' => "uListing Social Login",
			'subtitle' => "Make it easy for customers, enable social login for them, and the ability to use their social media accounts to register.",
			'image' => ULISTING_URL . '/assets/img/notices/ulisting-social-login.jpg',
		],
		[
			'title' => "uListing Wishlist",
			'subtitle' => "Guests liked some of the listings? Let them add their favorites to the wishlist and save for later.",
			'image' => ULISTING_URL . '/assets/img/notices/ulisting-wishlist.jpg',
		],
	];

    public function __construct()
    {
		add_action('admin_notices', [$this, 'render_review']);
		add_action('admin_notices', [$this, 'render_pro_notice']);
    }

    /**
     * @return StmAdminNotice
     */
    public static function init()
    {
        return new StmAdminNotice();
    }

	/**
	 * Render Review
	 */
	public function render_review()
	{
		$interval_value 	= 7;
		$now 				= date( 'Y-m-d h:i:s' );
		$installed_time 	= get_option('ulisting_installed');
		$canceled_rating 	= !empty(get_option('ulisting_canceled')) ? get_option('ulisting_canceled') : 'no';

		$now = new \DateTime( $now );
		if (!empty($installed_time)) {
			$installed_time = new \DateTime( $installed_time );
			$interval_value = round( ($now->format( 'U' ) - $installed_time->format('U')) / 86400 );
		}

		if ($interval_value >= 7 && $canceled_rating === 'no') {
			wp_enqueue_script('ulisting-notice-review', ULISTING_URL .'/assets/js/admin/stm-review-notice.js', array(), false, true);

			ulisting_render_template(ULISTING_ADMIN_PATH . '/views/notices/review.php', [], true);
		}
	}

	/**
	 * Render Pro Notice
	 */
	public function render_pro_notice()
	{
		if (!empty($_GET['post_type']) && in_array(sanitize_text_field($_GET['post_type']), array('listing_type', 'listing', 'stm_pricing_plans')) && !ulisting_subscription_active() && !ulisting_user_role_active()) {
            wp_enqueue_script('owl.carousel', ULISTING_URL .'/assets/js/admin/owl.carousel.min.js', array(), false, true);
            wp_enqueue_style('owl.carousel', ULISTING_URL . '/assets/css/admin/owl.carousel.min.css');
            ulisting_render_template(ULISTING_ADMIN_PATH . '/views/notices/pro_popup.php', ['features' => self::$features], true);
            ulisting_render_template(ULISTING_ADMIN_PATH . '/views/notices/pro_notice.php', [], true);
		}
	}

}