<?php

if(!isset($id) || empty($id))
	$id = 'id_'.time().'_'.rand(1111,9999);

$data['items'] = get_post_meta($listing->ID, $attribute->name, true);
$data['items'] = (!empty($data['items'])) ? json_decode($data['items'], true) : [];
?>
<label><?php echo esc_attr($attribute->title)?></label>

<div id="accordion_<?php echo esc_attr($id)?>" class="ulisting-main">
	<div class="p-t-30 p-b-30">
		<div v-for=" ( item, index) in items" class="card">
			<div class="card-header">
				<div class="row">
					<div class="col-12 col-sm-10">
						<input v-model="item.title" type="text" class="form-control" placeholder="<?php echo esc_attr('Title', 'ulisting')?>">
					</div>
					<div class="col-12 col-sm-2">
						<button @click="toggle_open(item)" type="button" class="btn btn-light float-right">
							<i v-if="!item.is_open" class="fa fa-angle-down"></i>
							<i v-if="item.is_open" class="fa fa-angle-up"></i>
						</button>
						<button @click="remove(items, index)" type="button" class="btn btn-danger float-right">
							<i class="fa fa-trash" aria-hidden="true"></i>
						</button>
					</div>
				</div>
			</div>
			<div v-if="item.is_open" v-bind:id="'accordion_item_panel_'+item.id" class="card-body">
				<div class="form-group">
					<label><?php _e("Options", "ulisting")?></label>
					<br>
					<div v-for=" ( param, _index) in item.options" class="row">
						<div  class="col-3 p-t-5">
							<input type="text" v-model="param.key" class="form-control" placeholder="<?php esc_attr_e("Key", "ulisting")?>">
						</div>
						<div  class="col-3 p-t-5">
							<input type="text" v-model="param.val" class="form-control" placeholder="<?php esc_attr_e("Value", "ulisting")?>">
						</div>
						<div  class="col-2 p-t-5">
							<button @click="remove(item.options, _index)" type="button" class="btn btn-danger float-left">
								<i class="fa fa-trash" aria-hidden="true"></i>
							</button>
						</div>
					</div>
				</div>
				<div class="p-t-15">
					<button type="button" @click="add_options(item)" class="btn btn-success"> + <?php _e("Add option", "ulisting")?> </button>
				</div>

				<hr>

				<div class="form-group">
					<label><?php _e("Shortcode", "ulisting")?></label>
					<br>
					<div class="row">
						<div  class="col-6 p-t-5">
							<input type="text" v-model="item.shortcode" class="form-control" placeholder="<?php esc_attr_e("[shortcode]", "ulisting")?>">
						</div>
					</div>
				</div>

				<hr>

				<textarea v-bind:id="'accordion_content_'+item.id"  v-model="item.content"></textarea>
			</div>
		</div>
	</div>
	<div class="text-right">
		<button type="button" @click="add_item" class="btn btn-success"> + <?php _e("Add", "ulisting")?> </button>
	</div>
	<br>
	<textarea class="hidden" name="ulisting_listing_meta_accordion[<?php echo esc_attr($attribute->name)?>]">
		{{get_content()}}
	</textarea>
</div>

<script>
	var data = json_parse("<?php echo ulisting_convert_content(json_encode($data))?>");
	new Vue({
		el:"#accordion_<?php echo esc_attr($id)?>",
		data:{
			items : []
		},
		created(){
			if( typeof data.items != "undefined") {
			    this.items = data.items;
				if( this.items.length ) {
					this.items.forEach(function(item) {
						item.is_open = false
					})
				}
			}
		},
		methods:{
			remove: function(items, index) {
				items.splice(index, 1);
			},
			add_options: function(item){
				if(typeof item.options == "undefined")
					item.options = [];
				item.options.push({
					id: this.generateRandomId(),
					key:"",
					val:"",
				})
			},
			add_item: function(){
				this.items.push({
					id: this.generateRandomId(),
					title: "",
					content: "",
					options: [],
					is_open: false
				})
			},
			get_content:function() {
				return JSON.stringify(this.items)
			},
			generateRandomId : function() {
				return parseFloat(Math.round( Math.random() * 100) / 100 ).toFixed(4) * 1000+'_'+Date.now();
			},
			toggle_open(item){
				item.is_open = !item.is_open

				if(item.is_open){

					setTimeout(function() {
						tinymce.remove('#accordion_content_'+item.id);
						tinymce.init({
							selector: '#accordion_content_'+item.id,
							plugins: 'image,wordpress,link',
							relative_urls : false,
							remove_script_host : false,
							convert_urls : true,
							toolbar: 'formatselect | bold italic strikethrough forecolor backcolor permanentpen formatpainter | link wp_code | alignleft aligncenter alignright alignjustify  | numlist bullist outdent indent | removeformat | addcomment',
							content_css: [
								'//fonts.googleapis.com/css?family=Lato:300,300i,400,400i',
								'//www.tiny.cloud/css/codepen.min.css'
							],
							height: 300,
							content_style: '.mce-annotation { background: #fff0b7; } .tc-active-annotation {background: #ffe168; color: black; }',
							init_instance_callback: function (editor) {
								editor.on('Change', function (e) {
									item.content = tinymce.get('accordion_content_'+item.id).getContent({format: 'html'})
								});
							}
						});

						tinymce.get('accordion_content_'+item.id).on('keyup',function(e){
							item.content = tinymce.get('accordion_content_'+item.id).getContent({format: 'html'})
						});
					}, 1)
				}
			}
		}
	})
</script>




