<?php
$address = current($listing->getListDataOptions('address'));
$latitude = current($listing->getListDataOptions('latitude'));
$longitude = current($listing->getListDataOptions('longitude'));
$postal_code = current($listing->getListDataOptions('postal_code'));

$type = uListing\Classes\StmListingSettings::get_current_map_type();
$is_google = $type === 'google';

$access_token_arr = uListing\Classes\StmListingSettings::get_current_map_api_key();
$access_token = isset($access_token_arr[$type]) ? $access_token_arr[$type] : '';
?>

<div id="google-map" class="panel-custom p-t-15 p-b-15">
    <div class="row">
        <div class="col-sm-5 col-xs-12">
            <?php if ($is_google): ?>
                <div class="autocomplete-panel">
                    <label><?php _e("Search", "ulisting") ?></label>
                    <gmap-autocomplete class="form-control" @place_changed="usePlace"></gmap-autocomplete>
                </div>
            <?php else: ?>
                <div class="autocomplete-panel">
                    <label><?php _e("Search", "ulisting") ?></label>
                    <input type="text" id="uListing_listing_search" class="form-control" placeholder="<?php echo esc_attr('Enter a location', 'ulisting')?>" autocomplete="off">
                </div>
            <?php endif; ?>

            <div class="form-group">
                <label><?php _e("Address", "ulisting") ?></label>
                <input class="form-control" type="text"
                       name="listing[options][address][<?php echo (!empty($address)) ? key($address) : '' ?>]"
                       v-model="address">
            </div>

            <div class="form-group">
                <label><?php _e("Postal code", "ulisting") ?></label>
                <input type="text"
                       name="listing[options][postal_code][<?php echo (!empty($postal_code)) ? key($postal_code) : '' ?>]"
                       v-model="postal_code" class="form-control">
            </div>

            <div class="stm-row">
                <div class="stm-col">
                    <div class="form-group">
                        <label><?php _e("latitude", "ulisting") ?></label>
                        <input type="text"
                               name="listing[options][latitude][<?php echo (!empty($latitude)) ? key($latitude) : '' ?>]"
                               v-model="latitude" class="form-control">
                    </div>
                </div>
                <div class="stm-col">
                    <div class="form-group">
                        <label><?php _e("Longitude", "ulisting") ?></label>
                        <input type="text"
                               name="listing[options][longitude][<?php echo (!empty($longitude)) ? key($longitude) : '' ?>]"
                               v-model="longitude" class="form-control">
                    </div>
                </div>
            </div>
        </div>
        <div class="col-sm-7 col-xs-12">
            <?php if ($is_google): ?>
                <google-map :center="center" @click="clickMap" :zoom="5" style="width: 100%; height: 350px">
                    <google-marker :position="marker" :clickable="true" :draggable="false"
                                   @click="center=marker"></google-marker>
                    <control-corner
                            :vSide="'top'"
                            :hSide="'almostright'">
                    </control-corner>
                </google-map>
            <?php else: ?>
                <div id="openstreetmap" style="width: 100%; height: 350px"></div>
            <?php endif; ?>
        </div>
    </div>
</div>
<div style="clear: both"></div>

<?php if ($is_google): ?>
    <script>
        Vue.use(VueGoogleMaps, {
            load: {
                key: '<?php echo esc_attr($access_token)?>',
                libraries: 'places'
            },
            installComponents: true
        });

        document.addEventListener('DOMContentLoaded', function () {
            Vue.component('google-map', VueGoogleMaps.Map);
            Vue.component('google-marker', VueGoogleMaps.Marker);

            var latitude = <?php echo (!empty($latitude) AND current($latitude)) ? current($latitude) : 0?>;
            var longitude = <?php echo (!empty($longitude) AND current($longitude)) ? current($longitude) : 0?>;
            var postal_code = <?php echo (!empty($postal_code) AND current($postal_code)) ? current($postal_code) : 0?>;

            new Vue({
                el: '#google-map',
                data: {
                    address: '<?php echo (!empty($address)) ? str_replace("'", "\'", current($address)) : ''?>',
                    latitude: latitude,
                    longitude: longitude,
                    postal_code: postal_code,
                    place: null,
                    center: {
                        lat: latitude,
                        lng: longitude
                    },
                    marker: {
                        lat: latitude,
                        lng: longitude
                    }
                },
                methods: {
                    clickMap(e) {
                        this.setPositionMrkers(e.latLng.lat(), e.latLng.lng())
                    },
                    usePlace(place) {
                        this.place = place;
                        if (this.place) {
                            this.setPositionMrkers(this.place.geometry.location.lat(), this.place.geometry.location.lng())
                            this.center.lat = this.place.geometry.location.lat();
                            this.center.lng = this.place.geometry.location.lng();
                            this.place = null;
                        }
                    },
                    setPositionMrkers(lat, lng) {
                        this.marker.lat = lat;
                        this.marker.lng = lng;
                        this.setLocation(lat, lng)
                    },
                    setLocation(lat, lng) {
                        var vm = this;
                        var LatLng = new google.maps.LatLng({lat: lat, lng: lng});
                        var geocoder = new google.maps.Geocoder;
                        geocoder.geocode({'location': LatLng}, function (results, status) {
                            if (status === 'OK') {
                                if (results[0]) {
                                    vm.address = results[0].formatted_address;
                                } else {
                                    window.alert('No results found');
                                }

                                for (var i = 0; i < results.length; i++) {
                                    for (var j = 0; j < results[i].address_components.length; j++) {
                                        for (var k = 0; k < results[i].address_components[j].types.length; k++) {
                                            if (results[i].address_components[j].types[k] == "postal_code") {
                                                vm.postal_code = results[i].address_components[j].short_name;
                                            }
                                        }
                                    }
                                }

                            } else {
                                window.alert('Geocoder failed due to: ' + status);
                            }
                        });
                        this.latitude = lat;
                        this.longitude = lng;
                    }

                }
            });
        });
    </script>
<?php else: ?>
    <script>
       window.onload = function () {
           let latitude = <?php echo (!empty($latitude) AND current($latitude)) ? current($latitude) : 0?>;
           let longitude = <?php echo (!empty($longitude) AND current($longitude)) ? current($longitude) : 0?>;
           let postal_code = <?php echo (!empty($postal_code) AND current($postal_code)) ? current($postal_code) : 0?>;

           new Vue({
               el: '#google-map',
               data: {
                   osm: null,
                   address: '<?php echo (!empty($address)) ? str_replace("'", "\'", current($address)) : ''?>',
                   latitude: latitude,
                   longitude: longitude,
                   postal_code: postal_code,
                   place: null,
                   access_token: '<?php echo esc_attr($access_token)?>',
                   ajax_url: 'https://nominatim.openstreetmap.org/search',
                   theMarker: null,
               },
               mounted() {
                   let vm = this;
                   vm.renderMap();
                   setTimeout(function () {
                       vm.renderAutoComplete();
                   }, 2000)
               },
               methods: {
                   renderMap: function () {
                       let vm = this;

                       vm.osm = L.map('openstreetmap').setView([vm.latitude, vm.longitude], 13);

                       let layer = vm.getLayer();

                       vm.osm.addLayer(layer);

                       vm.theMarker = L.marker([vm.latitude, vm.longitude]).addTo(vm.osm);

                       setTimeout(function () {
                           vm.osm.invalidateSize();
                       }, 1000);

                       vm.osm.on('click', function (e) {
                           vm.address = '';
                           vm.postal_code = '';
                           vm.latitude = e.latlng.lat;
                           vm.longitude = e.latlng.lng;
                           let latLon = [vm.latitude, vm.longitude];

                           if (vm.theMarker !== null) {
                               vm.osm.removeLayer(vm.theMarker);
                           }
                           vm.osm.setView([vm.latitude, vm.longitude]);
                           vm.theMarker = L.marker(latLon).addTo(vm.osm);
                       });
                   },

                   getLayer: function () {
                       let vm = this;

                       let tLayer = null;

                       if (vm.access_token !== '') {
                           tLayer = L.tileLayer('https://api.tiles.mapbox.com/v4/{id}/{z}/{x}/{y}.png?access_token=' + vm.access_token, {
                                   attribution: '&copy; <a href="https://www.openstreetmap.org/copyright">OpenStreetMap</a> contributors',
                                   maxZoom: 18,
                                   id: 'mapbox.streets',
                                   accessToken: vm.access_token
                               }
                           );
                       } else {
                           tLayer = L.tileLayer('https://{s}.tile.openstreetmap.org/{z}/{x}/{y}.png', {
                               attribution: '&copy; <a href="https://www.openstreetmap.org/copyright">OpenStreetMap</a> contributors'
                           });
                       }

                       return tLayer;
                   },

                   renderAutoComplete: function () {
                       let vm = this;
                       let query = '#uListing_listing_search';
                       $(query).autocomplete({
                           source: function (req, res) {
                               $.get(vm.ajax_url, {format: 'json', q: req.term,}, function (result) {
                                   if (!result.length) {
                                       res([{value: '', label: 'No results found'}]);
                                       return;
                                   }
                                   res(result.map(function (p) {
                                       return {
                                           latitude: p.lat,
                                           longitude: p.lon,
                                           label: p.display_name,
                                           value: p.display_name,

                                       };
                                   }));
                                   $('.ui-autocomplete').width($(query).parent().width());
                                   this.autocompleteText = $(query).val();
                               }, 'json');
                           },

                           select: function (e, placeFormat) {
                               placeFormat = placeFormat.item;
                               vm.postal_code = '';
                               vm.address = placeFormat.value;
                               vm.latitude = placeFormat.latitude;
                               vm.longitude = placeFormat.longitude;

                               if(vm.theMarker !== null){
                                   vm.osm.removeLayer(vm.theMarker);
                               }

                               vm.osm.setView([vm.latitude, vm.longitude]);
                               vm.theMarker = L.marker([vm.latitude, vm.longitude]).addTo(vm.osm);
                           }
                       });
                   },
               }
           })
       }
    </script>

<?php endif; ?>
