<?php

use uListing\Classes\StmListingAttribute;
use uListing\Admin\Classes\StmListingAttributeList;
use uListing\Classes\StmListingAttributeMeta;

$object = new StmListingAttributeList();
$editObject = $object->isset_edit_object();
$form_title =  esc_html__( 'Add New Attribute', "ulisting");
$options_title =  esc_html__( 'Options', "ulisting");
$submit_btn =  esc_html__( 'Create', "ulisting");

if($editObject){
	$form_title =  esc_html__( 'Edit attribute', "ulisting");
	$submit_btn =  esc_html__( 'Update', "ulisting");
}

?>

<div class="wrap nosubsub">
	<h1 class="wp-heading-inline"><?php echo esc_html__( 'Attribute', "ulisting"); ?></h1>

	<div id="col-container" class="wp-clearfix ">
		<div id="col-left">
			<div class="col-wrap ulisting-main">
				<div class="form-wrap">
					<h2><?php echo esc_html($form_title);?></h2>
					<div id="app">
						<form  v-on:submit="submit" method="post" action="<?php echo admin_url('edit.php?post_type=listing_type&page=listing_attribute')?>" class="validate">
							<?php if(isset($editObject->id)):?>
								<input type="hidden" name="StmListingAttribute[id]" value="<?php echo esc_attr($editObject->id);?>"/>
							<?php endif;?>

							<input type="hidden" name="action" value="listing_attribute_save"/>

							<div class="form-field form-required term-name-wrap" :class="{ 'form-invalid': $v.title.$error }">
								<label for="listing-attribute-title">Title</label>
								<input name="StmListingAttribute[title]" v-model.trim="title" v-on:change="setName" v-on:keyup="setName" id="listing-attribute-title"  type="text">
								<p>The title is how it appears on your site.</p>
								<div class="error" v-if="!$v.title.minLength">Title must have at least {{$v.title.$params.minLength.min}} letters.</div>
							</div>

							<div v-show="show_name" class="form-field form-required term-name-wrap" :class="{ 'form-invalid': $v.name.$error }">
								<label for="listing-attribute-title">Name</label>
								<input name="StmListingAttribute[name]" v-model.trim="name" v-on:change="validateName" v-on:keyup="validateName"  id="listing-attribute-title" type="text"  size="40" aria-required="true">
							</div>

							<div class="form-field form-required term-name-wrap" :class="{ 'form-invalid': $v.name.$error }">
								<label for="listing-attribute-title">Affix</label>
								<input name="StmListingAttribute[affix]" v-model.trim="affix"  type="text">
							</div>

							<div class="form-field form-required term-name-wrap"  :class="{ 'form-invalid': $v.type.$error }">
								<label for="listing-attribute-type">Type</label> <br>
								<select class="form-control" name="StmListingAttribute[type]" v-model.trim="type" @change="select_type" id="listing-attribute-type" type="text">
									<?php foreach (\uListing\Classes\StmListingAttribute::getType() as $key => $val):?>
										<?php $selected = false;  if(isset($editObject->type) and $editObject->type == $key) $selected = true; ?>
										<option value="<?php echo  esc_attr($key)?>" <?php echo ($selected)?'selected':'';   ?>> <?php echo  esc_attr($val)?> </option>
									<?php endforeach; ?>
								</select>
							</div>

							<div>

								<ul class="ulisting-nav min nav nav-justified clear-both" >
									<li class="nav-item"> <a v-bind:class="{active:icon_type == 0}" class="cursor-pointer nav-link" @click="icon_type = 0"> Icon </a> </li>
									<li class="nav-item"> <a v-bind:class="{active:icon_type == 1}" class="cursor-pointer nav-link" @click="icon_type = 1"> Image </a> </li>
								</ul>

								<div class="tab-content">
									<div v-if="icon_type == 0" class="tab-pane active">
										<input type="hidden" v-model="icon" name="StmListingAttribute[icon]">
										<stm-icon-picker :icon="icon" v-on:icon-event="console.log(icon = $event)"></stm-icon-picker>
									</div>
									<div v-if="icon_type == 1" class="tab-pane active">
										<div class="form-field form-required term-name-wrap" :class="{ 'form-invalid': $v.name.$error }">
											<stm-thumbnail-field
												btn_value="<?php esc_html_e('Choose image', "ulisting")?>"
												name="StmListingAttribute[thumbnail_id]"
											></stm-thumbnail-field>
										</div>
									</div>
								</div>
							</div>

                            <p class="submit"><input type="submit" name="submit" id="submit" class="button button-primary" value="<?php echo esc_attr($submit_btn); ?>"></p>
						</form>

					</div>

				</div>

			</div>
		</div><!-- /col-left -->

		<div id="col-right">
			<div class="col-wrap">
				<form method="post">
					<?php
						$object->prepare_items();
						$object->display();
					?>
				</form>
			</div>
		</div><!-- /col-right -->

	</div><!-- /col-container -->
</div>

<script>

	Vue.use(window.vuelidate.default);
	var required = window.validators.required;
	var minLength = window.validators.minLength;

	new Vue({
		el: '#app',
		data: {
			icon_type:0,
			icon:"",
			title: '<?php echo (isset($editObject->title)) ? trim($editObject->title) : '' ; ?>',
			name: '<?php echo (isset($editObject->name)) ? trim($editObject->name) : '' ; ?>',
			type: '<?php echo (isset($editObject->type)) ? trim($editObject->type) : '' ; ?>',
			affix: '<?php echo (isset($editObject->affix)) ? trim($editObject->affix) : '' ; ?>',
			show_name:1
		},
		validations: {
			title: {
				required: required,
				minLength: minLength(3)
			},
			name: {
				required: required,
				minLength: minLength(3)
			},
			type: {
				required: required
			}
		},
		methods:{
			select_type(){
				if(this.type == '<?php echo  StmListingAttribute::TYPE_LOCATION?>') {
					this.show_name = 0
					this.name = this.type
				}else{
					this.show_name = 1
					if(this.name == '<?php echo  StmListingAttribute::TYPE_LOCATION?>')
						this.name = '';
				}
			},
			submit(event){
				this.$v.$touch()
				if(this.$v.$invalid){
					event.preventDefault();
				}else{

				}
			},
			setName() {
				if(this.type == '<?php echo  StmListingAttribute::TYPE_LOCATION?>')
					return
				this.name = this.title;
				this.validateName();
			},
			validateName() {
				var name = '';
				for (var i = 0; i < this.name.length; i++) {
					if(
						(this.name.charCodeAt(i) >= 48  && this.name.charCodeAt(i) <= 57 ) ||
						(this.name.charCodeAt(i) >= 97  && this.name.charCodeAt(i) <= 122 ) ||
						(this.name.charCodeAt(i) >= 65  && this.name.charCodeAt(i) <= 90 ) ||
						this.name.charCodeAt(i) == 32 ||
						this.name.charCodeAt(i) == 45 ||
						this.name.charCodeAt(i) == 95
					) {
						name +=this.name.charAt(i)
						name = name.replace(/[- ]/g,'_');
					}
				}
				this.name = name.toLowerCase();
			}
		}
	})
</script>