<?php

use uListing\Admin\Classes\StmEmailTemplateManager;
$email_option = StmEmailTemplateManager::get_email_templates_store();
$emails = !empty($email_option) ? $email_option : [];
$email_logo = get_post(get_option("ulisting_email_logo"));
$email_banner = get_post(get_option("ulisting_email_banner"));

$socials = !empty(get_option(StmEmailTemplateManager::SOCIAL_OPTION))
    ? get_option(StmEmailTemplateManager::SOCIAL_OPTION)
    : StmEmailTemplateManager::get_socials();

?>

<style>
    .ulisting-panel-header .btn {
        display: none;
    }

    .upload-label {
        display: flex;
        vertical-align: center;
        align-items: center;
        height: 100%;
    }

    .disabled {
        opacity: .4;
        pointer-events: none;
    }
</style>

<?php if ( isset($_GET['email-type']) && !empty($_GET['email-type']) ): ?>
    <div class="clear-both">
        <a href="<?php echo admin_url('edit.php?post_type=listing_type&page=stm_listing_settings&panel=email')?>"><i class="fa fa-long-arrow-left"></i> <?php _e("Back", "ulisting")?></a>
        <hr>
        <?php echo StmEmailTemplateManager::render_settings(sanitize_text_field($_GET['email-type'])); ?>
    </div>
<?php else:?>
<div class="stm-min-height-500px " id="stm_listing_settings_email">
    <div class="p-t-30 p-l-30">
        <h4><?php _e("Email Manager Configurations", "ulisting"); ?></h4>
        <hr>
        <div class="form-group">
            <div class="stm-row">
                <div class="stm-col-3">
                    <div class="upload-label">
                        <label> <?php _e("Upload Logo", "ulisting"); ?></label>
                    </div>
                </div>
                <div class="stm-col-3">
                    <div id="ulisting-default-placeholder">
                        <stm-thumbnail-field
                                :value="logo"
                                @update="update"
                                image_type="ulisting_email_logo"
                                thumbnail_url="<?php echo ($email_logo) ? $email_logo->guid : null ?>"
                                btn_value="<?php esc_html_e('Choose image', "ulisting") ?>"
                        ></stm-thumbnail-field>
                     </div>
                 </div>
            </div>
        </div>
        <hr>

        <div class="form-group">
            <div class="stm-row">
                <div class="stm-col-3">
                    <div class="upload-label">
                        <label> <?php _e("Upload Banner", "ulisting"); ?></label>
                    </div>
                </div>
                <div class="stm-col-3">
                    <div id="ulisting-default-placeholder">
                        <stm-thumbnail-field
                                :value="banner"
                                @update="update"
                                image_type="ulisting_email_banner"
                                thumbnail_url="<?php echo ($email_banner) ? $email_banner->guid : null ?>"
                                btn_value="<?php esc_html_e('Choose image', "ulisting") ?>"
                        ></stm-thumbnail-field>
                    </div>
                </div>
            </div>
        </div>

        <div class="form-group" v-for="social in socials">
            <div class="stm-row">
                <div class="stm-col-3">
                    <label>{{ social.label }}</label>
                </div>
                <div class="stm-col-3">
                    <input class="form-control" type="text" v-model="social.link">
                </div>
            </div>
        </div>

        <div class="form-group">
            <div class="stm-row">
                <div class="stm-col-3">
                    <button class="btn btn-success" @click="sendData"><?php esc_html_e("Save", "ulisting")?></button>
                </div>
            </div>
        </div>
        <br>
    </div>
    <?php if(!ulisting_wishlist_active()):?>
        <?php \uListing\Classes\Notices::notice(\uListing\Classes\Notices::TYPE_ERROR, [__("Install <a target='_blank' href='https://stylemixthemes.com/wordpress-classified-plugin/'>uListing Wishlist</a> addon to unlock saved search feature!", "ulisting")])?>
    <?php endif;?>
    <div class="stm-row">
        <div class="stm-col-12 stm-col-md-12 p-t-15">
            <div class="p-l-15 p-b-15">
                <div v-for="(value, index) in Object.values(emails)" class="card card-default">
                    <div class="card-body" :class="{disabled: wishlist && value.slug === 'saved-search'}">
                        <div class="stm-row">
                            <div class="stm-col-8">
                                <h6 class="p-t-10">
                                    <span> <strong>{{ value.title }}</strong>  </span>
                                    <span v-if="! (value.is_active == 1 || value.is_active == true)" class="m-l-15 pull-right"> <span class="badge badge-warning"><?php _e('Inactive', "ulisting")?></span></span>
                                    <span v-if="(value.is_active == 1 || value.is_active == true)" class="m-l-15 pull-right"> <span class="badge badge-success"><?php _e('Active', "ulisting")?></span></span>
                                </h6>
                                <p>
                                    {{value.description}}
                                </p>
                            </div>
                            <div class="stm-col-4 text-right">
                                <a v-bind:href="'<?php echo admin_url('edit.php?post_type=listing_type&page=stm_listing_settings&panel=email&email-type=') ?>'+ value.slug"
                                   class="btn btn-success">
                                    <i class="fa fa-wrench"></i> <?php esc_html_e("Manage", "ulisting") ?>
                                </a>
                            </div>
                        </div>
                    </div>
                </div>
            </div>
        </div>
    </div>
</div>

<script>
    const store = {
        socials: <?php echo !empty($socials) ? json_encode($socials) : []?>,
        emails: <?php echo json_encode($emails);?>,
        wishlist: '<?php echo !ulisting_wishlist_active()?>'
    }
    new Vue({
        el: '#stm_listing_settings_email',
        data: {
            logo: null,
            banner: null,
            payload: {},
            socials: [],
            emails: store.emails,
            wishlist: store.wishlist,
        },

        created() {
            this.logo = "<?php echo ($email_logo) ? $email_logo->ID : null ?>";
            this.banner = "<?php echo ($email_banner) ? $email_banner->ID : null ?>";
            this.socials = store.socials;
        },

        methods: {

            update({id, option_name} = {}) {
                this.payload[option_name] = id;
            },

            sendData(e) {
                e.preventDefault();
                const data = {
                    socials: this.socials,
                    images: this.payload,
                    nonce: ulistingAjaxNonce
                }
                this.$http.post(currentAjaxUrl + '?action=stm_update_email_data', data).then(response => {
                    const status  = response.body['status'];
                    if (status === 'success')
                        location.reload();
                })
            }
        },
    })
</script>
<?php endif;?>