<?php
use uListing\Classes\StmListingSettings;

$maps = StmListingSettings::getMaps();
$open_map_hover = StmListingSettings::getMapHover();
$map_type = StmListingSettings::get_current_map_type();
$api_key = StmListingSettings::get_current_map_api_key();
$ulisting_back_slots = get_option('ulisting_back_slots');
$ulisting_allow_delete = get_option('allow_delete_listings');
$ulisting_remove_tables = get_option('ulisting_remove_tables');
$ulisting_feature_limit = get_option("ulisting_feature_limit");
$ulisting_category_limit = get_option("ulisting_category_limit");
$ulisting_default_placeholder = get_post(get_option("ulisting_default_placeholder"));
$wishlist_page = StmListingSettings::getPages("wishlist_page");

?>

<?php do_action("ulisting-settings-main-top") ?>

<style>
    .form-group.disabled {
        opacity: 0.5;
        pointer-events: none;
    }
</style>

<hr>

<h4><?php _e("Default photo placeholder", "ulisting"); ?></h4>

<div class="stm-row">

    <div class="stm-col-3">
        <div id="ulisting-default-placeholder">
            <stm-thumbnail-field
                    value="<?php echo ($ulisting_default_placeholder) ? esc_attr($ulisting_default_placeholder->ID) : null ?>"
                    thumbnail_url="<?php echo ($ulisting_default_placeholder) ? esc_attr($ulisting_default_placeholder->guid) : null ?>"
                    btn_value="<?php esc_html_e('Choose image', "ulisting") ?>"
                    name="StmListingsSettings[default_placeholder]"
            ></stm-thumbnail-field>
        </div>
    </div>
</div>
<hr>

<?php ulisting_render_template(ULISTING_ADMIN_PATH . '/views/listing-settings/currency-panel.php', null, true) ?>

<hr>

<h4><?php _e("Map settings", "ulisting"); ?></h4>
<div class="p-t-30" id="ulisting_maps">
        <stm-map-settings
                map_type="<?php echo esc_attr($map_type) ?>"
                :map_lists="maps"
                :api_keys="api_keys"
                :open_by_hover="open_by_hover">
        </stm-map-settings>
</div>

<script>
    new Vue({
        el: "#ulisting-default-placeholder"
    });

    new Vue({
        el: "#ulisting_maps",
        data: {
            maps: <?php echo json_encode($maps) ?>,
            api_keys: <?php echo json_encode($api_key) ?>,
            open_by_hover: '<?php echo esc_attr($open_map_hover)?>',
        },
    });
</script>

<hr>

<h4><?php _e("Pricing plan settings", "ulisting"); ?></h4>
<div class="p-t-30" id="pricing-plan-settings">
    <div class="form-group">
        <div class="stm-row">
            <div class="stm-col-3">
                <label> <?php _e("Delete Listings", "ulisting"); ?></label>
            </div>
            <div class="stm-col-2">
                <label class="ulisting-remove-table">
                    <input type="checkbox" v-model="allow_delete_listings">
                    <div class="ulisting-slider ulisting-round" @click="allow_delete_listings = !allow_delete_listings"></div>
                </label>
                <input type="hidden" name="StmListingsSettings[allow_delete_listings]" :value="allow_delete_listings">
            </div>
            <div class="stm-col-7">
               <code><?php echo __('It allows user to delete a listing or several listings from his pricing slot', 'ulisting')?></code>
            </div>
        </div>
    </div>
    <div class="form-group" :class="{disabled: !allow_delete_listings}">
        <div class="stm-row">
            <div class="stm-col-3">
                <label> <?php _e("Back Listing Slot", "ulisting"); ?></label>
            </div>
            <div class="stm-col-2">
                <label class="ulisting-remove-table">
                    <input type="checkbox" v-model="ulisting_back_slots">
                    <div class="ulisting-slider ulisting-round" @click="ulisting_back_slots = !ulisting_back_slots"></div>
                </label>
                <input type="hidden" name="StmListingsSettings[ulisting_back_slots]" :value="ulisting_back_slots">
            </div>
            <div class="stm-col-7">
                <code><?php _e("It allows backing the slot after listing deletion showing all unreserved listings", "ulisting"); ?></code>
            </div>
        </div>
    </div>
</div>

<hr>
<h4><?php _e("uListing short code", "ulisting"); ?></h4>
<div class="p-t-30">
    <div class="form-group">
        <div class="stm-row">
            <div class="stm-col-3">
                <label> <?php _e("Categories show at most", "ulisting"); ?></label>
            </div>
            <div class="stm-col-2">
                <input class="form-control" name="StmListingsSettings[ulisting_category_limit]" type="number"
                       value="<?php echo esc_attr($ulisting_category_limit) ?>">
            </div>
            <div class="stm-col-7">
                <?php _e("Short code", "ulisting"); ?> : <code>[ulisting-category category='category_one, category_two,
                    ...' listing_type_id="77"]</code>
            </div>
        </div>
    </div>
    <div class="form-group">
        <div class="stm-row">
            <div class="stm-col-3">
                <label> <?php _e("Featured Listings show at most", "ulisting"); ?></label>
            </div>
            <div class="stm-col-2">
                <input class="form-control" name="StmListingsSettings[ulisting_feature_limit]" type="number"
                       value="<?php echo esc_attr($ulisting_feature_limit) ?>">
            </div>
            <div class="stm-col-7">
                <?php _e("Short code", "ulisting"); ?> : <code>[ulisting-feature listing_type_id="77"]</code>
            </div>
        </div>
    </div>

    <div class="form-group">
        <div class="stm-row">
            <div class="stm-col-3">
                <label> <?php _e(" Regions List", "ulisting"); ?></label>
            </div>
            <div class="stm-col-9">
                <?php _e("Short code", "ulisting"); ?> : <code>[ulisting-region-list listing_type_id="77"
                    regions="category_one_id, category_two_id, ..."]</code>
            </div>
        </div>
    </div>
</div>

<hr>

<h4><?php _e("Extra", "ulisting"); ?></h4>
<div class="p-t-30">
    <div class="form-group">
        <div class="stm-row">
            <div class="stm-col-3">
                <label> <?php _e("Remove Database tables on Unistalling Ulisting plugin", "ulisting"); ?></label>
            </div>
            <div class="stm-col-2">
                <div  id="ulisting-remove-table">
                    <label class="ulisting-remove-table">
                        <input type="checkbox" v-model="checkbox" >
                        <div class="ulisting-slider ulisting-round" @click="checkbox = !checkbox"></div>
                    </label>
                    <input type="hidden" name="StmListingsSettings[ulisting_remove_tables]" :value="checkbox">
                </div>
            </div>
            <div class="stm-col-6">
                <code><?php echo __('Wipes all the current settings and data of the uListing plugin from the database.', 'ulisting')?></code>
            </div>
        </div>
    </div>
</div>
<script>
    new Vue({
        el: "#ulisting-remove-table",
        data: {
            checkbox: Boolean(<?php echo !empty($ulisting_remove_tables) ? $ulisting_remove_tables : 0;?>),
        },
        created() {
            if (['', '0', 'false'].indexOf(this.checkbox) !== -1)
                this.checkbox = false
        },
    });

    new Vue({
        el: "#pricing-plan-settings",
        data: {
            allow_delete_listings: <?php echo !empty($ulisting_allow_delete) ? $ulisting_allow_delete : 0;?>,
            ulisting_back_slots: <?php echo !empty($ulisting_back_slots) ? $ulisting_back_slots : 0;?>,
        },
        created() {
            if (['', '0', 'false'].indexOf(this.allow_delete_listings) !== -1)
                this.allow_delete_listings = false

            if (['', '0', 'false'].indexOf(this.ulisting_back_slots) !== -1)
                this.ulisting_back_slots = false
        },
        watch: {
            allow_delete_listings( value ) {
                if ( !value ) {
                    this.ulisting_back_slots = false
                }
            }
        }
    });
</script>
<hr>

<h3><?php _e("Wishlist", "ulisting"); ?></h3>

<div class="p-t-30">
    <div class="form-group">
        <div class="stm-row">

            <?php if (!ulisting_wishlist_active()): ?>
                <div class="stm-col-12">
                    <?php \uListing\Classes\Notices::notice(\uListing\Classes\Notices::TYPE_ERROR, [__("Install <a target='_blank' href='https://stylemixthemes.com/wordpress-classified-plugin/'>uListing Wishlist</a> addon to unlock this feature!", "ulisting")]) ?>
                </div>
            <?php endif; ?>

            <div class="stm-col-3">
                <label> <?php _e("Wishlist link short code", "ulisting"); ?></label>
            </div>
            <div class="stm-col-5">
                <div class="<?php echo (!ulisting_wishlist_active()) ? "ulisting-lock p-t-15  p-l-15  p-r-15 p-b-15" : null; ?>  ">
                    <code>[ulisting-wishlist-link]</code>
                </div>
            </div>
        </div>
    </div>
</div>
<br>

<?php do_action("ulisting-settings-main-bottom") ?>
