<?php
use uListing\Classes\UlistingUserRole;

$ulisting_lock_class = "ulisting-lock";
if(ulisting_user_role_active())
	$ulisting_lock_class = "";

$userRole = new UlistingUserRole();
$data['roles'] = [];
$data['ulisting_user_role_active'] = ulisting_user_role_active();
foreach ($userRole->roles as $key => $role) {
	$custom_fields = apply_filters("ulisting_custom_fields", ['custom_fields' => [], 'role' => $key]);
	$item = [
		"is_delete" => 0,
		"name" => $role['name'],
		"slug" => $key,
		"capabilities" => $role['capabilities'],
		"custom_fields" => $custom_fields['custom_fields']
	];
	$data['roles'][] = $item;
}

?>

<style>
	.ulisting-panel-header .btn{
		display: none;
	}
</style>

<?php if(!ulisting_user_role_active()):?>
	<div class="p-t-15 p-r-15 p-b-15 p-l-15">
		<?php \uListing\Classes\Notices::notice(\uListing\Classes\Notices::TYPE_ERROR, [__("Install <a target='_blank' href='https://stylemixthemes.com/wordpress-classified-plugin/'>uListing User Roles</a> addon to unlock this feature!", "ulisting")])?>
	</div>
<?php endif;?>

<div class="stm-min-height-500px clear-both" id="stm_listing_settings_user_roles">
		<div class="stm-row">
			<div class="stm-col-12 stm-col-md-12 p-t-15">
				<div class="p-l-15 p-b-15">
					<div class="card card-default">
						<div class="card-body">
							<div class="stm-row">
								<div class="stm-col-8">
									<h6 class="p-t-10">
										<span> <strong>Agencies</strong>  </span>
										<span v-if="!ulisting_agencies.is_active" class="m-l-15 pull-right"> <span class="badge badge-warning"><?php _e('Disabled', "ulisting")?></span></span>
										<span v-if="ulisting_agencies.is_active" class="m-l-15 pull-right"> <span class="badge badge-success"><?php _e('Enabled', "ulisting")?></span></span>
									</h6>
									<p>Agencies module enables additional User types "Agency" and "Agent (of Agency)" on your website.<br/>
									Deactivating this module moves all Agencies and Agents to simple User.</p>
								</div>
								<div class="stm-col-4 text-right">
									<span v-if="ulisting_agencies.loader">
										<div class="stm-spinner"> <div></div> <div></div> <div></div> <div></div> <div></div> </div>
									</span>
									<div v-if="!ulisting_agencies.loader">
										<span class="btn btn-success" v-if="!ulisting_agencies.is_active"  @click="agenciesSwitcher('install')">
											<i class="fa fa-check"></i> <?php esc_html_e("Enable", "ulisting")?>
										</span>
										<span class="btn btn-danger"  v-if="ulisting_agencies.is_active" @click="agenciesSwitcher('uninstall')">
											<i class="fa fa-close"></i> <?php esc_html_e("Disable", "ulisting")?>
										</span>
									</div>
								</div>
							</div>
						</div>
					</div>

					<div  v-for="(value, index) in roles" class="card card-default p-t-15" v-show="value.is_delete == 0">
						<div class="card-body">
							<div class="stm-row">
								<div class="stm-col-9 p-t-10">
									<span> <strong><?php _e('Name', "ulisting")?>:</strong> {{ value.name }} </span>
									<span class="m-l-15"> <strong><?php _e("Listings limit", "ulisting")?>:</strong> {{value.capabilities.listing_limit}}</span>
									<span class="m-l-15">
										<label class="ulisting-checkbox">
										<input type="checkbox" value="0" v-bind:true-value="1" v-bind:false-value="0" v-model="value.capabilities.allow_delete_listings">
										<span></span>
										<?php _e("Allow Delete Listings", "ulisting")?>
										</label>
									</span>
									<span v-if="value.capabilities.default" class="m-l-15 pull-right"> <span class="badge badge-info"><?php _e('Default', "ulisting")?></span></span>
								</div>
								<div class="stm-col-3 text-right">
									<button type="button" :disabled="(ulisting_user_role_active != 1) ? true : false" class="btn btn-danger" @click="value.is_delete = 1">
										<i class="fa fa-trash-o" aria-hidden="true"></i>
									</button>
									<button type="button" class="btn btn-light" @click="show_role_panel(index)">
										<i v-if="edit_panel_id == index" class="fa fa-angle-up"></i>
										<i v-if="edit_panel_id != index" class="fa fa-angle-down"></i>
									</button>
								</div>
							</div>
						</div>
						<div v-show="edit_panel_id == index" class="card-body">
							<div class="stm-row">
								<div class="stm-col-4">
									<div class="form-group">
										<label><?php _e("Name", "ulisting")?></label>
										<input type="text" class="form-control" v-model="value.name">
									</div>
								</div>
								<div class="stm-col-4">
									<div class="form-group">
										<label><?php _e("Slug", "ulisting")?></label>
										<input type="text" :disabled="value.is_new != 1"    v-on:change="validateSlug(value,'slug')" v-on:keyup="validateSlug(value,'slug')"  class="form-control" v-model="value.slug">
									</div>
								</div>
								<div class="stm-col-4">
									<div class="form-group">
										<label><?php _e("Listing limit", "ulisting")?></label>
										<input type="number" class="form-control" v-model="value.capabilities.listing_limit">
									</div>
								</div>
								<div class="stm-col-4">
									<div class="form-group">
										<label class="ulisting-checkbox">
											<input type="checkbox" value="1" v-bind:true-value="1" v-bind:false-value="0" v-model="value.capabilities.listing_moderation">
											<span></span>
											<?php _e('Listing Moderation', "ulisting")?>
										</label>
									</div>
								</div>
								<div class="stm-col-4">
									<label class="ulisting-checkbox">
										<input type="checkbox" @click="changeDefault(index, value)" value="1" v-bind:true-value="1" v-bind:false-value="0" v-model="value.capabilities.default">
										<span></span>
										<?php _e('Default role', "ulisting")?>
									</label>
								</div>

								<div class="stm-col-4">
									<div class="form-group">
										<label class="ulisting-checkbox">
											<input type="checkbox" value="1" v-bind:true-value="1" v-bind:false-value="0" v-model="value.capabilities.comment">
											<span></span>
											<?php _e('Comment', "ulisting")?>
										</label>
									</div>
								</div>

							</div>
							<hr>
							<div class="<?php echo esc_html($ulisting_lock_class)?>">
								<draggable v-model="value.custom_fields" class="ulisting-draggable-panel stm-min-height-100px" :options="{group:'attribute', handle:'.handle'}">
									<div class="ulisting-draggable-panel-items" v-for="(element, index) in value.custom_fields" :key="index">
										<div class="ulisting-draggable-panel-items-top">
											<div class="title">
											<span class="handle">
												<i class="fa fa-arrows"></i>
											</span>
												<p>{{element.name}} </p>
											</div>
											<div class="action p-r-10">
											<span class="btn" v-on:click="show_field_panel(element)">
												<i v-if="element.is_open" class="fa fa-angle-up"></i>
												<i v-if="!element.is_open" class="fa fa-angle-down"></i>
											</span>
												<span class="btn" @click="remove(value.custom_fields, index)">
												<i class="fa fa-trash"></i>
											</span>
											</div>
										</div>
										<div class="ulisting-draggable-panel-items-inside" v-show="element.is_open">
											<div class="panel-custom p-t-15 p-b-15 p-l-15 p-r-15">
												<div class="stm-row">
													<div  class="stm-col-3">
														<div class="form-group">
															<label><?php _e('Field type', "ulisting")?></label>
															<ulisting-select2 :options="type_list" v-model="element.type"></ulisting-select2>
														</div>
													</div>
													<div  class="stm-col-3">
														<div class="form-group">
															<label><?php _e('Name', "ulisting")?></label>
															<input type="text" class="form-control" v-model="element.name">
														</div>
													</div>
													<div  class="stm-col-3">
														<div class="form-group">
															<label><?php _e('Slug', "ulisting")?></label>
															<input type="text" class="form-control" v-on:change="validateSlug(element,'slug')" v-on:keyup="validateSlug(element,'slug')"  v-model="element.slug">
														</div>
													</div>
													<div  class="stm-col-3">
														<label class="ulisting-checkbox m-t-30">
															<input type="checkbox"  value="1" v-bind:true-value="1" v-bind:false-value="0" v-model="element.required">
															<span></span>
															<?php _e('Required', "ulisting")?>
														</label>
													</div>
													<div class="stm-col-12" v-if="show_field_items(element)">
														<p><?php _e("Custom field items", "ulisting")?></p>
														<div class="stm-row m-b-10" v-for="(item , item_index) in element.items">
															<div  class="stm-col-4">
																<input class="form-control" type="text" v-model="item.name" placeholder="<?php esc_attr_e("Name item", "ulisting")?>">
															</div>
															<div  class="stm-col-4">
																<input class="form-control" type="text"  v-on:change="validateSlug(item,'slug')" v-on:keyup="validateSlug(item,'slug')"  v-model="item.slug" placeholder="<?php esc_attr_e("Slug item", "ulisting")?>">
															</div>
															<div class="stm-col-2">
																<button type="button" class="btn btn-danger" @click="remove(element.items, item_index)"><i class="fa fa-trash"></i></button>
															</div>
														</div>
														<div class="stm-row">
															<div class="stm-col-6 m-t-10">
																<button type="button" class="btn btn-success" @click="add_item_field(element)"><?php _e("Add item", "ulisting")?></button>
															</div>
														</div>
													</div>
												</div>
											</div>
											<div style="clear:both;"></div>
										</div>
									</div>
								</draggable>
								<div class="stm-row">
									<div class="stm-col-3 m-t-15">
										<button :disabled="(ulisting_user_role_active != 1) ? true : false" class="btn btn-success" @click="add_custom_fields(value)" type="button"><?php _e("Add custom field")?></button>
									</div>
								</div>
							</div>
						</div>
					</div>
				</div>
			</div>
		</div>

		<div v-if="loading" class="panel-custom p-t-30 p-b-30 text-center">
			<div class="stm-spinner"> <div></div> <div></div> <div></div> <div></div> <div></div> </div>
		</div>

		<div v-if="!loading" class="stm-row stm-justify-content-end m-b-30">
			<div class="stm-col-3 text-right">
				<button type="button" :disabled="(ulisting_user_role_active != 1) ? true : false" class="btn btn-info cursor-pointer" v-bind:class="{'ulisting-lock-btn' : !ulisting_user_role_active}" @click="add">

					<i class="fa fa-plus-circle"></i> <?php _e('Add role', "ulisting")?>
				</button>
				<button type="button" class="btn btn-success cursor-pointer" @click="save"> <i class="fa fa-floppy-o"></i> <?php _e('Save', "ulisting")?></button>
			</div>
		</div>
</div>

<script>
	var ulisting_agencies_data = {
		currentAjaxUrl: '<?php echo admin_url( 'admin-ajax.php', 'relative' )?>',
		loader: false,
		is_active: <?php echo (get_role("agency")) ? 1 : 0;?>
	}
	var ulisting_user_roles_data = json_parse('<?php echo ulisting_convert_content(json_encode($data))?>');
	new Vue({
		el:'#stm_listing_settings_user_roles',
		data: {
			loading: false,
			message: "",
			edit_panel_id: null,
			roles: [],
			type_list: [
				{
					id:"text",
					text:"Text",
				},
				{
					id:"number",
					text:"Number",
				},
				{
					id:"select",
					text:"Select",
				},
				{
					id:"checkbox",
					text:"Checkbox",
				},
				{
					id:"radio",
					text:"Radio",
				},
				{
					id:"textarea",
					text:"Textarea",
				}
			],
			ulisting_agencies: {}
		},
		created() {
			if(typeof ulisting_user_roles_data == "undefined")
				return;

			if(typeof ulisting_user_roles_data.roles != "undefined")
				this.roles = ulisting_user_roles_data.roles;

			for(index in this.roles){
				this.check_custom_fields(this.roles[index]);
			}

			if(typeof ulisting_user_roles_data.ulisting_user_role_active != "undefined")
				this.ulisting_user_role_active = ulisting_user_roles_data.ulisting_user_role_active;
			
			if(typeof ulisting_agencies_data != 'undefined' ) {
				this.ulisting_agencies = ulisting_agencies_data
			}
		},
		methods: {
			check_custom_fields: function(role){
				if( !role.custom_fields.length )
					role.custom_fields = [];
			},
			show_field_panel: function(element) {
				if(element.is_open){
					element.is_open = false
					return;
				}
				element.is_open = true
			},
			show_field_items(field){
				if(field.type == 'select' || field.type == 'checkbox' || field.type == 'radio')
					return true;
				return false;

			},
			remove: function(items, index) {
				items.splice(index, 1);
			},
			add_custom_fields: function(role){
				if(this.ulisting_user_role_active != 1)
					return;
				role.custom_fields.push({
					type     : "text",
					name     : "Name",
					slug     : "",
					required : 1,
					items    : [],
					is_open  : 0
				});
			},
			add_item_field: function(field){
				if(this.ulisting_user_role_active != 1)
					return;
				field.items.push({
					slug : "",
					name : ""
				});
			},
			show_role_panel: function(index){
				if(this.edit_panel_id == index){
					this.edit_panel_id = null;
					return;
				}
				this.edit_panel_id = index;
			},
			add: function(){
				if(this.ulisting_user_role_active != 1)
					return;
				this.roles.push({
					is_delete:0,
					is_new:1,
					name:"Simple Name",
					slug:"simple_slug",
					custom_fields:[],
					capabilities:{
						default:0,
						listing_limit:0,
						comment:0,
						stm_listing_role:1,
						listing_moderation:0,
                        allow_delete_listings:0,
					}
				})
			},
			changeDefault: function(index, value){
				for(key in this.roles){
					if(index != key) {
						this.roles[key].capabilities.default = 0;
					}
				}
			},
			validateSlug(model, value) {
				var name = '';
				for (var i = 0; i < model[value].length; i++) {
					if(
						(model[value].charCodeAt(i) >= 48  && model[value].charCodeAt(i) <= 57 ) ||
						(model[value].charCodeAt(i) >= 97  && model[value].charCodeAt(i) <= 122 ) ||
						(model[value].charCodeAt(i) >= 65  && model[value].charCodeAt(i) <= 90 ) ||
						model[value].charCodeAt(i) == 32 ||
						model[value].charCodeAt(i) == 45 ||
						model[value].charCodeAt(i) == 95
					) {
						name +=model[value].charAt(i)
						name = name.replace(/[- ]/g,'_');
					}
				}
				model[value] = name.toLowerCase();

			},
			save: function() {
				var vm = this;
				vm.message = null;
				vm.loading = true;
				this.$http.post("ulisting-user/role/save", {
				    roles: vm.roles,
                    nonce: ulistingAjaxNonce
				}).then(function(response){
					vm.loading = false;
					vm.message = response.body['message'];
					if(response.body['success']) {
					}
				});
			},
			agenciesSwitcher:function(mode) {
				var formData = new FormData();
				formData.append('StmAgenciesSwitcher[mode]', mode);
				this.ulisting_agencies.loader = true;
				this.$http.post(ulisting_agencies_data.currentAjaxUrl + '?action=stm_agencies_switcher', formData).then(function(response){
					if(response.body.status == 'success') {
						this.ulisting_agencies.is_active = (mode == 'install') ? true : false;
						location.reload();
					}
					this.ulisting_agencies.loader = false;
				});
			},
		}
	})

</script>