<?php
use uListing\Classes\StmListingAttribute;
$id = rand(10000, 99999);
$usedAttributeIds    = $listingType->getMeta('listing_type_attribute', true);
$attribute_required  = $listingType->getMeta('stm_listing_type_attribute_require', true);
$availableAttributes = StmListingAttribute::all();

$data = [
	"used" => [],
	"available" => [],
];

foreach ($availableAttributes as  $attribute){

	if(isset($usedAttributeIds[$attribute->id])){
		$data['used'][] = [
			"id" =>  $attribute->id,
			"name" =>  $attribute->title,
			"is_options" =>  $attribute->isOptions(),
			"is_open" =>  0,
			"required" =>  (isset($attribute_required[$attribute->id])) ? 1 : 0,
			"action_panel" => true
		];
	}else{
		$data['available'][] = [
			"id" =>  $attribute->id,
			"name" =>  $attribute->title,
			"is_options" =>  $attribute->isOptions(),
			"is_open" =>  0,
			"required" =>  (isset($attribute_required[$attribute->id])) ? 1 : 0,
			"action_panel" => true
		];
	}
}


$sort_used = [];
if ( ! empty( $usedAttributeIds ) ) {
    foreach ($usedAttributeIds as $sort_id => $attr_id) {
        foreach ($data['used'] as $used) {
            if($sort_id == $used['id']) {
                $sort_used[] = $used;
            }
        }
    }
}

if(!empty($sort_used)) $data['used'] = $sort_used;

?>

<div class="stm-row">
	<div class="stm-col-12 stm-col-md-8" id="listing-type-attribute">
		<div class="stm-row">
			<div class="stm-col-12 stm-col-md-6 ulisting-overflow">
				<div class="stm-height-100 p-l-15 p-r-15">
					<div>
                        <div>
                            <div class="sidebar-name">
                                <h2><?php _e("Used attribute", "ulisting")?> <span class="spinner"></span></h2>
                            </div>
                            <div style="display: flex; justify-content: space-between">
                                <div class="sidebar-description">
                                    <p class="description"><?php _e("Description", "ulisting")?></p>
                                </div>
                            </div>
                        </div>

                    </div>
					<draggable v-model="used" class="ulisting-draggable-panel stm-min-height-100px stm-height-100" :options="{group:'attribute', handle:'.handle'}">
						<div class="ulisting-draggable-panel-items" v-for="(element, index) in used" :key="index">
							<div class="ulisting-draggable-panel-items-top">
								<div class="title">
									<span class="handle">
										<i class="fa fa-arrows"></i>
									</span>

									<p>{{element.name}} </p>
									<span class="required" v-if="element.required">*</span>

								</div>
								<div v-if="element.action_panel" class="action">
									<span class="btn" v-on:click="openPanel(index)">
									<i v-if="element.is_open" class="fa fa-angle-up"></i>
									<i v-if="!element.is_open" class="fa fa-angle-down"></i>
									</span>
									<span class="btn" @click="remove_from_use(index)">
										<i class="fa fa-trash"></i>
									</span>
								</div>
							</div>
							<div class="ulisting-draggable-panel-items-inside" v-show="element.is_open">
								<div class="panel-custom p-t-15 p-b-15 p-l-15 p-r-15">
									<div class="stm-row">
										<div class="stm-col-6">
											<label class="ulisting-checkbox">
												<input type="checkbox" v-model="element.required">
												<span></span>
												<?php esc_html_e('Required', "ulisting")?>
											</label>
										</div>
										<div class="stm-col-6 text-right">
											<div class="action">
												<a v-if="element.is_options" target="_blank" :href="'<?php echo admin_url('edit-tags.php?taxonomy=listing-attribute-options&post_type=listing_type&attribute_id=')?>'+element.id" class="btn">
													<i class="fa fa-list-ul"></i>
												</a>
												<a target="_blank" :href="'<?php echo admin_url('edit.php?post_type=listing_type&page=listing_attribute_edit&attribute_id=')?>'+element.id" class="btn">
													<i class="fa fa-edit"></i>
												</a>
											</div>
										</div>
									</div>
								</div>
								<div style="clear:both;"></div>
							</div>
						</div>
					</draggable>
				</div>
			</div>
			<div class="stm-col-12 stm-col-md-6 ulisting-overflow">
				<div class="p-l-15 p-r-15 stm-height-100">
					<div class="sidebar-name">
						<h2><?php _e("All available attribute", "ulisting")?> <span class="spinner"></span></h2>
					</div>
					<div class="sidebar-description">
						<p class="description"><?php _e("Description", "ulisting")?></p>
					</div>
					<draggable v-model="available" class="ulisting-draggable-panel stm-min-height-100px stm-height-100" :options="{group:'attribute', handle:'.handle'}">
						<div class="ulisting-draggable-panel-items" v-for="(element, index) in available" :key="index">
							<div class="ulisting-draggable-panel-items-top">
								<div class="title">
									<span class="handle">
										<i class="fa fa-arrows"></i>
									</span>
									<p>{{element.name}}</p>
								</div>
							</div>
							<div class="ulisting-draggable-panel-items-inside" v-show="element.is_open">
								<div class="panel-custom p-t-15 p-b-15">
									<div class="col-xs-12">
										<label class="ulisting-checkbox">
											<input type="checkbox" v-model="element.required">
											<span></span>
											<?php esc_html_e('Required', "ulisting")?>
										</label>
									</div>
								</div>
								<div style="clear:both;"></div>
							</div>
						</div>
					</draggable>
				</div>
			</div>
		</div>
		<input v-for="element in used" type="hidden" name="attribute[]" v-model="element.id">
		<input v-for="element in used" v-if="element.required" type="hidden" v-bind:name="'attribute_required[]'" v-model="element.id">
	</div>
	<div class="stm-col-12 stm-col-md-4">
		<div class="sidebar-name"><h2><?php _e("Add new attribute","ulisting")?> <span class="spinner"></span></h2></div>
		<div class="sidebar-description"><p class="description"><?php _e("Description","ulisting")?></p></div>
		<?php ulisting_render_template(ULISTING_ADMIN_PATH . '/views/listing-types/add-attribute-form.php', ['listingType' => $listingType, "id" => $id], true);?>
	</div>
</div>

<script>
	const listing_type_attribute = json_parse('<?php echo ulisting_convert_content(json_encode($data))?>');
    const listing_type_attribute_app =  new Vue({
		el: '#listing-type-attribute',
		data: {
			used: [],
			available: [],
		},
		created(){
            const vm = this;
			if(typeof listing_type_attribute == "undefined")
				return;

			if(typeof listing_type_attribute.used != "undefined")
				this.used = listing_type_attribute.used;

			if(typeof listing_type_attribute.available != "undefined")
				this.available = listing_type_attribute.available;

			UlistingEventBus.$on("add-attribute_<?php echo esc_attr($id)?>", function($data) {
				vm.add_attribute($data);
			})
		},
		methods: {
			add_attribute: function($data){
				this.available.push($data);
			},
			openPanel: function(index) {
				if(this.used[index].is_open)
					this.used[index].is_open = false
				else{
					this.used.forEach(function(item) {
						item.is_open = false
					})
					this.used[index].is_open = true
				}

			},
			remove_from_use: function(index){
				this.available.push(this.used[index])
				if(this.used.length > 1)
					this.used.splice(index, 1);
				else
					this.used = [];
			}
		}
	})
</script>



