<?php
use uListing\Classes\StmListingAttribute;
$data = [
	'available' => [],
	'used' => []
];

$ulisting_lock_class = "ulisting-lock";
if(ulisting_listing_compare_active())
	$ulisting_lock_class = "";

wp_enqueue_script('ulisting-listing-compare', ULISTING_URL . '/assets/js/admin/listing-compare.js', array('vue'), ULISTING_VERSION, true);

$usedAttributeIds = $listingType->getMeta('ulisting_listing_compare_attribute', true);
$availableAttributes = StmListingAttribute::all();
$usedAttributes = [];

$availableAttributes[] = (object) array(
	'id' => 'category',
	'title' => esc_html__('Category', "ulisting"),
);

foreach ($availableAttributes as $key => $attribute){
	if(isset($usedAttributeIds[$attribute->id])){
		$usedAttributes[$usedAttributeIds[$attribute->id]] = $attribute;
		unset($availableAttributes[$key]);
	}else{
		if(!apply_filters('ulisting_listing_compare_available_attributes_check', $attribute) )
			continue;
		$data['available'][] = [
			'id' => $attribute->id,
			'name' => str_replace('"', '\"',  $attribute->title)
		];
	}
}
ksort($usedAttributes);

foreach ($usedAttributes as $usedAttribute) {
	$data['used'][] = [
		'id' => $usedAttribute->id,
		'name' => str_replace('"', '\"',  $usedAttribute->title)
	];
}

wp_add_inline_script('ulisting-listing-compare', "var ulisting_listing_compare_data = json_parse('".str_replace("'", "\'", json_encode($data))."');", 'before');
?>

<div id="ulisting-listing-compare">
	<?php if(!ulisting_listing_compare_active()):?>
		<div class="p-t-15 p-r-15 p-b-15 p-l-15">
			<?php \uListing\Classes\Notices::notice(\uListing\Classes\Notices::TYPE_ERROR, [__("Install <a target='_blank' href='https://stylemixthemes.com/wordpress-classified-plugin/'>uListing Compare</a> addon to unlock this feature!", "ulisting")])?>
		</div>
	<?php endif;?>
	<div class="stm-row">

		<div class="stm-col-12">
			<div class="p-l-15  p-r-15 p-t-15 p-b-15">
				<?php _e("Short code", "ulisting")?> <code>[ulisting-compare-link]</code>
			</div>
		</div>

		<div class="stm-col-12 stm-col-sm-6 <?php echo esc_html($ulisting_lock_class)?> ulisting-overflow">
			<div class="stm-height-100 p-l-15  p-r-15">
				<div class="sidebar-name">
					<h2><?php esc_html_e('Used attribute', "ulisting")?><span class="spinner"></span></h2>
				</div>
				<div class="sidebar-description">
					<p class="description"><?php esc_html_e('Description', "ulisting")?></p>
				</div>
				<draggable v-model="used" class="ulisting-draggable-panel stm-min-height-100px stm-height-100" :options="{group:'attribute', handle:'.handle'}">
					<div class="ulisting-draggable-panel-items" v-for="(element, index) in used" :key="index">
						<div class="ulisting-draggable-panel-items-top">
							<div class="title">
								<span class="handle">
									<i class="fa fa-arrows"></i>
								</span>
								<p>{{element.name}} </p>
							</div>
						</div>
					</div>
				</draggable>
			</div>
		</div>
		<div class="stm-col-12 stm-col-sm-6 <?php echo esc_html($ulisting_lock_class)?> ulisting-overflow">
			<div class="stm-height-100 p-l-15 p-r-15">
				<div class="sidebar-name">
					<h2><?php esc_html_e('All available attribute', "ulisting")?><span class="spinner"></span></h2>
				</div>
				<div class="sidebar-description">
					<p class="description"><?php esc_html_e('Description', "ulisting")?></p>
				</div>
				<draggable v-model="available" class="ulisting-draggable-panel stm-min-height-100px stm-height-100" :options="{group:'attribute', handle:'.handle'}">
					<div class="ulisting-draggable-panel-items" v-for="(element, index) in available" :key="index">
						<div class="ulisting-draggable-panel-items-top">
							<div class="title">
								<span class="handle">
									<i class="fa fa-arrows"></i>
								</span>
								<p>{{element.name}} </p>
							</div>
						</div>
					</div>
				</draggable>
			</div>
		</div>
	</div>
	<input v-for="element in used" type="hidden" name="StmListingTypeListingCompareAttribute[]" v-model="element.id">
</div>
