<?php
use uListing\Classes\StmListingAttribute;

$listingsOrder = $listingType->getListingsOrder();

$view_type = (isset($listingsOrder['view_type'])) ? $listingsOrder['view_type'] : 'list';
$order_by_default = (isset($listingsOrder['order_by_default'])) ? $listingsOrder['order_by_default'] : null;
$items = (isset($listingsOrder['items'])) ? $listingsOrder['items'] : [];
$data = [];

foreach ($items as $item){

	if( !isset( $item['order_by']) )
		continue;

	$data['used'][] = [
		"id" => $item['order_by'].'#'.$item['order_type'],
		"label" => $item['label'],
		"order_by" => (isset($item['order_by'])) ? $item['order_by'] : null,
		"order" => (isset($item['order_type'])) ? $item['order_type'] : null,
		"is_open" => (isset($item['is_open'])) ? $item['is_open'] : false,
	];
}

foreach ($listingType->availableSortField() as $k => $v){
	$data['order_field_list'][] = [
		"id" => $k,
		"text" => $v,
	];
}

foreach (StmListingAttribute::getOrderList() as $k => $v){
	$data['order_type_list'][] = [
		"id" => $k,
		"text" => $v,
	];
}
?>

<div id="listing-order">
	<div class="row">
		<div class="col-sm-6 col-xs-12">
			<div class="p-l-15 p-r-15">
				<div class="sidebar-name">
					<h2><?php _e('Ordering options', "ulisting")?>  <span class="spinner"></span></h2>
				</div>
				<div class="sidebar-description custom">
					<p class="description"><?php _e('These options will appear in the "Order By" dropdown in the Inventory page. Click on an option to edit. Drag & Drop to reorder.', "ulisting")?></p>
				</div>

				<draggable v-model="used" class="ulisting-draggable-panel stm-min-height-100px" :options="{group:'attribute', handle:'.handle'}">
					<div class="ulisting-draggable-panel-items" v-for="(element, index) in used" :key="index">
						<div class="ulisting-draggable-panel-items-top">
							<div class="title">
									<span class="handle">
										<i class="fa fa-arrows"></i>
									</span>
								<p>{{element.label}}</p>
							</div>
							<div class="action text-right p-r-10">
								<span class="btn" v-on:click="openPanel(index)">
									<i v-if="element.is_open" class="fa fa-angle-up"></i>
									<i v-if="!element.is_open" class="fa fa-angle-down"></i>
								</span>
								<span class="btn" @click="remove(index)">
									<i class="fa fa-trash"></i>
								</span>
							</div>
						</div>
						<div class="ulisting-draggable-panel-items-inside" style="display: block" v-show="element.is_open">
							<div class="panel-custom p-t-15 p-b-15 p-r-15 p-l-15">
								<div class="form-group">
									<label><?php _e("Label", "ulisting")?></label>
									<input :name="'listingOrder[items]['+index+'][label]'" type="text" v-model="element.label" class="form-control" >
								</div>

								<div class="form-group">
									<label><?php _e("Order by", "ulisting")?></label>
									<ulisting-select2
										:options='order_field_list'
										v-model='element.order_by'
										:placeholder="'Select field'"
										class="ulisting-select"
										:name="'listingOrder[items]['+index+'][order_by]'">
									</ulisting-select2>
								</div>

								<div class="form-group">
									<label><?php _e("Order", "ulisting")?></label>
									<ulisting-select2 :options='order_type_list'
										v-model='element.order'
										:placeholder="'Select order type'"
										class="ulisting-select"
										:name="'listingOrder[items]['+index+'][order_type]'">
									</ulisting-select2>
								</div>
							</div>
							<div style="clear: both"></div>
						</div>
					</div>
				</draggable>

				<button v-on:click="add" type="button" class="btn btn-success w-full m-t-15"><?php _e('Add option', "ulisting")?> </button>

			</div>
		</div>
		<div class="col-sm-6 col-xs-12">
			<div class="p-l-15 p-r-15">
				<div class="sidebar-name">
					<h2><?php _e('Default', "ulisting")?>  <span class="spinner"></span></h2>
				</div>
				<div class="sidebar-description">
					<br>
					<br>
					<p class="description custom"><?php //_e("Description", "ulisting")?></p>
				</div>

				<div class="panel-custom ulisting-boder-1  ulisting-boder-color-base-7 p-t-15 p-b-15 p-l-15 p-r-15">

					<div class="col-sm-12 col-xs-12">
						<div class="form-group p-t-15">
							<label>
								<?php _e("View order panel", "ulisting")?>
							</label>
							<div class="form-group">

								<label class="ulisting-radio-button m-r-15">
									<input type="radio" class="form-control" v-model="view_type" value="list" name="listingOrder[view_type]">
									<span></span>
									<?php _e("List", "ulisting")?>
								</label>

								<label class="ulisting-radio-button">
									<input type="radio" class="form-control"  v-model="view_type" value="dropdowns" name="listingOrder[view_type]">
									<span></span>
									<?php _e("Dropdowns", "ulisting")?>
								</label>

							</div>
						</div>
					</div>
					<div class="col-sm-12 col-xs-12">
						<div class="form-group p-t-15">
							<label><?php _e("Order by default", "ulisting")?></label>
							<ulisting-select2 :options='used'
								v-model='order_by_default'
								:placeholder="'<?php esc_html_e("Select order by default", "ulisting")?>'"
								class="ulisting-select"
								:text="'label'">
							</ulisting-select2>
							<input type="hidden" name="listingOrder[order_by_default]" v-model="order_by_default">
						</div>
					</div>

				</div>


			</div>
		</div>
	</div>

</div>

<script>

	var listing_order_data = json_parse('<?php echo ulisting_convert_content(json_encode($data))?>');

	new Vue({
		el: '#listing-order',
		data: {
			used: [],
			view_type: '<?php echo esc_attr($view_type);?>',
			order_by_default:'<?php echo esc_attr($order_by_default)?>',
			order_field_list:[],
			order_type_list: [],
		},
		created(){
			if(typeof listing_order_data == "undefined")
				return;

			if(typeof listing_order_data.used != "undefined")
				this.used = listing_order_data.used;

			if(typeof listing_order_data.order_field_list != "undefined")
				this.order_field_list = listing_order_data.order_field_list;

			if(typeof listing_order_data.order_type_list != "undefined")
				this.order_type_list = listing_order_data.order_type_list;

		},
		methods: {
			openPanel(index) {
				if(this.used[index].is_open)
					this.used[index].is_open = false
				else{
					this.used.forEach(function(item) {
						item.is_open = false
					})
					this.used[index].is_open = true
				}
			},
			remove(index) {
				this.used.splice(index, 1);
			},
			add() {
				this.used.push({
					label: "New option",
					order_by: null,
					order: 'ASC',
					is_open: false
				});
				var length = this.used.length;
				this.openPanel( length -1 );
			},
			change(index) {
				this.used[index].id = this.used[index].order_by+'#'+this.used[index].order
			}
		}
	})

</script>