<?php
$search_data = [
    'available' => [],
    'used' => []
];

$search_usedAttributeIds = $listingType->getMeta('stm_uListing_listing_search_category', true);
$search_usedAttributes = [];

$search_availableAttributes[] = (object)array(
    'id' => 'category',
    "is_open" => 0,
    "col" => 12,
    'title' => esc_html__('Category', "ulisting"),
);

$search_availableAttributes[] = (object)array(
    'id' => 'region',
    "is_open" => 0,
    "col" => 12,
    'title' => esc_html__('Region', "ulisting"),
);

foreach ($search_availableAttributes as $key => $attribute) {
    if (isset($search_usedAttributeIds[$attribute->id])) {
        $search_usedAttributes[$search_usedAttributeIds[$attribute->id]] = $attribute;
        unset($search_availableAttributes[$key]);
    } else {
        $search_data['available'][] = [
            'id' => $attribute->id,
            "is_open" => 0,
            "col" => 12,
            'name' => $attribute->title
        ];
    }
}
ksort($search_usedAttributes);

foreach ($search_usedAttributes as $search_usedAttribute) {
    $search_data['used'][] = [
        'id' => $search_usedAttribute->id,
        "is_open" => 0,
        'name' => $search_usedAttribute->title
    ];
}
?>

<div id="uListingAutoComplete">
    <hr style="margin: 50px 0 30px 25px; border: 2px solid black; width: 96%;">
    <div class="stm-row">
        <div class="stm-col-12 stm-col-sm-6 ulisting-overflow">
            <div class="stm-height-100 p-l-15 p-r-15">
                <div class="sidebar-name">
                    <h2><?php esc_html_e('Used attributes for autocomplete results', "ulisting") ?><span
                                class="spinner"></span></h2>
                </div>
                <div class="sidebar-description">
                    <p class="description"><?php esc_html_e('Description', "ulisting") ?></p>
                </div>
                <draggable v-model="used" class="ulisting-draggable-panel stm-min-height-100px stm-height-100"
                           :options="{group:'attribute', handle:'.handle'}">
                    <div class="ulisting-draggable-panel-items" v-for="(element, index) in used" :key="index">
                        <div class="ulisting-draggable-panel-items-top">
                            <div class="title">
								<span class="handle">
									<i class="fa fa-arrows"></i>
								</span>
                                <p>{{element.name}} </p>
                            </div>
                            <div class="action">
								<span class="btn" v-on:click="openPanel(index)">
									<i v-if="element.is_open" class="fa fa-angle-up"></i>
									<i v-if="!element.is_open" class="fa fa-angle-down"></i>
								</span>
                            </div>
                        </div>

                        <div class="ulisting-draggable-panel-items-inside" v-show="element.is_open">
                            <div class="panel-custom p-t-15 p-b-15 p-l-15 p-r-15">
                                <div class="stm-row">
                                    <div class="stm-col-12">
                                        <ulisting-select2
                                                :options='cols'
                                                v-model='element.col'
                                                :placeholder="'<?php esc_html_e('Select col', 'ulisting') ?>'"
                                                class="ulisting-select"
                                                :name="'submit_form_col['+element.id+']'">
                                        </ulisting-select2>
                                    </div>
                                </div>
                            </div>
                            <div style="clear:both;"></div>
                        </div>
                    </div>
                </draggable>
            </div>
        </div>
        <div class="stm-col-12 stm-col-sm-6 ulisting-overflow">
            <div class="stm-height-100 p-l-15 p-r-15">
                <div class="sidebar-name">
                    <h2><?php esc_html_e('All available attributes for autocomplete results', "ulisting") ?><span
                                class="spinner"></span></h2>
                </div>
                <div class="sidebar-description">
                    <p class="description"><?php esc_html_e('Description', "ulisting") ?></p>
                </div>
                <draggable v-model="available" class="ulisting-draggable-panel stm-min-height-100px stm-height-100"
                           :options="{group:'attribute', handle:'.handle'}">
                    <div class="ulisting-draggable-panel-items" v-for="(element, index) in available" :key="index">
                        <div class="ulisting-draggable-panel-items-top">
                            <div class="title">
								<span class="handle">
									<i class="fa fa-arrows"></i>
								</span>
                                <p>{{element.name}} </p>
                            </div>
                        </div>
                    </div>
                </draggable>
            </div>
        </div>
    </div>
    <input v-for="element in used" type="hidden" name="StmListingTypeSearchCategory[]" v-model="element.id">
</div>

<script>
    let stm_search_form_data = <?php echo json_encode($search_data);?>;
    new Vue({
        el: '#uListingAutoComplete',
        data: {
            used: [],
            available: [],
            cols: [
                {
                    id: 2,
                    text: 2
                },
                {
                    id: 3,
                    text: 3
                },
                {
                    id: 4,
                    text: 4
                },
                {
                    id: 6,
                    text: 6
                },
                {
                    id: 8,
                    text: 8
                },
                {
                    id: 10,
                    text: 10
                },
                {
                    id: 12,
                    text: 12
                }
            ]
        },
        created: function () {
            if (typeof stm_search_form_data != 'undefined') {
                this.used = stm_search_form_data.used;
                this.available = stm_search_form_data.available;
            } else {
                this.used = [];
                this.available = [];
            }
        },
        methods: {
            openPanel: function (index) {

                if (this.used[index].is_open)
                    this.used[index].is_open = false;
                else {
                    this.used.forEach(function (item) {
                        item.is_open = false
                    });
                    this.used[index].is_open = true;
                }

            }
        }

    })
</script>