<?php
use uListing\Classes\StmListingAttribute;
$data = [
	'available' => [],
	'used' => []
];

$submit_form_col = $listingType->getMeta('stm_listing_type_submit_form_col');
$usedAttributeIds = $listingType->getMeta('stm_listing_type_subnit_form', true);
$availableAttributes = StmListingAttribute::all();
$usedAttributes = [];

$availableAttributes[] = (object) array(
	'id' => 'category',
	"is_open" =>  0,
	"col" =>  12,
	'title' => esc_html__('Category', "ulisting"),
);

$availableAttributes[] = (object) array(
	'id' => 'region',
	"is_open" =>  0,
	"col" =>  12,
	'title' => esc_html__('Region', "ulisting"),
);

foreach ($availableAttributes as $key => $attribute){
	if(isset($usedAttributeIds[$attribute->id])){
		$usedAttributes[$usedAttributeIds[$attribute->id]] = $attribute;
		unset($availableAttributes[$key]);
	}else{
		$data['available'][] = [
			'id' => $attribute->id,
			"is_open" =>  0,
			"col" =>  12,
			'name' => str_replace('"', '\"', $attribute->title)
		];
	}
}
ksort($usedAttributes);

foreach ($usedAttributes as $usedAttribute) {
	$data['used'][] = [
		'id' => $usedAttribute->id,
		"is_open" =>  0,
		"col" => (is_array($submit_form_col) AND isset($submit_form_col[$usedAttribute->id])) ? $submit_form_col[$usedAttribute->id] : 12,
		'name' => str_replace('"', '\"', $usedAttribute->title)
	];
}

wp_add_inline_script('stm-submit-form', "var stm_admin_submit_form_data = json_parse('".str_replace("'", "\'", json_encode($data))."');", 'before');
?>

<div id="stm-admin-submit-form">
	<div class="stm-row">
		<div class="stm-col-12 stm-col-sm-6 ulisting-overflow">
			<div class="stm-height-100 p-l-15 p-r-15">
				<div class="sidebar-name">
					<h2><?php esc_html_e('Used attribute', "ulisting")?><span class="spinner"></span></h2>
				</div>
				<div class="sidebar-description">
					<p class="description"><?php esc_html_e('Description', "ulisting")?></p>
				</div>
				<draggable v-model="used" class="ulisting-draggable-panel stm-min-height-100px stm-height-100" :options="{group:'attribute', handle:'.handle'}">
					<div class="ulisting-draggable-panel-items" v-for="(element, index) in used" :key="index">
						<div class="ulisting-draggable-panel-items-top">
							<div class="title">
								<span class="handle">
									<i class="fa fa-arrows"></i>
								</span>
								<p>{{element.name}} </p>
							</div>
							<div class="action">
								<span class="btn" v-on:click="openPanel(index)">
									<i v-if="element.is_open" class="fa fa-angle-up"></i>
									<i v-if="!element.is_open" class="fa fa-angle-down"></i>
								</span>
							</div>
						</div>

						<div class="ulisting-draggable-panel-items-inside" v-show="element.is_open">
							<div class="panel-custom p-t-15 p-b-15 p-l-15 p-r-15">
								<div class="stm-row">
									<div class="stm-col-12">
										<ulisting-select2
											:options='cols'
											v-model='element.col'
											:placeholder="'<?php esc_html_e( 'Select col', 'ulisting')?>'"
											class="ulisting-select"
											:name="'submit_form_col['+element.id+']'">
										</ulisting-select2>
									</div>
								</div>
							</div>
							<div style="clear:both;"></div>
						</div>
					</div>
				</draggable>
			</div>
		</div>
		<div class="stm-col-12 stm-col-sm-6 ulisting-overflow">
			<div class="stm-height-100 p-l-15 p-r-15">
				<div class="sidebar-name">
					<h2><?php esc_html_e('All available attribute', "ulisting")?><span class="spinner"></span></h2>
				</div>
				<div class="sidebar-description">
					<p class="description"><?php esc_html_e('Description', "ulisting")?></p>
				</div>
				<draggable v-model="available" class="ulisting-draggable-panel stm-min-height-100px stm-height-100" :options="{group:'attribute', handle:'.handle'}">
					<div class="ulisting-draggable-panel-items" v-for="(element, index) in available" :key="index">
						<div class="ulisting-draggable-panel-items-top">
							<div class="title">
								<span class="handle">
									<i class="fa fa-arrows"></i>
								</span>
								<p>{{element.name}} </p>
							</div>
						</div>
					</div>
				</draggable>
			</div>
		</div>
	</div>
	<input v-for="element in used" type="hidden" name="StmListingTypeSubnitForm[]" v-model="element.id">
</div>
