<?php

namespace uListing\Classes;

class StmAjaxAction {

    /**
     * @param string   $tag             The name of the action to which the $function_to_add is hooked.
     * @param callable $function_to_add The name of the function you wish to be called.
     * @param boolean  $nopriv          Optional. Boolean argument for adding wp_ajax_nopriv_action. Default false.
     * @param int      $priority        Optional. Used to specify the order in which the functions
     *                                  associated with a particular action are executed. Default 10.
     *                                  Lower numbers correspond with earlier execution,
     *                                  and functions with the same priority are executed
     *                                  in the order in which they were added to the action.
     * @param int      $accepted_args   Optional. The number of arguments the function accepts. Default 1.
     * @return true Will always return true.
     */
    public static function addAction($tag, $function_to_add, $nopriv = false, $priority = 10, $accepted_args = 1) {
        add_action('wp_ajax_'.$tag, $function_to_add, $priority = 10, $accepted_args = 1);
        if ( $nopriv ) add_action('wp_ajax_nopriv_'.$tag, $function_to_add);
        return true;
    }

    public static function init() {
        StmAjaxAction::addAction('stm_listing_login', [ StmListingAuth::class ,'stm_listing_login'], true);
        StmAjaxAction::addAction('stm_listing_register', [ StmListingAuth::class ,'stm_listing_register'], true);
        StmAjaxAction::addAction('stm_listing_profile_edit', [ StmListingAuth::class ,'stm_listing_profile_edit']);
        StmAjaxAction::addAction('stm_listing_ajax', [ StmListing::class ,'listing_ajax']);
        StmAjaxAction::addAction('stm_listing_file_ajax', [ StmListing::class ,'listing_file_ajax']);
        StmAjaxAction::addAction('stm_install_module', [ StmModules::class ,'ajax_settings_module']);
        StmAjaxAction::addAction('stm_update_attributes', [ StmListingType::class ,'stm_update_listing_type_attr']);
        StmAjaxAction::addAction('stm_export_current_layout', [ StmListingType::class ,'stm_export_current_layout_callback']);
        StmAjaxAction::addAction('stm_generate_default_pages', [ StmListingSettings::class ,'generateDefaultPages']);
        StmAjaxAction::addAction('stm_update_email_data', [ StmListingSettings::class ,'stm_update_email_data']);
        StmAjaxAction::addAction('stm_agencies_switcher', [ StmListingSettings::class ,'install_uninstall_ulisting_agencies']);
        StmAjaxAction::addAction('stm_settings_payment_method', [ StmPaymentMethod::class ,'ajax_settings_payment_method']);
        StmAjaxAction::addAction('stm_user_click', [ UlistingPageStatistics::class , 'page_statistics_for_user_phone_click'], true);
        StmAjaxAction::addAction('stm_listing_quick_view', [ StmListing::class ,'listing_quick_view_ajax'], true);

        $ajax_actions = apply_filters("ulisting_ajax", []);
        foreach ($ajax_actions as $ajax_action) {
            if(isset($ajax_action['is_admin']))
                StmAjaxAction::addAction($ajax_action['tag'], $ajax_action['action'], $ajax_action['is_admin']);
        }

        if(is_admin()) {
            StmAjaxAction::addAction('stm_attribute_option_save', [StmListingAttributeOption::class,'ajaxActionSave']);
            StmAjaxAction::addAction('stm_attribute_ajax_create', [StmListingAttribute::class,'ajaxActionCreate']);
            foreach ($ajax_actions as $ajax_action) {
                if(isset($ajax_action['is_admin']) AND $ajax_action['is_admin'])
                    StmAjaxAction::addAction($ajax_action['tag'], $ajax_action['action']);
            }
        }
    }
}