<?php

namespace uListing\Classes;

use uListing\Classes\StmCron;
use uListing\Admin\Classes\StmEmailTemplateManager;
use uListing\Classes\UlistingUserRole;
use WP_User_Query;

class StmListingSettings
{

    static $instance;
    public $object;

    const  ULISTING_PAGES = 'stm_listing_pages';
    const  ULISTING_OPEN_BY_HOVER = 'stm_open_by_hover';
    const  ULISTING_GOOGLE_API_SETTINGS = 'google_api_key';
    const  ULISTINGCURRENCY_SETTINGS = 'stm_currency_page';
    const  ULISTINGCURRENT_MAP_TYPE = 'stm_current_map_type';
    const  ULISTINGCURRENT_MAP_API_KEY = 'stm_current_map_api_key';
    const  ULISTING_DEFAULT_PLACEHOLDER = 'ulisting_default_placeholder';

    const  PAGE_LISTINGS_TYPE_PAGE = 'listing_type_page';
    const  PAGE_ACCOUNT_PAGE = 'account_page';
    const  PAGE_ADD_LISTING = 'add_listing';
    const  PAGE_PRICING_PLAN = 'pricing_plan';

    const  ULISTINGCURRENCY_POSITION_LEFT = 'left';
    const  ULISTINGCURRENCY_POSITION_RIGHT = 'right';
    const  ULISTINGCURRENCY_POSITION_LEFT_SPACE = 'left_space';
    const  ULISTINGCURRENCY_POSITION_RIGHT_SPACE = 'right_space';

    const ULISTINGMAP_GOOGLE = 'google';
    const ULISTINGMAP_OSM = 'osm';
    const ULISTINGMAP_MAPBOX = 'mapbox';


    const ULISTING_PAGES_STORE = [
        self::PAGE_ACCOUNT_PAGE => 'my-account',
        self::PAGE_ADD_LISTING  => 'add-new-property',
        self::PAGE_PRICING_PLAN => 'pricing-plan',
        'compare_page'          => 'compare',
        'wishlist_page'         => 'wishlist',
    ];

    public function __construct()
    {

    }

    public static function init()
    {
        $stmListingSettings = new StmListingSettings();
    }

    public static function getSettingsPanel()
    {
        $panels = [
            "main" => [
                "title" => __('Main', "ulisting"),
                "view" => ulisting_render_template(ULISTING_ADMIN_PATH . '/views/listing-settings/main.php', null),
            ],
            "pages" => [
                "title" => __('Pages', "ulisting"),
                "view" => ulisting_render_template(ULISTING_ADMIN_PATH . '/views/listing-settings/pages-panel.php', null),
            ],
            "inventory_layout" => [
                "title" => __('Inventory layout', "ulisting"),
                "view" => ulisting_render_template(ULISTING_ADMIN_PATH . '/views/listing-settings/inventory-layout.php', null),
            ],
            "user_roles" => [
                "title" => __('User roles', "ulisting"),
                "view" => ulisting_render_template(ULISTING_ADMIN_PATH . '/views/listing-settings/user-roles.php', null),
            ],
            "payment" => [
                "title" => __('Payment', "ulisting"),
                "view" => ulisting_render_template(ULISTING_ADMIN_PATH . '/views/listing-settings/payment-method.php', null),
            ],
            "email" => [
                "title" => __('Email', "ulisting"),
                "view" => ulisting_render_template(ULISTING_ADMIN_PATH . '/views/listing-settings/email-settings.php', null),
            ],
            "social_login" => [
                "title" => __('Social Login', "ulisting"),
                "view" => ulisting_render_template(ULISTING_ADMIN_PATH . '/views/listing-settings/social-login.php', null),
            ]

        ];
        return apply_filters('ulisting_settings_panels', $panels);
    }

    public static function render_settings()
    {
        if (isset($_POST['StmListingsSettings']['pages']))
            self::savePages(ulisting_sanitize_array($_POST['StmListingsSettings']['pages']));

        if (isset($_POST['StmListingsSettings']['currency_settings']))
            self::saveCurrency(ulisting_sanitize_array($_POST['StmListingsSettings']['currency_settings']));

        if (isset($_POST['StmListingsSettings']['ulisting_category_limit']))
            update_option("ulisting_category_limit", sanitize_text_field($_POST['StmListingsSettings']['ulisting_category_limit']));

        if (isset($_POST['StmListingsSettings']['ulisting_feature_limit']))
            update_option("ulisting_feature_limit", sanitize_text_field($_POST['StmListingsSettings']['ulisting_feature_limit']));

        if (isset($_POST['StmListingsSettings']['ulisting_remove_tables']))
            update_option("ulisting_remove_tables", sanitize_text_field($_POST['StmListingsSettings']['ulisting_remove_tables']));

        if (isset($_POST['StmListingsSettings']['allow_delete_listings']))
            update_option("allow_delete_listings", sanitize_text_field($_POST['StmListingsSettings']['allow_delete_listings']));

        if (isset($_POST['StmListingsSettings']['ulisting_back_slots']))
            update_option("ulisting_back_slots", sanitize_text_field($_POST['StmListingsSettings']['ulisting_back_slots']));

        if (isset($_POST['StmListingsSettings']['map']['open_by_hover'])){
            update_option(self::ULISTING_OPEN_BY_HOVER, sanitize_text_field($_POST['StmListingsSettings']['map']['open_by_hover']));
        }       if (isset($_POST['StmListingsSettings']['map']['open_by_hover'])){
            update_option(self::ULISTING_OPEN_BY_HOVER, sanitize_text_field($_POST['StmListingsSettings']['map']['open_by_hover']));
        }

        if (isset($_POST['StmListingsSettings']['map']['type'])){
            update_option(self::ULISTINGCURRENT_MAP_TYPE, sanitize_text_field($_POST['StmListingsSettings']['map']['type']));
            if($_POST['StmListingsSettings']['map']['type'] === 'google' && isset($_POST['StmListingsSettings']['map']['api_key']['google']))
                update_option(self::ULISTING_GOOGLE_API_SETTINGS, sanitize_text_field($_POST['StmListingsSettings']['map']['api_key']['google']));
        }


        if (isset($_POST['StmListingsSettings']['map']['api_key']))
            update_option(self::ULISTINGCURRENT_MAP_API_KEY, ulisting__sanitize_array($_POST['StmListingsSettings']['map']['api_key']));

        if (isset($_POST['StmListingsSettings']['default_placeholder'])) {
            update_option(self::ULISTING_DEFAULT_PLACEHOLDER, sanitize_text_field($_POST['StmListingsSettings']['default_placeholder']));
        }

        do_action('ulisting_settings_save', ulisting_sanitize_array($_POST));

        ulisting_render_template(ULISTING_ADMIN_PATH . '/views/listing-settings/listing-settings.php', ['panels' => self::getSettingsPanel()], true);
    }

    /**
     * Get Map Open by hover
     * @return string
     */
    public static function getMapHover()
    {
        if(!get_option(self::ULISTING_OPEN_BY_HOVER))
            update_option(self::ULISTING_OPEN_BY_HOVER, 'yes');

        return get_option(self::ULISTING_OPEN_BY_HOVER);
    }

    /**
     * @param $settings
     */
    public static function saveCurrency($settings)
    {
        foreach ($settings as $key => $val) {
            $settings[$key] = $val;
        }
        update_option(self::ULISTINGCURRENCY_SETTINGS, $settings);
    }

    /**
     * Save pages settings
     * @param $data
     */
    public static function savePages($data)
    {
        foreach ($data as $key => $val) {
            if (!is_array($val))
                $data[$key] = sanitize_text_field($val);
            else {
                foreach ($val as $k => $v) {
                    $data[$key][$k] = sanitize_text_field($v);
                }
            }
        }
        update_option(self::ULISTING_PAGES, $data);
    }

    /**
     * @param null $page page code name
     *
     * @return mixed id page or null
     */
    public static function getPages($page = null)
    {
        $pages = get_option(self::ULISTING_PAGES);
        if ($page AND isset($pages[$page]))
            return $pages[$page];
        if (!$page)
            return $pages;
        return [];
    }

    /**
     * Update Email Template Images by key
     */
    public static function stm_update_email_data() {
    	if ( ! current_user_can( 'manage_options' ) || ! StmVerifyNonce::verifyAjaxNonce() ) {
    		return false;
    	}

        $result = [
            'status'  => 'success',
            'success' => true
        ];

        $request_body = file_get_contents('php://input');
        $request_data = json_decode($request_body, true);

        if (isset($request_data['socials'])) {
            update_option('ulisting_email_socials', $request_data['socials']);
        }

        if ( isset($request_data['images']) && is_array($request_data['images']) ) {
	        if ( array_key_exists('ulisting_email_banner', $request_data['images']) ) {
		        update_option('ulisting_email_banner', sanitize_text_field($request_data['images']['ulisting_email_banner']));
	        }
	        if ( array_key_exists('ulisting_email_logo', $request_data['images']) ) {
		        update_option('ulisting_email_logo', sanitize_text_field($request_data['images']['ulisting_email_logo']));
	        }
        }
        wp_send_json($result);
    }

    public static function generateDefaultPages()
    {
        $result = [
            'success' => false,
            'message' => "Cannot generate pages"
        ];

        if (isset($_POST['StmListingsSettings']) && !empty($_POST['StmListingsSettings']['pages'])) {

            $pages = apply_filters('ulisting_sanitize_array', $_POST['StmListingsSettings']['pages']);
            self::savePages($pages);

            foreach ($pages as $page_key => $page_value) {
                if (!is_array($page_value)) {
                    $page__id = StmListingSettings::getPages($page_key);
                    if (!$page__id) {
                        $page_name = self::ULISTING_PAGES_STORE[$page_key];
                        $posts = get_posts($args = array(
                            'name' => $page_name,
                            'post_type' => 'page',
                            'post_status' => 'publish',
                            'posts_per_page' => 1
                        ));

                        if (count($posts) > 0)
                            $pages[$page_key] = $posts[0]->ID;
                        else
                            $pages[$page_key] = self::create_page($page_key);
                    }
                } elseif($page_key !== 'account_endpoint') {

                    if(count($page_value) > 0)
                        foreach ($page_value as $type_id => $page_id ){
                            $page_post = get_post($page_id);
                            if(!$page_post){
                                $title = get_the_title($type_id);

                                $posts = get_posts($args = array(
                                    'title' => $title,
                                    'post_type' => 'page',
                                    'post_status' => 'publish',
                                    'posts_per_page' => 1
                                ));

                                if(count($posts) > 0)
                                    $page_value[$type_id] = $posts[0]->ID;
                                else
                                    $page_value[$type_id] = self::create_page($page_key, $title);

                                $pages[$page_key] = $page_value;
                            }
                        }
                }
            }

            $result['success'] = true;
            $result['message'] = "Pages generated successfully";
            self::savePages($pages);
        }

        wp_send_json($result);
    }

    public static function create_page($key, $page_title = '')
    {
        $post_names = [];
        $post_name = self::ULISTING_PAGES_STORE[$key];

        if(!empty($page_title)) $post_name = $page_title;

        if(is_array(explode('-', $post_name))){
            foreach ( explode('-', $post_name) as $value )
                $post_names[] = ucfirst($value);
            $title = ucfirst(join(" ", $post_names));
        }else {
            $title = ucfirst($post_name);
        }

        $post_id = wp_insert_post(
            array(
                'post_author'    => 1,
                'post_title'     => $title,
                'post_name'      => $post_name,
                'post_status'    => 'publish',
                'post_type'      => 'page',
            )
        );

        return $post_id;
    }

    /**
     * @return string
     */
    public static function getAddListingPageUrl()
    {
        return get_page_link(StmListingSettings::getPages(StmListingSettings::PAGE_ADD_LISTING));
    }

    /**
     * @param null $params
     *
     * @return object
     */
    public static function getCurrency($params = null)
    {
        $default = [
            'currency' => '',
            'position' => null,
            'thousands_separator' => '',
            'decimal_separator' => '',
            'characters_after' => 0,
        ];
        $currency_settings = (object)get_option(self::ULISTINGCURRENCY_SETTINGS, $default);
        if ($params)
            return $currency_settings->$params;
        return $currency_settings;
    }

    /**
     * @return array
     */
    public static function getCurrencyPositionList()
    {
        return array(
            self::ULISTINGCURRENCY_POSITION_LEFT => __('Left', "ulisting"),
            self::ULISTINGCURRENCY_POSITION_RIGHT => __('Right', "ulisting"),
            self::ULISTINGCURRENCY_POSITION_LEFT_SPACE => __('Left space', "ulisting"),
            self::ULISTINGCURRENCY_POSITION_RIGHT_SPACE => __('Right space', "ulisting"),
        );
    }

    /**
     * @return array
     */
    public static function getMaps()
    {
        return array(
            self::ULISTINGMAP_GOOGLE => __('Google', "ulisting"),
            self::ULISTINGMAP_OSM => __('OpenStreetMap', "ulisting"),
            self::ULISTINGMAP_MAPBOX => __('MapBox', "ulisting"),
        );
    }

    /**
     * @param null $currency
     *
     * @return array|mixed
     */
    public static function get_stm_currencies($currency = null)
    {
        $currencies = array('AED' => __('United Arab Emirates dirham', 'ulisting'),
            'AFN' => __('Afghan afghani', 'ulisting'),
            'ALL' => __('Albanian lek', 'ulisting'),
            'AMD' => __('Armenian dram', 'ulisting'),
            'ANG' => __('Netherlands Antillean guilder', 'ulisting'),
            'AOA' => __('Angolan kwanza', 'ulisting'),
            'ARS' => __('Argentine peso', 'ulisting'),
            'AUD' => __('Australian dollar', 'ulisting'),
            'AWG' => __('Aruban florin', 'ulisting'),
            'AZN' => __('Azerbaijani manat', 'ulisting'),
            'BAM' => __('Bosnia and Herzegovina convertible mark', 'ulisting'),
            'BBD' => __('Barbadian dollar', 'ulisting'),
            'BDT' => __('Bangladeshi taka', 'ulisting'),
            'BGN' => __('Bulgarian lev', 'ulisting'),
            'BHD' => __('Bahraini dinar', 'ulisting'),
            'BIF' => __('Burundian franc', 'ulisting'),
            'BMD' => __('Bermudian dollar', 'ulisting'),
            'BND' => __('Brunei dollar', 'ulisting'),
            'BOB' => __('Bolivian boliviano', 'ulisting'),
            'BRL' => __('Brazilian real', 'ulisting'),
            'BSD' => __('Bahamian dollar', 'ulisting'),
            'BTC' => __('Bitcoin', 'ulisting'),
            'BTN' => __('Bhutanese ngultrum', 'ulisting'),
            'BWP' => __('Botswana pula', 'ulisting'),
            'BYR' => __('Belarusian ruble (old)', 'ulisting'),
            'BYN' => __('Belarusian ruble', 'ulisting'),
            'BZD' => __('Belize dollar', 'ulisting'),
            'CAD' => __('Canadian dollar', 'ulisting'),
            'CDF' => __('Congolese franc', 'ulisting'),
            'CHF' => __('Swiss franc', 'ulisting'),
            'CLP' => __('Chilean peso', 'ulisting'),
            'CNY' => __('Chinese yuan', 'ulisting'),
            'COP' => __('Colombian peso', 'ulisting'),
            'CRC' => __('Costa Rican col&oacute;n', 'ulisting'),
            'CUC' => __('Cuban convertible peso', 'ulisting'),
            'CUP' => __('Cuban peso', 'ulisting'),
            'CVE' => __('Cape Verdean escudo', 'ulisting'),
            'CZK' => __('Czech koruna', 'ulisting'),
            'DJF' => __('Djiboutian franc', 'ulisting'),
            'DKK' => __('Danish krone', 'ulisting'),
            'DOP' => __('Dominican peso', 'ulisting'),
            'DZD' => __('Algerian dinar', 'ulisting'),
            'EGP' => __('Egyptian pound', 'ulisting'),
            'ERN' => __('Eritrean nakfa', 'ulisting'),
            'ETB' => __('Ethiopian birr', 'ulisting'),
            'EUR' => __('Euro', 'ulisting'),
            'FJD' => __('Fijian dollar', 'ulisting'),
            'FKP' => __('Falkland Islands pound', 'ulisting'),
            'GBP' => __('Pound sterling', 'ulisting'),
            'GEL' => __('Georgian lari', 'ulisting'),
            'GGP' => __('Guernsey pound', 'ulisting'),
            'GHS' => __('Ghana cedi', 'ulisting'),
            'GIP' => __('Gibraltar pound', 'ulisting'),
            'GMD' => __('Gambian dalasi', 'ulisting'),
            'GNF' => __('Guinean franc', 'ulisting'),
            'GTQ' => __('Guatemalan quetzal', 'ulisting'),
            'GYD' => __('Guyanese dollar', 'ulisting'),
            'HKD' => __('Hong Kong dollar', 'ulisting'),
            'HNL' => __('Honduran lempira', 'ulisting'),
            'HRK' => __('Croatian kuna', 'ulisting'),
            'HTG' => __('Haitian gourde', 'ulisting'),
            'HUF' => __('Hungarian forint', 'ulisting'),
            'IDR' => __('Indonesian rupiah', 'ulisting'),
            'ILS' => __('Israeli new shekel', 'ulisting'),
            'IMP' => __('Manx pound', 'ulisting'),
            'INR' => __('Indian rupee', 'ulisting'),
            'IQD' => __('Iraqi dinar', 'ulisting'),
            'IRR' => __('Iranian rial', 'ulisting'),
            'IRT' => __('Iranian toman', 'ulisting'),
            'ISK' => __('Icelandic kr&oacute;na', 'ulisting'),
            'JEP' => __('Jersey pound', 'ulisting'),
            'JMD' => __('Jamaican dollar', 'ulisting'),
            'JOD' => __('Jordanian dinar', 'ulisting'),
            'JPY' => __('Japanese yen', 'ulisting'),
            'KES' => __('Kenyan shilling', 'ulisting'),
            'KGS' => __('Kyrgyzstani som', 'ulisting'),
            'KHR' => __('Cambodian riel', 'ulisting'),
            'KMF' => __('Comorian franc', 'ulisting'),
            'KPW' => __('North Korean won', 'ulisting'),
            'KRW' => __('South Korean won', 'ulisting'),
            'KWD' => __('Kuwaiti dinar', 'ulisting'),
            'KYD' => __('Cayman Islands dollar', 'ulisting'),
            'KZT' => __('Kazakhstani tenge', 'ulisting'),
            'LAK' => __('Lao kip', 'ulisting'),
            'LBP' => __('Lebanese pound', 'ulisting'),
            'LKR' => __('Sri Lankan rupee', 'ulisting'),
            'LRD' => __('Liberian dollar', 'ulisting'),
            'LSL' => __('Lesotho loti', 'ulisting'),
            'LYD' => __('Libyan dinar', 'ulisting'),
            'MAD' => __('Moroccan dirham', 'ulisting'),
            'MDL' => __('Moldovan leu', 'ulisting'),
            'MGA' => __('Malagasy ariary', 'ulisting'),
            'MKD' => __('Macedonian denar', 'ulisting'),
            'MMK' => __('Burmese kyat', 'ulisting'),
            'MNT' => __('Mongolian t&ouml;gr&ouml;g', 'ulisting'),
            'MOP' => __('Macanese pataca', 'ulisting'),
            'MRO' => __('Mauritanian ouguiya', 'ulisting'),
            'MUR' => __('Mauritian rupee', 'ulisting'),
            'MVR' => __('Maldivian rufiyaa', 'ulisting'),
            'MWK' => __('Malawian kwacha', 'ulisting'),
            'MXN' => __('Mexican peso', 'ulisting'),
            'MYR' => __('Malaysian ringgit', 'ulisting'),
            'MZN' => __('Mozambican metical', 'ulisting'),
            'NAD' => __('Namibian dollar', 'ulisting'),
            'NGN' => __('Nigerian naira', 'ulisting'),
            'NIO' => __('Nicaraguan c&oacute;rdoba', 'ulisting'),
            'NOK' => __('Norwegian krone', 'ulisting'),
            'NPR' => __('Nepalese rupee', 'ulisting'),
            'NZD' => __('New Zealand dollar', 'ulisting'),
            'OMR' => __('Omani rial', 'ulisting'),
            'PAB' => __('Panamanian balboa', 'ulisting'),
            'PEN' => __('Peruvian nuevo sol', 'ulisting'),
            'PGK' => __('Papua New Guinean kina', 'ulisting'),
            'PHP' => __('Philippine peso', 'ulisting'),
            'PKR' => __('Pakistani rupee', 'ulisting'),
            'PLN' => __('Polish z&#x142;oty', 'ulisting'),
            'PRB' => __('Transnistrian ruble', 'ulisting'),
            'PYG' => __('Paraguayan guaran&iacute;', 'ulisting'),
            'QAR' => __('Qatari riyal', 'ulisting'),
            'RON' => __('Romanian leu', 'ulisting'),
            'RSD' => __('Serbian dinar', 'ulisting'),
            'RUB' => __('Russian ruble', 'ulisting'),
            'RWF' => __('Rwandan franc', 'ulisting'),
            'SAR' => __('Saudi riyal', 'ulisting'),
            'SBD' => __('Solomon Islands dollar', 'ulisting'),
            'SCR' => __('Seychellois rupee', 'ulisting'),
            'SDG' => __('Sudanese pound', 'ulisting'),
            'SEK' => __('Swedish krona', 'ulisting'),
            'SGD' => __('Singapore dollar', 'ulisting'),
            'SHP' => __('Saint Helena pound', 'ulisting'),
            'SLL' => __('Sierra Leonean leone', 'ulisting'),
            'SOS' => __('Somali shilling', 'ulisting'),
            'SRD' => __('Surinamese dollar', 'ulisting'),
            'SSP' => __('South Sudanese pound', 'ulisting'),
            'STD' => __('S&atilde;o Tom&eacute; and Pr&iacute;ncipe dobra', 'ulisting'),
            'SYP' => __('Syrian pound', 'ulisting'),
            'SZL' => __('Swazi lilangeni', 'ulisting'),
            'THB' => __('Thai baht', 'ulisting'),
            'TJS' => __('Tajikistani somoni', 'ulisting'),
            'TMT' => __('Turkmenistan manat', 'ulisting'),
            'TND' => __('Tunisian dinar', 'ulisting'),
            'TOP' => __('Tongan pa&#x2bb;anga', 'ulisting'),
            'TRY' => __('Turkish lira', 'ulisting'),
            'TTD' => __('Trinidad and Tobago dollar', 'ulisting'),
            'TWD' => __('New Taiwan dollar', 'ulisting'),
            'TZS' => __('Tanzanian shilling', 'ulisting'),
            'UAH' => __('Ukrainian hryvnia', 'ulisting'),
            'UGX' => __('Ugandan shilling', 'ulisting'),
            'USD' => __('United States (US) dollar', 'ulisting'),
            'UYU' => __('Uruguayan peso', 'ulisting'),
            'UZS' => __('Uzbekistani som', 'ulisting'),
            'VEF' => __('Venezuelan bol&iacute;var', 'ulisting'),
            'VND' => __('Vietnamese &#x111;&#x1ed3;ng', 'ulisting'),
            'VUV' => __('Vanuatu vatu', 'ulisting'),
            'WST' => __('Samoan t&#x101;l&#x101;', 'ulisting'),
            'XAF' => __('Central African CFA franc', 'ulisting'),
            'XCD' => __('East Caribbean dollar', 'ulisting'),
            'XOF' => __('West African CFA franc', 'ulisting'),
            'XPF' => __('CFP franc', 'ulisting'),
            'YER' => __('Yemeni rial', 'ulisting'),
            'ZAR' => __('South African rand', 'ulisting'),
            'ZMW' => __('Zambian kwacha', 'ulisting')
        );
        if ($currency)
            return $currencies[$currency];
        return $currencies;
    }

    /**
     * @param null $currency
     *
     * @return array|mixed
     */
    public static function get_stm_currency_symbol($currency = null)
    {
        $currency_symbol = array(
            'AED' => '&#x62f;.&#x625;',
            'AFN' => '&#x60b;',
            'ALL' => 'L',
            'AMD' => 'AMD',
            'ANG' => '&fnof;',
            'AOA' => 'Kz',
            'ARS' => '&#36;',
            'AUD' => '&#36;',
            'AWG' => 'Afl.',
            'AZN' => 'AZN',
            'BAM' => 'KM',
            'BBD' => '&#36;',
            'BDT' => '&#2547;&nbsp;',
            'BGN' => '&#1083;&#1074;.',
            'BHD' => '.&#x62f;.&#x628;',
            'BIF' => 'Fr',
            'BMD' => '&#36;',
            'BND' => '&#36;',
            'BOB' => 'Bs.',
            'BRL' => '&#82;&#36;',
            'BSD' => '&#36;',
            'BTC' => '&#3647;',
            'BTN' => 'Nu.',
            'BWP' => 'P',
            'BYR' => 'Br',
            'BYN' => 'Br',
            'BZD' => '&#36;',
            'CAD' => '&#36;',
            'CDF' => 'Fr',
            'CHF' => '&#67;&#72;&#70;',
            'CLP' => '&#36;',
            'CNY' => '&yen;',
            'COP' => '&#36;',
            'CRC' => '&#x20a1;',
            'CUC' => '&#36;',
            'CUP' => '&#36;',
            'CVE' => '&#36;',
            'CZK' => '&#75;&#269;',
            'DJF' => 'Fr',
            'DKK' => 'DKK',
            'DOP' => 'RD&#36;',
            'DZD' => '&#x62f;.&#x62c;',
            'EGP' => 'EGP',
            'ERN' => 'Nfk',
            'ETB' => 'Br',
            'EUR' => '&euro;',
            'FJD' => '&#36;',
            'FKP' => '&pound;',
            'GBP' => '&pound;',
            'GEL' => '&#x20be;',
            'GGP' => '&pound;',
            'GHS' => '&#x20b5;',
            'GIP' => '&pound;',
            'GMD' => 'D',
            'GNF' => 'Fr',
            'GTQ' => 'Q',
            'GYD' => '&#36;',
            'HKD' => '&#36;',
            'HNL' => 'L',
            'HRK' => 'Kn',
            'HTG' => 'G',
            'HUF' => '&#70;&#116;',
            'IDR' => 'Rp',
            'ILS' => '&#8362;',
            'IMP' => '&pound;',
            'INR' => '&#8377;',
            'IQD' => '&#x639;.&#x62f;',
            'IRR' => '&#xfdfc;',
            'IRT' => '&#x062A;&#x0648;&#x0645;&#x0627;&#x0646;',
            'ISK' => 'kr.',
            'JEP' => '&pound;',
            'JMD' => '&#36;',
            'JOD' => '&#x62f;.&#x627;',
            'JPY' => '&yen;',
            'KES' => 'KSh',
            'KGS' => '&#x441;&#x43e;&#x43c;',
            'KHR' => '&#x17db;',
            'KMF' => 'Fr',
            'KPW' => '&#x20a9;',
            'KRW' => '&#8361;',
            'KWD' => '&#x62f;.&#x643;',
            'KYD' => '&#36;',
            'KZT' => 'KZT',
            'LAK' => '&#8365;',
            'LBP' => '&#x644;.&#x644;',
            'LKR' => '&#xdbb;&#xdd4;',
            'LRD' => '&#36;',
            'LSL' => 'L',
            'LYD' => '&#x644;.&#x62f;',
            'MAD' => '&#x62f;.&#x645;.',
            'MDL' => 'MDL',
            'MGA' => 'Ar',
            'MKD' => '&#x434;&#x435;&#x43d;',
            'MMK' => 'Ks',
            'MNT' => '&#x20ae;',
            'MOP' => 'P',
            'MRO' => 'UM',
            'MUR' => '&#x20a8;',
            'MVR' => '.&#x783;',
            'MWK' => 'MK',
            'MXN' => '&#36;',
            'MYR' => '&#82;&#77;',
            'MZN' => 'MT',
            'NAD' => '&#36;',
            'NGN' => '&#8358;',
            'NIO' => 'C&#36;',
            'NOK' => '&#107;&#114;',
            'NPR' => '&#8360;',
            'NZD' => '&#36;',
            'OMR' => '&#x631;.&#x639;.',
            'PAB' => 'B/.',
            'PEN' => 'S/.',
            'PGK' => 'K',
            'PHP' => '&#8369;',
            'PKR' => '&#8360;',
            'PLN' => '&#122;&#322;',
            'PRB' => '&#x440;.',
            'PYG' => '&#8370;',
            'QAR' => '&#x631;.&#x642;',
            'RMB' => '&yen;',
            'RON' => 'lei',
            'RSD' => '&#x434;&#x438;&#x43d;.',
            'RUB' => '&#8381;',
            'RWF' => 'Fr',
            'SAR' => '&#x631;.&#x633;',
            'SBD' => '&#36;',
            'SCR' => '&#x20a8;',
            'SDG' => '&#x62c;.&#x633;.',
            'SEK' => '&#107;&#114;',
            'SGD' => '&#36;',
            'SHP' => '&pound;',
            'SLL' => 'Le',
            'SOS' => 'Sh',
            'SRD' => '&#36;',
            'SSP' => '&pound;',
            'STD' => 'Db',
            'SYP' => '&#x644;.&#x633;',
            'SZL' => 'L',
            'THB' => '&#3647;',
            'TJS' => '&#x405;&#x41c;',
            'TMT' => 'm',
            'TND' => '&#x62f;.&#x62a;',
            'TOP' => 'T&#36;',
            'TRY' => '&#8378;',
            'TTD' => '&#36;',
            'TWD' => '&#78;&#84;&#36;',
            'TZS' => 'Sh',
            'UAH' => '&#8372;',
            'UGX' => 'UGX',
            'USD' => '&#36;',
            'UYU' => '&#36;',
            'UZS' => 'UZS',
            'VEF' => 'Bs F',
            'VND' => '&#8363;',
            'VUV' => 'Vt',
            'WST' => 'T',
            'XAF' => 'CFA',
            'XCD' => '&#36;',
            'XOF' => 'CFA',
            'XPF' => 'Fr',
            'YER' => '&#xfdfc;',
            'ZAR' => '&#82;',
            'ZMW' => 'ZK',
        );
        if ($currency)
            return $currency_symbol[$currency];
        return $currency_symbol;
    }

    /**
     * @return mixed|void
     */
    public static function get_account_endpoint()
    {
        return StmUser::get_account_endpoint();
    }

    public static function get_google_api()
    {
        $google_api = get_option(self::ULISTING_GOOGLE_API_SETTINGS, "");
        return $google_api;
    }

    public static function separate_api_keys()
    {
        $google_api = self::get_google_api();
        if (empty($google_api))
            return ['google' => ''];
        return ['google' => $google_api];
    }


    /**
     * @return array
     */
    public static function get_current_map_api_key()
    {
        $current_map_api_key = get_option(self::ULISTINGCURRENT_MAP_API_KEY, "");
        if (empty($current_map_api_key))
            return self::separate_api_keys();

        return $current_map_api_key;
    }

    /**
     * @return string
     */
    public static function get_current_map_type()
    {
        $type = get_option(self::ULISTINGCURRENT_MAP_TYPE, "");

        if (empty($type)) {
            $google = self::separate_api_keys();
            $api_key = !empty($google['google']) ? $google['google'] : '';

            update_option(self::ULISTINGCURRENT_MAP_API_KEY, sanitize_text_field($api_key));
            update_option(self::ULISTINGCURRENT_MAP_TYPE, sanitize_text_field(self::ULISTINGMAP_GOOGLE));

            $type = self::ULISTINGMAP_GOOGLE;
        }

        return $type;
    }

	/**
	 * @param $id
	 * @return bool|object
	 */
	public static function get_wpml_default_language_page($id) {
		if (class_exists('SitePress')) {
			global $sitepress;
			if ( defined('ICL_LANGUAGE_CODE') && ICL_LANGUAGE_CODE != $sitepress->get_default_language()) {
				if ($id) {
					$page = get_post( $id );
				} else {
					$page = (object) array(
						'ID' => 0,
						'post_title' => esc_html("-- No page --", "ulisting"),
					);
				}
				return $page;
			}
		}
		return false;
	}

	/**
	 * @return bool
	 */
	public static function is_default_language() {
		if (class_exists('SitePress')) {
			global $sitepress;
			if (ICL_LANGUAGE_CODE != $sitepress->get_default_language()) {
				return false;
			}
		}
		return true;
	}

    /**
     * install/uninstall agency roles
     */
    public static function install_uninstall_ulisting_agencies()
    {
        $result = array(
            'errors' => [],
            'message' => null,
            'status'  => 'error'
        );
        
        if ( isset($_POST['StmAgenciesSwitcher']['mode']) && $mode = $_POST['StmAgenciesSwitcher']['mode'] ) {
            if ( $mode == 'install' ) {
                if (!get_role("agency")) {
                    add_role("agency", "Agency", [
                        "default" => 0,
                        "listing_limit" => "0",
                        "comment" => "1",
                        "listing_moderation" => "1",
                        "stm_listing_role" => "1",
                    ]);
                }
                if (!get_role("agent")) {
                    add_role("agent", "Agent", [
                        "default" => 0,
                        "listing_limit" => "0",
                        "comment" => "1",
                        "listing_moderation" => "0",
                        "stm_listing_role" => "1",
                        "stm_listing_role_hidden" => "1",
                    ]);
                }
                $result['status'] = 'success';
                $result['message'] = esc_html__('Installing completed successfully.', "ulisting");
            } elseif ( $mode == "uninstall" ) {
                $user_query = new WP_User_Query( array(
                    'number' => -1,
                    'role__in' => array( 'agency', 'agent' )
                ) );
                if ( !empty( $user_query->results ) ) {
                    foreach ( $user_query->results as $user ) {
                        $user->set_role( 'user' );
                    }
                }
                remove_role("agency");
                remove_role("agent");
                $result['status'] = 'success';
                $result['message'] = esc_html__('Uninstalling completed successfully.', "ulisting");
            }
        }

        wp_send_json($result);
        die;
    }

    /**
     * install default currency ulisting category limit
     */
    public static function install_default_ulisting_category_limit()
    {
        if (!get_option("ulisting_category_limit")) {
            update_option("ulisting_category_limit", 5);
        }
    }

    /**
     * install default currency ulisting feature limit
     */
    public static function install_default_ulisting_feature_limit()
    {
        if (!get_option("ulisting_feature_limit")) {
            update_option("ulisting_feature_limit", 5);
        }
    }

    /**
     * install default currency
     */
    public static function install_default_currency()
    {
        if (!get_option("stm_currency_page")) {
            $default_settings = [
                "currency" => "USD",
                "position" => "left_space",
                "thousands_separator" => ",",
                "decimal_separator" => ".",
                "characters_after" => "2",
            ];
            update_option(self::ULISTINGCURRENCY_SETTINGS, $default_settings);
        }
    }

    public static function set_plugin_version() {
        if (empty(get_option('ulisting-version')))
            update_option('ulisting-version', ULISTING_VERSION);
    }

    public static function set_plugin_db_version() {
        if (empty(get_option('ulisting-db-version')))
            update_option('ulisting-db-version', ULISTING_DB_VERSION);
    }

    public static function install_default_email_templates() {
        if (empty(StmEmailTemplateManager::get_email_templates_store())) {
            StmEmailTemplateManager::update_email_templates_store(StmEmailTemplateManager::email_templates_list());
        }
    }

    public static function email_socials() {
        if (empty(get_option(StmEmailTemplateManager::SOCIAL_OPTION)))
            update_option(StmEmailTemplateManager::SOCIAL_OPTION, StmEmailTemplateManager::get_socials());
    }

    public static function set_pricing_plan_settings() {
        update_option('allow_delete_listings', false);
        update_option('ulisting_back_slots', false);
    }

    /**
     * Install default settings
     */
    public static function install_default_settings()
    {
        self::set_plugin_version();
        self::set_plugin_db_version();
        self::install_default_currency();
        self::set_pricing_plan_settings();
        self::install_default_ulisting_category_limit();
        self::install_default_ulisting_feature_limit();
        self::install_default_email_templates();
    }
}
