<?php

namespace uListing\Classes;

use uListing\Admin\Classes\StmEmailTemplateManager;

/**
 * Class StmUpdates
 * @package uListing\Classes
 */
class StmUpdates {
    private static $updates = [
        '1.0.2' => [
            'update_email_templates',
        ],

        '1.0.3' => [
            'update_email_templates',
        ],
    ];

    public static function init() {
        if (version_compare( get_option( 'ulisting-version' ), ULISTING_VERSION, '<' ) )
            self::update_version();
    }

    public static function get_updates() {
        return self::$updates;
    }

    public static function needs_to_update() {
        $current_db_version = get_option( 'ulisting-db-version', 1 );
        $update_versions    = array_keys( self::get_updates() );
        usort( $update_versions, 'version_compare' );
        return ! is_null( $current_db_version ) && version_compare( $current_db_version, end( $update_versions ), '<' );
    }

    private static function maybe_update_db_version() {
        if ( self::needs_to_update() ) {
            $updates = self::get_updates();
            $db_version = get_option('ulisting-db-version');

            foreach ( $updates as $version => $callback_arr)
                if ( version_compare( $db_version, $version, '<' ))
                    foreach ($callback_arr as $callback)
                        call_user_func( [self::class, $callback] );
            return;
        }
        update_option('ulisting-db-version', ULISTING_DB_VERSION, true);
    }

    public static function update_version() {
        update_option('ulisting-version', ULISTING_VERSION, true);
        self::maybe_update_db_version();
    }

    public static function update_email_templates() {
        if (empty(get_option(StmEmailTemplateManager::EMAIL_OPTION)))
            StmEmailTemplateManager::update_email_templates_store(StmEmailTemplateManager::email_templates_list());

        if (!empty(get_option('ulisting_saved_searches'))) {
            $saved_search = get_option('ulisting_saved_searches');
            $new_saved_search = StmEmailTemplateManager::find_by_slug('saved-search');
            $new_saved_search['subject'] = $saved_search['subject'];
            $new_saved_search['content'] = $saved_search['content'];
            StmEmailTemplateManager::update_by_slug('saved-search', $new_saved_search);
        }

        if (empty(get_option(StmEmailTemplateManager::SOCIAL_OPTION)))
            update_option(StmEmailTemplateManager::SOCIAL_OPTION, StmEmailTemplateManager::get_socials());

        update_option('admin_email', bloginfo('admin_email'));
    }
}