<?php

use uListing\Lib\PayPal\Classes\PayPal;
use uListing\Lib\PricingPlan\Classes\StmUserPlan;
use uListing\Lib\PricingPlan\Classes\StmPricingPlans;
$paypal = new PayPal();
$pricing_plans = StmPricingPlans::query()
                              ->where('post_type','stm_pricing_plans')
                              ->where('post_status','publish')
                              ->find();

$stm_paypal_settings_data = [];
$stm_paypal_settings_data['modes'] = PayPal::getMode();
$stm_paypal_settings_data['mode_selected'] = (isset($data['mode'])) ? $data['mode'] : PayPal::MODE_SANDBOX;

$stm_paypal_settings_data['access_update_account'] = StmUserPlan::get_active_plans_count();

foreach ($pricing_plans as $plan) {
	$plan_data       = $plan->getData();
	$plan->plan_data = $plan;
	$plan->post_content = '';
	if($plan_data['payment_type'] == StmPricingPlans::PRICING_PLANS_PAYMENT_TYPE_SUBSCRIPTION AND $plan_data['status'] == StmPricingPlans::STATUS_ACTIVE) {
		$stm_paypal_settings_data['pricing_plans'][] = array(
			'id'             => $plan->ID,
			'name'           => $plan->post_title,
			'paypal_plan_id' => PayPal::getPayPalPlanId($plan->ID),
		);
	}
}

if(!isset($data['client_id']))
	$data['client_id'] = "";

if(!isset($data['client_secret']))
	$data['client_secret'] = "";

if(!isset($data['webhook_id']))
	$data['webhook_id'] = "";

wp_add_inline_script('stm-paypal-settings', "var stm_paypal_settings_data = json_parse('". ulisting_convert_content(json_encode($stm_paypal_settings_data)) ."');", 'before');
?>

<?php if(!PayPal::checkCurrency()):?>
	<div class="panel-custom p-t-30 p-b-30">
		<div class="p-t-30 p-b-30 p-l-30 p-r-30 ulisting-bg-white">
			<div class="alert alert-danger" role="alert"><?php esc_html_e('Gateway disabled: PayPal does not support your store currency.', "ulisting")?></div>
		</div>
	</div>
<?php else:?>
	<div id="stm-paypal-settings"  class="panel-custom ">
		<h3><?php esc_html_e("PayPal Settings", "ulisting")?></h3>
		<div class="ulisting-main form-horizontal  p-b-30 p-l-30 p-r-30 ulisting-bg-white">

			<?php if(!ulisting_is_https()):?>
				<div class="alert alert-danger">
					<?php esc_html_e('Subscriptio Paypal payment gateway requires full SSL support and enforcement during Checkout. Only test mode will work until this is solved.')?>
				</div>
			<?php endif;?>

			<form method="post">
				<div class="form-group">
					<label class="col-xs-12 col-sm-2 "><?php esc_html_e("Client ID", "ulisting")?></label>
					<div class="col-xs-12 col-sm-6">
                        <input :disabled="access_update_account !== 0"  type="text" class="form-control"  name="StmPaypal[client_id]" value="<?php echo esc_attr($data['client_id'])?>">
					</div>
				</div>

				<div class="form-group">
					<label class="col-xs-12 col-sm-2 "><?php esc_html_e("Client Secret", "ulisting")?></label>
					<div class="col-xs-12 col-sm-6">
						<input :disabled="access_update_account !== 0"  type="text" class="form-control"  name="StmPaypal[client_secret]" value="<?php echo esc_attr($data['client_secret'])?>">
					</div>
				</div>

				<div class="form-group">
					<label class="col-xs-12 col-sm-2 "><?php esc_html_e("Webhook ID", "ulisting")?></label>
					<div class="col-xs-12 col-sm-6">
						<input :disabled="access_update_account !== 0"  type="text" class="form-control"  name="StmPaypal[webhook_id]" value="<?php echo esc_attr($data['webhook_id'])?>">
					</div>
				</div>

				<div class="form-group">
					<label class="col-xs-12 col-sm-2 "><?php esc_html_e("Web hook url", "ulisting")?></label>
					<div class="col-xs-12 col-sm-6">
						<?php echo \uListing\Lib\PayPal\Classes\WebHook::getWebHookUrl()?>
					</div>
				</div>

				<div class="form-group">
					<label class="col-xs-12 col-sm-2 "><?php esc_html_e("Mode", "ulisting")?></label>
					<div class="col-xs-12 col-sm-6">
						<select :disabled="access_update_account !== 0"  name="StmPaypal[mode]" class="form-control" v-model="mode_selected">
							<option v-for="(mode, key) in modes" v-bind:value="key">{{mode}}</option>
						</select>
					</div>
				</div>
				<button v-if="access_update_account !== 0" class="button-primary"><?php esc_html_e("Save", "ulisting")?></button>
			</form>
		</div>

		<hr>

		<?php if( ( isset($data['client_id']) AND !empty($data['client_id']) ) AND  ( isset($data['client_id']) AND !empty($data['client_id']) ) ):?>
			<h3 v-if="pricing_plans.length"><?php esc_html_e("Synchronization to PayPal", "ulisting")?></h3>
			<div v-if="pricing_plans.length" class="ulisting-main form-horizontal p-t-30 p-b-30 p-l-30 p-r-30 ulisting-bg-white">
				<div class="p-t-15 p-b-15">
					<ul class="list-unstyled">
						<li v-for="plan in pricing_plans">
							<h4>
								{{plan.name}} <span class="label label-info" v-if="plan.paypal_plan_id">PayPal ID ( {{plan.paypal_plan_id}} )</span>
								<i v-if="plan.check" class="fa fa-check-circle"></i>
								<div v-if="plan.id == synchronization_item" class="stm-spinner" style="height: 10px"> <div></div> <div></div> <div></div> <div></div> <div></div> </div>
							</h4>
							<div v-if="plan.message" class="alert alert-danger" role="alert">{{plan.message}}</div>
						</li>
					</ul>

					<hr>

					<button  type="button" v-if="!synchronization_load" @click="synchronization" class="button-primary">
						<?php esc_html_e("Run synchronization ", "ulisting")?>
					</button>
					<div v-if="synchronization_load" class="text-center">
						<p>
							<?php esc_html_e("Synchronization", "ulisting")?>
						</p>
						<div class="stm-spinner"> <div></div> <div></div> <div></div> <div></div> <div></div> </div>
					</div>
				</div>
			</div>
		<?php endif;?>
	</div>
<?php endif;?>

<?php
/*
use uListing\Lib\PayPal\Classes\StmPayPalPlan;
use uListing\Lib\PayPal\Classes\ResultPrinter;

$plan_list = StmPayPalPlan::getList();
foreach ($plan_list->plans as $plan) {

 	 echo "<pre>";
	 print_r($plan->getName());
	 echo "</pre>";

	 echo "<pre>";
	 print_r($plan);
	 echo "</pre>";
 }
*/
?>
