<?php

use uListing\Lib\Stripe\Classes\Stripe;
use uListing\Lib\PricingPlan\Classes\StmUserPlan;
use uListing\Lib\PricingPlan\Classes\StmPricingPlans;

$stripe              = new Stripe();
$balance             = null;
$stripe_account_data = array('success' => true);

if ( ( isset($data['publishable_key']) AND !empty($data['publishable_key']) ) AND  ( isset($data['secret_key']) AND !empty($data['secret_key']) ) ) {
	$stripe_account_data = Stripe::get_balance();
	$balance = $stripe_account_data['balance'];
}

$pricing_plans = StmPricingPlans::query()
                                ->where('post_type','stm_pricing_plans')
                                ->where('post_status','publish')
                                ->find();

$stm_stripe_settings_data = [];
$stm_stripe_settings_data['access_update_account'] = StmUserPlan::get_active_plans_count();

foreach ($pricing_plans as $plan) {
	$plan_data          = $plan->getData();
	$plan->plan_data    = $plan;
	$plan->post_content = '';
	if ($plan_data['payment_type'] == StmPricingPlans::PRICING_PLANS_PAYMENT_TYPE_SUBSCRIPTION AND $plan_data['status'] == StmPricingPlans::STATUS_ACTIVE) {
		$stm_stripe_settings_data['pricing_plans'][] = array(
			'id'             => $plan->ID,
			'name'           => $plan->post_title,
			'stripe_plan_id' => Stripe::get_stripe_plan_id($plan->ID),
		);
	}
}
if (!isset($data['publishable_key']))
	$data['publishable_key'] = "";

if (!isset($data['secret_key']))
	$data['secret_key'] = "";

if (!isset($data['whsec']))
	$data['whsec'] = "";

wp_add_inline_script('stm-stripe-settings', "var stm_stripe_settings_data = json_parse('". ulisting_convert_content(json_encode($stm_stripe_settings_data)) ."');", 'before');
?>
<?php if (!Stripe::checkCurrency()):?>
	<div class="panel-custom p-t-30 p-b-30">
		<div class="p-t-30 p-b-30 p-l-30 p-r-30 ulisting-bg-white">
			<div class="alert alert-danger" role="alert"><?php esc_html_e('Gateway disabled: Stripe does not support your store currency.', "ulisting")?></div>
		</div>
	</div>
<?php else:?>
	<div id="stm-stripe-settings"  class="panel-custom p-b-30">
		<h3><?php esc_html_e("Stripe Settings", "ulisting")?></h3>
		<div class="ulisting-main form-horizontal p-b-30 p-l-30 p-r-30 ulisting-bg-white">

			<?php if (!ulisting_is_https()):?>
				<div class="alert alert-danger">
					<?php esc_html_e('Subscriptio Stripe payment gateway requires full SSL support and enforcement during Checkout. Only test mode will work until this is solved.')?>
				</div>
			<?php endif;?>

			<?php if (!$stripe_account_data['success']):?>
				<div class="alert alert-danger">
					<?php echo esc_attr($stripe_account_data['message']) ?>
				</div>
			<?php endif;?>

			<div class="form-group">
				<label class="col-xs-12 col-sm-2 "><?php esc_html_e("Web hook url", "ulisting")?></label>
				<div class="col-xs-12 col-sm-6">
					<?php echo \uListing\Lib\Stripe\Classes\WebHook::getWebHookUrl()?>
				</div>
			</div>

			<form method="post">
				<div class="form-group">
					<label class="col-xs-12 col-sm-2 "><?php esc_html_e("Publishable key", "ulisting")?></label>
					<div class="col-xs-12 col-sm-6">
						<input :disabled="access_update_account !== 0" type="text" class="form-control" name="StmStripe[publishable_key]" value="<?php echo esc_attr($data['publishable_key'])?>">
					</div>
				</div>

				<div class="form-group">
					<label class="col-xs-12 col-sm-2 "><?php esc_html_e("Secret key", "ulisting")?></label>
					<div class="col-xs-12 col-sm-6">
						<input :disabled="access_update_account !== 0" type="text" class="form-control"  name="StmStripe[secret_key]" value="<?php echo esc_attr($data['secret_key'])?>">
					</div>
				</div>

				<div class="form-group">
					<label class="col-xs-12 col-sm-2 "><?php esc_html_e("Signing secret", "ulisting")?></label>
					<div class="col-xs-12 col-sm-6">
						<input :disabled="access_update_account !== 0" type="text" class="form-control" name="StmStripe[whsec]" value="<?php echo esc_attr($data['whsec'])?>">
					</div>
				</div>
                <?php if (empty($data['publishable_key']) || empty($data['secret_key'])):?>
                <div class="form-group">
                    <div class="alert alert-danger">
                        <?php echo __('Payment method will be available only after filling all fields', 'ulisting');?>
                    </div>
                <?php endif; ?>

				<button class="button-primary"><?php esc_html_e("Save", "ulisting")?></button>
			</form>
		</div>
		<hr>
		<?php if ( $balance ):?>
			<div v-if="pricing_plans.length" class="p-b-15 p-l-30 p-r-30 ulisting-bg-white">
				<div class="stm-row">
					<div class="stm-col-4">
						<h3>
							<?php esc_html_e('Your stripe account balance', "ulisting")?> :
						</h3>
					</div>
					<div class="stm-col-4">
						<h3 class="text-uppercase">
							<?php echo esc_html($balance->available[0]->amount) .' '.esc_html($balance->available[0]->currency) ?>
						</h3>
					</div>
				</div>
			</div>
		<?php endif;?>
		<?php if ( ulisting_subscription_active() ):?>
			<hr>
			<div v-if="pricing_plans.length" class="p-l-30 p-r-30 ulisting-bg-white">
				<div class="p-t-15 p-b-15">
					<h3 v-if="pricing_plans.length"><?php esc_html_e("Synchronization Stripe", "ulisting")?></h3>
					<hr>
					<ul class="list-unstyled">
						<li v-for="plan in pricing_plans">
							<h4>
								{{plan.name}} <span class="label label-info" v-if="plan.stripe_plan_id">Stripe ID ( {{plan.stripe_plan_id}} )</span>
								<i v-if="plan.check" class="fa fa-check-circle"></i>
								<div v-if="plan.id == synchronization_item" class="stm-spinner" style="height: 10px"> <div></div> <div></div> <div></div> <div></div> <div></div> </div>
							</h4>
							<div v-if="plan.message" class="alert alert-danger" role="alert">{{plan.message}}</div>
						</li>
					</ul>
					<hr>
					<button type="button" v-if="!synchronization_load" @click="synchronization" class="button-primary">
						<?php esc_html_e("Run synchronization ", "ulisting")?>
					</button>
					<div v-if="synchronization_load" class="text-center">
						<p>
							<?php esc_html_e("Synchronization", "ulisting")?>
						</p>
						<div class="stm-spinner"> <div></div> <div></div> <div></div> <div></div> <div></div> </div>
					</div>
				</div>
			</div>
		<?php endif;?>
	</div>
<?php endif;?>

